﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using NUnit.Framework;

namespace EFramework.Core.Tests.Contracts
{
    [TestFixture]
    public class EnsureFixture
    {
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardStringIfItIsNotNull()
        {
            var instance = default(string);

            Ensure.IsNotNull(() => instance);
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardObjectIfItIsNotNull()
        {
            var instance = default(object);

            Ensure.IsNotNull(() => instance);
        }

        [Test]
        public void CanPassObjectIfItIsNotNull()
        {
            var instance = new object();

            Ensure.IsNotNull(() => instance);
        }


        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardStringIfItIsNotNullWithNotNullNorEmptyStatement()
        {
            var instance = default(string);

            Ensure.IsNotNullNorEmpty(() => instance);
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardStringIfItIsNotEmptyWithNotNullNorEmptyStatement()
        {
            var instance = String.Empty;

            Ensure.IsNotNullNorEmpty(() => instance);
        }

        [Test]
        public void CanPassStringIfItIsNotNullNorEmpty()
        {
            var instance = "Abc";

            Ensure.IsNotNullNorEmpty(() => instance);
        }


        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardIfProvidedInstanceImplementsInterface()
        {
            var instance = new object();

            Ensure.Is<IComparable>(() => instance);
        }

        [Test]
        public void CanPassObjectIfItProvidesImplementedInterface()
        {
            var instance = "Abc";

            Ensure.Is<IComparable>(() => instance);
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardIfProvidedTypeImplementsInterface()
        {
            var type = typeof(object);

            Ensure.Is<IComparable>(() => type);
        }
    }
}
