﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Helpers;
using NUnit.Framework;

namespace EFramework.Core.Tests.Helpers
{
    [TestFixture]
    public class NumberHelperFixture
    {
        [Test]
        public void CanDetectIfShortNumberIsDecimalNumber()
        {
            var instance = "123";

            Assert.IsTrue(NumberHelper.IsNumberDec(instance), "Text should be clasified as a number.");
        }

        [Test]
        public void CanDetectIfLongNumberIsDecimalNumber()
        {
            var instance = "1234567890123456789012345678901234567890";

            Assert.IsTrue(NumberHelper.IsNumberDec(instance), "Text should be clasified as a number.");
        }

        [Test]
        public void CanDetectIfEmptyStringIsNotDecimalNumber()
        {
            var instance = String.Empty;

            Assert.IsFalse(NumberHelper.IsNumberDec(instance), "Text should not be clasified as a number.");
        }

        [Test]
        public void CanDetectIfNotNumberIsNotDecimalNumber()
        {
            var instance = "Abc";

            Assert.IsFalse(NumberHelper.IsNumberDec(instance), "Text should not be clasified as a number.");
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullParameterInIsDecimalNumberMethod()
        {
            NumberHelper.IsNumberDec(null);
        }


        [Test]
        public void CanDetectIfShortNumberIsHexNumber()
        {
            var instance = "0x12ab";

            Assert.IsTrue(NumberHelper.IsNumberHex(instance), "Text should be clasified as a number.");
        }

        [Test]
        public void CanDetectIfShortNumberIsHexNumberWithoutPrefix()
        {
            var instance = "12ab";

            Assert.IsTrue(NumberHelper.IsNumberHex(instance), "Text should be clasified as a number.");
        }
        
        [Test]
        public void CanDetectIfLongNumberIsHexNumber()
        {
            var instance = "0x12AB12AB88FE66AC00128844338811";

            Assert.IsTrue(NumberHelper.IsNumberHex(instance), "Text should be clasified as a number.");
        }

        [Test]
        public void CanDetectIfLongNumberIsHexNumberWithoutPrefix()
        {
            var instance = "12AB12AB88FE66AC00128844338811";

            Assert.IsTrue(NumberHelper.IsNumberHex(instance), "Text should be clasified as a number.");
        }
        
        [Test]
        public void CanDetectIfEmptyStringIsNotHexNumber()
        {
            var instance = String.Empty;

            Assert.IsFalse(NumberHelper.IsNumberHex(instance), "Text should not be clasified as a number.");
        }

        [Test]
        public void CanDetectIfNotNumberIsNotHexNumber()
        {
            var instance = "Abz";

            Assert.IsFalse(NumberHelper.IsNumberHex(instance), "Text should not be clasified as a number.");
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullParameterInIsHexNumberMethod()
        {
            NumberHelper.IsNumberDec(null);
        }
    }
}
