﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;

namespace EFramework.Core.Contracts
{
    public static class Ensure
    {
        public static void IsNotNull<T>(Expression<Func<T>> expression)
            where T : class
        {
            var memberExpression = expression.Body as MemberExpression;
            var compiledExpression = expression.Compile();

            if (memberExpression != null && compiledExpression != null)
            {
                var memberValue = compiledExpression();
                var memberName = memberExpression.Member.Name;

                if (memberValue == null)
                {
                    throw new ArgumentException(String.Format("The value of '{0}' cannot be null.", memberName), memberName);
                }
            }
        }

        public static void IsNotNullNorEmpty(Expression<Func<string>> expression)
        {
            var memberExpression = expression.Body as MemberExpression;
            var compiledExpression = expression.Compile();

            if (memberExpression != null && compiledExpression != null)
            {
                var memberValue = compiledExpression();
                var memberName = memberExpression.Member.Name;

                if (String.IsNullOrEmpty(memberValue) == true)
                {
                    throw new ArgumentException(String.Format("The value of '{0}' cannot be null or empty.", memberName), memberName);
                }
            }
        }

        public static void Is<TInterface>(Expression<Func<object>> expression)
        {
            var memberExpression = expression.Body as MemberExpression;
            var compiledExpression = expression.Compile();

            if (memberExpression != null && compiledExpression != null)
            {
                var memberValue = compiledExpression();
                var memberName = memberExpression.Member.Name;

                if (memberValue is Type == true && typeof(TInterface).IsAssignableFrom(memberValue as Type) == false)
                {
                    throw new ArgumentException(String.Format("The value of '{0}' should implement '{1}' interface.", memberName, typeof(TInterface).Name), memberName);
                }
                
                if (memberValue is Type == false && memberValue is TInterface == false)
                {
                    throw new ArgumentException(String.Format("The value of '{0}' should implement '{1}' interface.", memberName, typeof(TInterface).Name), memberName);
                }
            }
        }
    }
}
