﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;

namespace EFramework.Core.Events
{
    public class EventsManager : SynchronizableObject
    {
        private readonly object sender;
        private readonly Dictionary<string, EventsRepository> events;
        private readonly EventsTrigger eventsTrigger;


        public EventsManager(object sender, EventsTrigger trigger = null)
        {
            this.sender = sender;
            this.events = new Dictionary<string, EventsRepository>();
            this.eventsTrigger = trigger ?? EventsTrigger.Default;
        }


        public void Add(string eventName, Delegate handler)
        {
            Ensure.IsNotNullNorEmpty(() => eventName);
            Ensure.IsNotNull(() => handler);

            lock (this.Synchronizer)
            {
                if (this.events.ContainsKey(eventName) == true)
                {
                    this.events[eventName].Add(handler);
                }
                else
                {
                    this.events.Add(eventName, new EventsRepository());
                    this.events[eventName].Add(handler);
                }
            }
        }

        public void Remove(string eventName, Delegate handler)
        {
            Ensure.IsNotNullNorEmpty(() => eventName);
            Ensure.IsNotNull(() => handler);

            lock (this.Synchronizer)
            {
                if (this.events.ContainsKey(eventName) == true)
                {
                    this.events[eventName].Remove(handler);
                }
            }
        }

        public void Invoke<TEventArgs>(string eventName, TEventArgs args)
            where TEventArgs : EventArgs
        {
            Ensure.IsNotNullNorEmpty(() => eventName);
            Ensure.IsNotNull(() => args);

            lock (this.Synchronizer)
            {
                if (this.events.ContainsKey(eventName) == true)
                {
                    this.eventsTrigger.Invoke(this.events[eventName].Enumerate(), this.sender, args);
                }
            }
        }

        public void BeginInvoke<TEventArgs>(string eventName, TEventArgs args)
            where TEventArgs : EventArgs
        {
            Ensure.IsNotNullNorEmpty(() => eventName);
            Ensure.IsNotNull(() => args);

            lock (this.Synchronizer)
            {
                if (this.events.ContainsKey(eventName) == true)
                {
                    this.eventsTrigger.BeginInvoke(this.events[eventName].Enumerate(), this.sender, args);
                }
            }
        }
    }
}
