﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;

namespace EFramework.Core.Events
{
    public abstract class EventsTrigger
    {
        internal static readonly EventsTrigger Default = new DefaltEventsTrigger();


        public abstract void Invoke<TEventArgs>(IEnumerable<Delegate> handlers, object sender, TEventArgs args)
            where TEventArgs : EventArgs;

        public abstract void BeginInvoke<TEventArgs>(IEnumerable<Delegate> handlers, object sender, TEventArgs args)
            where TEventArgs : EventArgs;


        private sealed class DefaltEventsTrigger : EventsTrigger
        {
            public override void Invoke<TEventArgs>(IEnumerable<Delegate> handlers, object sender, TEventArgs args)
            {
                Ensure.IsNotNull(() => args);
                Ensure.IsNotNull(() => handlers);

                foreach (var handler in handlers)
                {
                    handler.DynamicInvoke(sender, args);
                }
            }

            public override void BeginInvoke<TEventArgs>(IEnumerable<Delegate> handlers, object sender, TEventArgs args)
            {
                Ensure.IsNotNull(() => args);
                Ensure.IsNotNull(() => handlers);

                foreach (var handler in handlers)
                {
                    var methodInfo = handler.GetType().GetMethod("BeginInvoke");

                    foreach (var invocation in handler.GetInvocationList())
                    {
                        methodInfo.Invoke(invocation, new object[] { sender, args, null, null });
                    }
                }
            }
        }
    }
}
