﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace EFramework.Core
{
    public abstract class SynchronizableObject
    {
        private readonly object synchronizer;


        public SynchronizableObject()
        {
            this.synchronizer = new object();
        }


        protected object Synchronizer
        {
            get { return this.synchronizer; }
        }

        public IDisposable Synchronize()
        {
            return new SynchronizerScope(this);
        }


        public class SynchronizerScope : IDisposable
        {
            private readonly SynchronizableObject target;

            public SynchronizerScope(SynchronizableObject target)
            {
                this.target = target;
                Monitor.Enter(target);
            }


            public void Dispose()
            {
                Monitor.Exit(this.target);
            }
        }
    }
}
