﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Model.Tracking;
using NUnit.Framework;

namespace EFramework.Model.Tests
{
    [TestFixture]
    public class ContainerFixture
    {
        #region Constructor Tests

        [Test]
        public void CanCreateNewInstance()
        {
            var id = Guid.NewGuid();
            var instance = new Container(id);

            Assert.AreEqual(id, instance.Identifier, "Instance should properly set its identifier.");
            Assert.NotNull(instance.Entries, "Instance should properly set its collection.");
            CollectionAssert.IsEmpty(instance.Entries, "Instance should have empty its collection.");
        }

        #endregion    

        #region Add and Remove Tests

        [Test]
        public void CanAddEntity()
        {
            var instance = new Container(Guid.NewGuid());
            var entity = new EntityA(Guid.NewGuid());

            instance.Add(entity);

            CollectionAssert.IsNotEmpty(instance.Entries, "Instance should not have empty collection.");
            CollectionAssert.Contains(instance.GetAll(), entity, "Instance should contain added entity.");
        }

        [Test]
        public void CanRemoveEntity()
        {
            var instance = new Container(Guid.NewGuid());
            var entity = new EntityA(Guid.NewGuid());

            instance.Add(entity);
            instance.Remove(entity);

            CollectionAssert.IsEmpty(instance.Entries, "Instance should have empty collection.");
            CollectionAssert.DoesNotContain(instance.GetAll(), entity, "Instance should not contain removed entity.");
        }

        #endregion


        internal class EntityA : Entity
        {
            public EntityA(Guid id)
                : base(Identifier.From<EntityA>(id))
            {
            }
        }

        internal class EntityB : Entity
        {
            public EntityB(Guid id)
                : base(Identifier.From<EntityB>(id))
            {
            }
        }
    }
}
