﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;

namespace EFramework.Model.Tests
{
    [TestFixture]
    public class IdentifierFixture
    {
        [Test]
        public void CanCreateNewTypedIdentifier()
        {
            var id = Guid.NewGuid();
            var instance = Identifier.From<EntityA>(id);

            Assert.NotNull(instance, "Created instance should be not null.");
            Assert.AreEqual(id, instance.Id, "Created instance should have assigned id.");
        }

        [Test]
        public void CanCreateNewFreeIdentifier()
        {
            var id = Guid.NewGuid();
            var instance = Identifier.From(typeof(EntityA), id);

            Assert.NotNull(instance, "Created instance should be not null.");
            Assert.AreEqual(id, instance.Id, "Created instance should have assigned id.");
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullTypeInCreatingFreeIdentifier()
        {
            Identifier.From(null, Guid.NewGuid());
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardTypeNotInheritingFromEntityWhenCreatingFreeIdentifier()
        {
            Identifier.From(typeof(object), Guid.NewGuid());
        }

        [Test]
        public void CanCompareTwoIdentifiersPositive()
        {
            var id = Guid.NewGuid();
            var instance1 = Identifier.From<Entity>(id);
            var instance2 = Identifier.From<Entity>(id);

            Assert.AreEqual(instance1, instance2, "Two instances should be equal.");
            Assert.AreEqual(instance1.GetHashCode(), instance2.GetHashCode(), "Two instance should provide the same HashCode.");
        }

        [Test]
        public void CanCheckEntityType()
        {
            var instance = Identifier.From<EntityA>(Guid.NewGuid());

            Assert.IsTrue(instance.Is<EntityA>());
            Assert.IsFalse(instance.Is<Entity>());
        }


        internal class EntityA : Entity
        {
            public EntityA(Guid id)
                : base(Identifier.From<EntityA>(id))
            {
            }
        }
    }
}
