﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Events;
using EFramework.Model.Tracking;

namespace EFramework.Model
{
    public class Container : ContainerBase
    {
        private readonly Guid id;
        private readonly Tracker tracker;
        private readonly EventsManager eventsManager;

        public Container(Guid id)
        {
            this.id = id;
            this.tracker = new Tracker(this, State.Unchanged);
            this.tracker.Changed += this.OnChanged;
            this.eventsManager = new EventsManager(this);
        }


        public Guid Identifier
        {
            get { return this.id; }
        }


        private const string ChangedEvent = "Changed";
        public event EventHandler<TrackerEventArgs> Changed
        {
            add { this.eventsManager.Add(Container.ChangedEvent, value); }
            remove { this.eventsManager.Remove(Container.ChangedEvent, value); }
        }

        private void OnChanged(object sender, TrackerEventArgs e)
        {
            var entity = e.Owner as Entity;

            if (entity != null)
                this.eventsManager.Invoke(Container.ChangedEvent, e);
        }


        public void Add(params Entity[] entities)
        {
            this.Add(entities.AsEnumerable());
        }

        internal void Add(IEnumerable<Entity> entities)
        {
            foreach (var entity in entities)
            {
                this.Entries.Add(entity.Identifier, entity);
                this.tracker.Add(entity.Tracker);
            }
        }


        public void Remove<TEntity>(params Guid[] ids)
            where TEntity : Entity
        {
            var query = from id in ids
                        join entry in this.Entries on new { Id = id, Owner = typeof(TEntity) } equals new { entry.Key.Id, entry.Key.Owner }
                        select entry.Value;

            this.Remove(query.ToArray());
        }

        public void Remove(params Entity[] entities)
        {
            foreach (var entity in entities)
            {
                this.Entries.Remove(entity.Identifier);
                this.tracker.Remove(entity.Tracker);
            }
        }


        public void Clear()
        {
            this.Entries.Clear();
            this.tracker.Clear();
        }
    }
}
