﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace EFramework.Model
{
    public static class ContainerExtensions
    {
        public static TEntity Get<TEntity>(this ContainerBase container, Guid id)
            where TEntity : Entity
        {
            using (container.Synchronize())
            {
                return (TEntity)container.Entries[Model.Identifier.From<TEntity>(id)];
            }
        }

        public static Entity Get(this ContainerBase container, Identifier identifier)
        {
            using (container.Synchronize())
            {
                return container.Entries[identifier];
            }
        }

        public static TEntity Get<TEntity>(this ContainerBase container, Identifier identifier)
            where TEntity : Entity
        {
            using (container.Synchronize())
            {
                return (TEntity)container.Entries[identifier];
            }
        }

        public static ICollection<TEntity> Get<TEntity>(this ContainerBase container, ICollection<Guid> ids)
            where TEntity : Entity
        {
            using (container.Synchronize())
            {
                var query = from entry in container.Entries
                            join id in ids on new { entry.Key.Id, entry.Key.Owner } equals new { Id = id, Owner = typeof(TEntity) }
                            select (TEntity)entry.Value;

                return query.ToArray();
            }
        }

        public static ICollection<TEntity> Get<TEntity>(this ContainerBase container, ICollection<Identifier> identifiers)
            where TEntity : Entity
        {
            using (container.Synchronize())
            {
                var query = from entry in container.Entries
                            join identifier in identifiers on entry.Key equals identifier
                            select (TEntity)entry.Value;

                return query.ToArray();
            }
        }

        public static ICollection<Entity> GetAll(this ContainerBase container)
        {
            using (container.Synchronize())
            {
                return container.Entries.Values.ToArray();
            }
        }

        public static ICollection<TEntity> GetAll<TEntity>(this ContainerBase container)
            where TEntity : Entity
        {
            using (container.Synchronize())
            {
                return container.Entries.Values.OfType<TEntity>().ToArray();
            }
        }


        public static Package<TEntity> PackTo<TEntity>(this ContainerBase container, Identifier leader = null)
            where TEntity : Entity
        {
            using (container.Synchronize())
            {
                var result = new Package<TEntity>(leader);

                result.Add(container.GetAll());

                return result;
            }
        }


        public static bool Contains(this ContainerBase container, Identifier identifier)
        {
            using (container.Synchronize())
            {
                return container.Entries.ContainsKey(identifier);
            }
        }
    }
}
