﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;

namespace EFramework.Model
{
    public class Identifier
    {
        private readonly Guid id;
        private readonly Type owner;


        private Identifier(Type owner, Guid id)
        {
            Ensure.IsNotNull(() => owner);

            this.owner = owner;
            this.id = id;
        }


        internal Type Owner
        {
            get { return this.owner; }
        }

        public Guid Id
        {
            get { return this.id; }
        }


        public bool Is<T>()
            where T : Entity
        {
            return this.Owner == typeof(T);
        }


        public static Identifier From<T>(Guid id)
            where T : Entity
        {
            return new Identifier(typeof(T), id);
        }

        internal static Identifier From(Type type, Guid id)
        {
            Ensure.IsNotNull(() => type);
            Ensure.Is<Entity>(() => type);

            return new Identifier(type, id);
        }


        public override int GetHashCode()
        {
            return this.id.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            var other = obj as Identifier;

            return other != null
                && other.id == this.id
                && other.owner == this.owner;
        }
    }
}
