﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace EFramework.Model.Merging
{
    public static class MergeExtensions
    {
        public static void Merge<TEntity>(this Container container, Package<TEntity> package, MergeSettings settings)
            where TEntity : Entity
        {
            container.Merge<TEntity>(new[] { package }, settings);
        }

        public static void Merge<TEntity>(this Container container, IEnumerable<Package<TEntity>> packages, MergeSettings settings)
            where TEntity : Entity
        {
            foreach (var package in packages)
            {
                foreach (var entity in package.GetAll())
                {
                    var entityToTake = entity;

                    if (container.Contains(entityToTake.Identifier) == false)
                    {
                        if (settings.CloneEntities)
                            entityToTake = entityToTake.Clone();

                        container.Add(entityToTake);

                        if (settings.OnAdded != null)
                        {
                            settings.OnAdded(entityToTake);
                        }
                    }
                    else
                    {
                        var source = entityToTake;
                        var target = container.Get(entityToTake.Identifier);

                        var news = from sourceReference in source.GetAll()
                                   join targetReference in target.GetAll() on sourceReference equals targetReference into groups
                                   from element in groups.DefaultIfEmpty()
                                   where element == null
                                   select sourceReference;

                        target.Add(news.ToArray());
                    }
                }
            }
        }
    }
}
