﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Text;

namespace EFramework.Networking
{
    /// <summary>
    /// Contains methods related to modifying requests and responses.
    /// </summary>
    internal interface INetworkModifier
    {
        /// <summary>
        /// Modifies the <see cref="Uri"/> class.
        /// </summary>
        /// <param name="uri">The uri to modify.</param>
        /// <returns>The modified uri.</returns>
        Uri Modify(Uri uri);

        /// <summary>
        /// Modifies the <see cref="HttpWebRequest"/> class.
        /// </summary>
        /// <param name="request">The request to modify.</param>
        void Apply(HttpWebRequest request);

        /// <summary>
        /// Modifies the <see cref="HttpWebResponse"/> class.
        /// </summary>
        /// <param name="response">The response to modify.</param>
        void Apply(HttpWebResponse response);
    }
}
