﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;

namespace EFramework.Networking
{
    /// <summary>
    /// Represents the container with cookies to use.
    /// </summary>
    [Serializable]
    public class NetworkCookies : INetworkModifier
    {
        /// <summary>
        /// The internal cookie container.
        /// </summary>
        private readonly CookieContainer container;

        /// <summary>
        /// The internal cookie string.
        /// </summary>
        private readonly string data;


        /// <summary>
        /// Initializes an instance of the <see cref="NetworkCookies"/> class.
        /// </summary>
        public NetworkCookies()
        {
            this.container = new CookieContainer();
        }

        /// <summary>
        /// Initializes an instance of the <see cref="NetworkCookies"/> class.
        /// </summary>
        /// <param name="url">The URL which the cookie use for.</param>
        /// <param name="cookieString">The cookie string.</param>
        public NetworkCookies(string url, string cookieString)
        {
            this.container = new CookieContainer();
            this.data = cookieString;
        }


        /// <summary>
        /// Modifies the <see cref="Uri"/> class.
        /// </summary>
        /// <param name="uri">The uri to modify.</param>
        /// <returns>The modified uri.</returns>
        Uri INetworkModifier.Modify(Uri uri)
        {
            return uri;
        }

        /// <summary>
        /// Modifies the <see cref="HttpWebRequest"/> class applying cookies.
        /// </summary>
        /// <param name="request">The request to modify.</param>
        void INetworkModifier.Apply(HttpWebRequest request)
        {
            if (!String.IsNullOrWhiteSpace(this.data))
            {
                request.Headers.Add("Cookie", this.data);
            }
            else
            {
                request.CookieContainer = this.container;
            }
        }

        /// <summary>
        /// Modifies the <see cref="HttpWebResponse"/> class.
        /// </summary>
        /// <param name="response">The response to modify.</param>
        void INetworkModifier.Apply(HttpWebResponse response)
        {
        }


        public byte[] ToBytes()
        {
            using (var writeStream = new MemoryStream())
            {
                var binaryFormatter = new BinaryFormatter();
                binaryFormatter.Serialize(writeStream, this);

                return writeStream.ToArray();
            }
        }

        public static NetworkCookies FromBytes(byte[] bytes)
        {
            using (var readStream = new MemoryStream(bytes, false))
            {
                var binaryFormatter = new BinaryFormatter();
                var networkCookies = binaryFormatter.Deserialize(readStream) as NetworkCookies;

                return networkCookies;
            }
        }
    }
}
