﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking.Parser;

namespace EFramework.Networking
{
    internal class NetworkGatewayImpl : NetworkGateway
    {
        private NetworkService service;


        internal NetworkGatewayImpl(NetworkService service)
        {
            this.service = service;
        }


        public override HDocument GetPage(NetworkRequest request)
        {
            using (var stream = this.service.GetStream(request.Secure()))
            using (var reader = new StreamReader(stream))
            {
                return HDocumentFactory.Create(reader.ReadToEnd());
            }
        }

        public override string GetText(NetworkRequest request)
        {
            using (var stream = this.service.GetStream(request.Secure()))
            using (var reader = new StreamReader(stream))
            {
                return reader.ReadToEnd();
            }
        }

        public override byte[] GetImage(NetworkRequest request)
        {
            var results = new List<byte[]>();

            using (var stream = this.service.GetStream(request.Secure()))
            {
                while (true)
                {
                    var buffer = new byte[16 * 1024];
                    var read = stream.Read(buffer, 0, buffer.Length);

                    if (read <= 0)
                        break;

                    Array.Resize(ref buffer, read);
                    results.Add(buffer);
                }
            }

            if (!results.Any())
                return new byte[] { };

            var size = results.Sum(x => x.Length);
            var total = new byte[size];
            var position = 0;

            for (int i = 0; i < results.Count; i++)
            {
                Array.Copy(results[i], 0, total, position, results[i].Length);
                position += results[i].Length;
            }

            return total;
        }

        public override Stream GetStream(NetworkRequest request)
        {
            return this.service.GetStream(request.Secure());
        }
    }
}
