﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace EFramework.Networking
{
    public static class NetworkParser
    {
        public static bool HasContentDisposition(NetworkCallback callback)
        {
            return callback.Contains("Content-Disposition");
        }

        public static string GetContentDispositionAttachment(NetworkCallback callback)
        {
            var disposition = callback["Content-Disposition"];

            if (String.IsNullOrWhiteSpace(disposition) == false)
            {
                var parts = disposition.Split(new[] { ';' }, 2, StringSplitOptions.None);
                if (parts.Length > 1)
                {
                    var segments = parts[1].Split(new[] { '=' }, StringSplitOptions.None);

                    return segments[1];
                }
            }

            return String.Empty;
        }


        public static bool HasContentType(NetworkCallback callback)
        {
            return callback.Contains("Content-Type");
        }

        public static string GetContentType(NetworkCallback callback)
        {
            return callback["Content-Type"];
        }


        public static bool HasContentLength(NetworkCallback callback)
        {
            return callback.Contains("Content-Length");
        }

        public static long GetContentLength(NetworkCallback callback)
        {
            var contentLength = callback["Content-Length"];
            var size = Int64.Parse(contentLength);

            return size;
        }
    }
}
