﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace EFramework.Networking
{
    public class NetworkRequest
    {
        private readonly string url;
        private readonly Guid channel;
        private NetworkCookies cookies;
        private NetworkGetData getData;
        private NetworkPostData postData;
        private NetworkHeaders headers;
        private NetworkCallback callback;


        public NetworkRequest(string url, Guid channel)
        {
            this.url = url;
            this.channel = channel;
        }


        public string Url
        {
            get { return this.url; }
        }

        public Guid Channel
        {
            get { return this.channel; }
        }


        public NetworkCookies Cookies
        {
            get { return this.cookies; }
            set { this.cookies = value; }
        }

        public NetworkGetData GetData
        {
            get { return this.getData; }
            set { this.getData = value; }
        }

        public NetworkPostData PostData
        {
            get { return this.postData; }
            set { this.postData = value; }
        }

        public NetworkHeaders Headers
        {
            get { return this.headers; }
            set { this.headers = value; }
        }

        public NetworkCallback Callback
        {
            get { return this.callback; }
            set { this.callback = value; }
        }


        internal INetworkModifier[] GetModifiers()
        {
            return new INetworkModifier[] { this.cookies, this.getData, this.postData, this.headers, this.callback };
        }

        internal NetworkRequest Secure()
        {
            return new NetworkRequest(this.url, this.channel)
            {
                Cookies = this.cookies ?? new NetworkCookies(),
                GetData = this.getData ?? NetworkGetData.Null,
                PostData = this.postData ?? NetworkPostData.Null,
                Headers = this.headers ?? NetworkHeaders.Null,
                Callback = this.callback ?? NetworkCallback.Null,
            };
        }
    }
}
