﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace EFramework.Networking.Parser
{
    public abstract class HElement
    {
        protected HElement()
        {
        }

        public abstract HContainer Owner { get; }

        public IEnumerable<HContainer> Ancestors()
        {
            var node = this;

            while (node.Owner != null)
            {
                yield return node.Owner;
                node = node.Owner;
            }
        }

        public IEnumerable<HTag> Ancestors(string tag, Predicate<HAttribute> predicate)
        {
            var node = this;

            while (node.Owner != null && node.Owner is HTag)
            {
                var tagnode = node.Owner as HTag;

                if (tagnode.Tag == tag && tagnode.Attributes.Any(x => predicate(x)))
                    yield return tagnode;

                node = node.Owner;
            }
        }

        public IEnumerable<T> Nexts<T>() where T : HElement
        {
            if (this.Owner == null)
                yield break;

            var skipping = true;
            foreach (var item in this.Owner.Children.OfType<HElement>())
            {
                if (!skipping && item is T)
                    yield return item as T;

                if (Object.ReferenceEquals(item, this))
                    skipping = false;
            }
        }

        public IEnumerable<T> Prevs<T>() where T : HElement
        {
            return this.PrevsReversed<T>().Reverse();
        }

        private IEnumerable<T> PrevsReversed<T>() where T : HElement
        {
            if (this.Owner == null)
                yield break;

            foreach (var item in this.Owner.Children.OfType<HElement>())
            {
                if (Object.ReferenceEquals(item, this))
                    yield break;

                if (item is T)
                    yield return item as T;
            }
        }

        public abstract XNode ToXml();
    }
}
