﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace EFramework.Networking.Parser
{
    [DebuggerDisplay("Script: {content}")]
    public sealed class HScript : HElement
    {
        private readonly HAttributeCollection attributes;
        private readonly string content;
        private readonly HContainer owner;

        internal HScript(HContainer owner, string content)
        {
            this.owner = owner;
            this.content = content ?? String.Empty;
            this.attributes = new HAttributeCollection();
        }

        public override HContainer Owner
        {
            get { return this.owner; }
        }

        public string Content
        {
            get { return this.content; }
        }

        public HAttributeCollection Attributes
        {
            get { return this.attributes; }
        }

        public string[] GetLines()
        {
            return this.content.Split('\r', '\n');
        }

        public override XNode ToXml()
        {
            return new XElement("script",
                from attribute in this.Attributes.ToArray()
                where attribute.Name != "xmlns"
                select attribute.ToXml(),
                new XCData(this.content));
        }
    }
}
