﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Xml.Linq;

namespace EFramework.Networking.Parser
{
    [DebuggerDisplay("Element: {tag}; Children: {children.Count}; Attributes: {attributes.Count}")]
    public sealed class HTag : HContainer
    {
        private readonly HAttributeCollection attributes;
        private readonly string tag;
        private readonly HContainer owner;

        internal HTag(HContainer owner, string tag)
        {
            this.tag = tag;
            this.owner = owner;
            this.attributes = new HAttributeCollection();
        }

        public override HContainer Owner
        {
            get { return this.owner; }
        }

        public string Tag
        {
            get { return this.tag; }
        }

        public HAttributeCollection Attributes
        {
            get { return this.attributes; }
        }

        public override XNode ToXml()
        {
            return new XElement(this.Tag,
                from attribute in this.Attributes.ToArray()
                where attribute.Name != "xmlns"
                select attribute.ToXml(),
                from child in this.Children.ToArray()
                select child.ToXml());
        }
    }
}
