﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Threading;
using Microsoft.Practices.Prism.Interactivity.InteractionRequest;

namespace MDownloader.Core.Client.Behaviours.Actions
{
    public abstract class DispatcherTriggerAction : TriggerAction<DependencyObject>
    {
        private Dispatcher dispatcher;

        protected override void OnAttached()
        {
            base.OnAttached();
            this.dispatcher = this.AssociatedObject.Dispatcher;
        }

        protected override void OnDetaching()
        {
            this.dispatcher = null;
            base.OnDetaching();
        }

        protected sealed override void Invoke(object parameter)
        {
            var args = parameter as InteractionRequestedEventArgs;

            if (this.dispatcher == null || this.dispatcher.CheckAccess())
            {
                this.InvokeAction(args);
            }
            else
            {
                this.dispatcher.BeginInvoke(new Action<InteractionRequestedEventArgs>(this.InvokeAction), args);
            }
        }

        protected abstract void InvokeAction(InteractionRequestedEventArgs args);
    }
}
