﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Microsoft.Practices.Prism.Interactivity.InteractionRequest;

namespace MDownloader.Core.Client.Behaviours.Actions
{
    public class MessageAction : DispatcherTriggerAction
    {
        public static readonly DependencyProperty ButtonProperty =
            DependencyProperty.Register(
                "Button",
                typeof(MessageBoxButton),
                typeof(MessageAction),
                new PropertyMetadata(MessageBoxButton.OK));

        public static readonly DependencyProperty IconProperty =
            DependencyProperty.Register(
                "Icon",
                typeof(MessageBoxImage),
                typeof(MessageAction),
                new PropertyMetadata(MessageBoxImage.None));

        public static readonly DependencyProperty OptionsProperty =
            DependencyProperty.Register(
                "Options",
                typeof(MessageBoxOptions),
                typeof(MessageAction),
                new PropertyMetadata(MessageBoxOptions.None));

        public static readonly DependencyProperty DefaultResultProperty =
            DependencyProperty.Register(
                "DefaultResult",
                typeof(MessageBoxResult),
                typeof(MessageAction),
                new PropertyMetadata(MessageBoxResult.None));

        public static readonly DependencyProperty ContentProperty =
           DependencyProperty.Register(
               "Content",
               typeof(string),
               typeof(MessageAction),
               new PropertyMetadata(String.Empty));

        public static readonly DependencyProperty TitleProperty =
            DependencyProperty.Register(
                "Title",
                typeof(string),
                typeof(MessageAction),
                new PropertyMetadata(String.Empty));

        public MessageBoxButton Button
        {
            get { return (MessageBoxButton)this.GetValue(MessageAction.ButtonProperty); }
            set { this.SetValue(MessageAction.ButtonProperty, value); }
        }

        public MessageBoxImage Icon
        {
            get { return (MessageBoxImage)this.GetValue(MessageAction.IconProperty); }
            set { this.SetValue(MessageAction.IconProperty, value); }
        }

        public MessageBoxOptions Options
        {
            get { return (MessageBoxOptions)this.GetValue(MessageAction.OptionsProperty); }
            set { this.SetValue(MessageAction.OptionsProperty, value); }
        }

        public MessageBoxResult DefaultResult
        {
            get { return (MessageBoxResult)this.GetValue(MessageAction.DefaultResultProperty); }
            set { this.SetValue(MessageAction.DefaultResultProperty, value); }
        }

        public string Title
        {
            get { return (string)this.GetValue(MessageAction.TitleProperty); }
            set { this.SetValue(MessageAction.TitleProperty, value); }
        }

        public string Content
        {
            get { return (string)this.GetValue(MessageAction.ContentProperty); }
            set { this.SetValue(MessageAction.ContentProperty, value); }
        }

        protected override void InvokeAction(InteractionRequestedEventArgs args)
        {
            var result = MessageBox.Show(
                this.Content,
                this.Title,
                this.Button,
                this.Icon,
                this.DefaultResult,
                this.Options);

            var confirmation = args.Context as Confirmation;
            if (confirmation != null)
            {
                confirmation.Confirmed = result == MessageBoxResult.OK || result == MessageBoxResult.Yes;
            }

            args.Callback();
        }
    }
}
