﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Microsoft.Practices.Prism.Interactivity.InteractionRequest;
using MDownloader.Client.Notifications;
using MDownloader.Client.Interfaces;

namespace MDownloader.Core.Client.Behaviours.Actions
{
    public class ShowAction : DispatcherTriggerAction
    {
        public static readonly DependencyProperty ConnectorNameProperty =
            DependencyProperty.Register(
                "ConnectorName",
                typeof(string),
                typeof(ShowAction),
                new PropertyMetadata());

        public static readonly DependencyProperty StyleProperty =
            DependencyProperty.Register(
                "Style",
                typeof(Style),
                typeof(ShowAction),
                new PropertyMetadata(null));

        public static readonly DependencyProperty IsModalProperty =
            DependencyProperty.Register(
                "IsModal",
                typeof(bool),
                typeof(ShowAction),
                new PropertyMetadata(true));

        public static readonly DependencyProperty IsOwnedProperty =
            DependencyProperty.Register(
                "IsOwned",
                typeof(bool),
                typeof(ShowAction),
                new PropertyMetadata(true));

        public static readonly DependencyProperty ServiceProviderProperty =
            DependencyProperty.Register(
                "ServiceProvider",
                typeof(IServiceProvider),
                typeof(ShowAction),
                new PropertyMetadata());

        public string ConnectorName
        {
            get { return (string)this.GetValue(ShowAction.ConnectorNameProperty); }
            set { this.SetValue(ShowAction.ConnectorNameProperty, value); }
        }

        public Style Style
        {
            get { return (Style)this.GetValue(ShowAction.StyleProperty); }
            set { this.SetValue(ShowAction.StyleProperty, value); }
        }

        public bool IsModal
        {
            get { return (bool)this.GetValue(ShowAction.IsModalProperty); }
            set { this.SetValue(ShowAction.IsModalProperty, value); }
        }

        public bool IsOwned
        {
            get { return (bool)this.GetValue(ShowAction.IsOwnedProperty); }
            set { this.SetValue(ShowAction.IsOwnedProperty, value); }
        }

        public IServiceProvider ServiceProvider
        {
            get { return (IServiceProvider)this.GetValue(ShowAction.ServiceProviderProperty); }
            set { this.SetValue(ShowAction.ServiceProviderProperty, value); }
        }

        protected override void InvokeAction(InteractionRequestedEventArgs e)
        {
            var window = Activator.CreateInstance<Window>();
            var interaction = e.Context as InteractionBase;
            var connector = ConnectorFactory.Find(this.ServiceProvider, interaction.GetConnectorType(), this.ConnectorName);
            var callback = new Action(e.Callback);

            if (this.IsOwned)
            {
                window.Owner = Window.GetWindow(this.AssociatedObject);
            }

            if (connector is IControllerAware)
            {
                callback = () =>
                {
                    e.Callback();
                    (connector as IControllerAware).DisposeController();
                };
            }

            window.Style = this.Style;
            window.Content = connector;
            window.WindowStartupLocation = WindowStartupLocation.CenterOwner;

            interaction.AssignConnector(connector);

            if (this.IsModal)
            {
                bool result = window.ShowDialog() ?? false;

                var confirmation = e.Context as Confirmation;
                if (confirmation != null)
                {
                    confirmation.Confirmed = result;
                }

                callback();
            }
            else
            {
                window.Show();
                window.Closed += (sender, args) => callback();
            }
        }
    }
}
