﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;

namespace MDownloader.Client.Controllers
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    public class ControllerRegistry
    {
        [Import]
        private IServiceProvider serviceProvider;

        private readonly object synchronizer;
        private readonly Dictionary<Guid, ControllerBase> controllersById;
        private readonly Dictionary<Uri, ControllerBase> controllersByUri;


        public ControllerRegistry()
        {
            this.synchronizer = new object();
            this.controllersById = new Dictionary<Guid, ControllerBase>();
            this.controllersByUri = new Dictionary<Uri, ControllerBase>();
        }


        public TController GetController<TController>(Guid id)
            where TController : ControllerBase
        {
            lock (this.synchronizer)
            {
                return (TController)this.controllersById[id];
            }
        }

        public TController GetController<TController>(Uri url)
            where TController : ControllerBase
        {
            lock (this.synchronizer)
            {
                if (!this.controllersByUri.ContainsKey(url))
                {
                    var controller = (TController)this.serviceProvider.GetService(typeof(TController));

                    this.controllersByUri.Add(url, controller);
                    this.controllersById.Add(controller.Id, controller);
                }

                return (TController)this.controllersByUri[url];
            }
        }


        public void Release(Guid id)
        {
            lock (this.synchronizer)
            {
                var controller = this.controllersById[id];
                var url = this.controllersByUri.First(x => x.Value == controller).Key;

                this.controllersById.Remove(id);
                this.controllersByUri.Remove(url);
            }
        }

        public void Release(Uri url)
        {
            lock (this.synchronizer)
            {
                var controller = this.controllersByUri[url];
                var id = this.controllersById.First(x => x.Value == controller).Key;

                this.controllersById.Remove(id);
                this.controllersByUri.Remove(url);
            }
        }

        public void Release(ControllerBase controller)
        {
            lock (this.synchronizer)
            {
                var url = this.controllersByUri.First(x => x.Value == controller).Key;
                var id = this.controllersById.First(x => x.Value == controller).Key;

                this.controllersById.Remove(id);
                this.controllersByUri.Remove(url);
            }
        }
    }
}
