﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;

namespace MDownloader.Client
{
    public class DelegateCommand : ICommand
    {
        private readonly Action execute;
        private readonly Func<bool> canExecute;
        private EventHandler canExecuteChangedHandler;


        public DelegateCommand(Action execute, Func<bool> canExecute = null)
        {
            this.execute = execute;
            this.canExecute = canExecute;
        }


        event EventHandler ICommand.CanExecuteChanged
        {
            add { this.canExecuteChangedHandler += value; }
            remove { this.canExecuteChangedHandler -= value; }
        }

        
        bool ICommand.CanExecute(object parameter)
        {
            return this.canExecute == null || this.canExecute();
        }

        void ICommand.Execute(object parameter)
        {
            this.execute();
        }


        public void Update()
        {
            var handler = this.canExecuteChangedHandler;
            var args = EventArgs.Empty;

            if (handler != null)
                handler(this, args);
        }
    }

    public class DelegateCommand<TParameter> : ICommand
    {
        private readonly Action<TParameter> execute;
        private readonly Func<TParameter, bool> canExecute;
        private EventHandler canExecuteChangedHandler;


        public DelegateCommand(Action<TParameter> execute, Func<TParameter, bool> canExecute = null)
        {
            this.execute = execute;
            this.canExecute = canExecute;
        }


        event EventHandler ICommand.CanExecuteChanged
        {
            add { this.canExecuteChangedHandler += value; }
            remove { this.canExecuteChangedHandler -= value; }
        }


        bool ICommand.CanExecute(object parameter)
        {
            return this.canExecute == null || this.canExecute((TParameter)parameter);
        }

        void ICommand.Execute(object parameter)
        {
            this.execute((TParameter)parameter);
        }


        public void Update()
        {
            var handler = this.canExecuteChangedHandler;
            var args = EventArgs.Empty;

            if (handler != null)
                handler(this, args);
        }
    }
}
