﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using System.Text;

namespace MDownloader.Client
{
    public abstract class PropertyNotifier : INotifyPropertyChanged
    {
        private PropertyChangedEventHandler propertyChanged;

        event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged
        {
            add { this.propertyChanged += value; }
            remove { this.propertyChanged -= value; }
        }


        protected void RaisePropertyChanged(string propertyName)
        {
            var handler = this.propertyChanged;
            var args = new PropertyChangedEventArgs(propertyName);

            if (handler != null)
                handler(this, args);
        }

        protected void RaisePropertyChanged<T>(Expression<Func<T>> propertyExpression)
        {
            var propertyName = PropertySupport.ExtractPropertyName(propertyExpression);
            var handler = this.propertyChanged;
            var args = new PropertyChangedEventArgs(propertyName);

            if (handler != null)
                handler(this, args);
        }
    }
}
