﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using EFramework.Core.Events;
using MDownloader.Client.Controllers;
using MDownloader.Main.Client.Events;
using MDownloader.Main.Client.Services;
using MDownloader.Main.Entities;

namespace MDownloader.Main.Client.Controllers
{
    [Export]
    internal class CaptchasController : ControllerBase, IPartImportsSatisfiedNotification
    {
        private readonly CaptchasModel model;
        private readonly EventsManager eventsManager;

        [Import]
        private InfoServiceProxy infoServiceProxy;

        [Import]
        private CaptchaServiceProxy captchaServiceProxy;


        public CaptchasController()
        {
            this.model = new CaptchasModel();
            this.eventsManager = new EventsManager(this);
        }


        public CaptchasModel Model
        {
            get { return this.model; }
        }


        public void Proceed(Captcha captcha)
        {
            this.captchaServiceProxy.Proceed(captcha.Identifier.Id, captcha.Text);
        }


        private const string CaptchaAddedEvent = "CaptchaAdded";
        public event EventHandler<CaptchaEventArgs> CaptchaAdded
        {
            add { this.eventsManager.Add(CaptchasController.CaptchaAddedEvent, value); }
            remove { this.eventsManager.Remove(CaptchasController.CaptchaAddedEvent, value); }
        }

        private void OnCaptchaAdded(CaptchaEventArgs args)
        {
            this.eventsManager.Invoke(CaptchasController.CaptchaAddedEvent, args);
        }

        private const string CaptchaRemovedEvent = "CaptchaRemoved";
        public event EventHandler<CaptchaEventArgs> CaptchaRemoved
        {
            add { this.eventsManager.Add(CaptchasController.CaptchaRemovedEvent, value); }
            remove { this.eventsManager.Remove(CaptchasController.CaptchaRemovedEvent, value); }
        }

        private void OnCaptchaRemoved(CaptchaEventArgs args)
        {
            this.eventsManager.Invoke(CaptchasController.CaptchaRemovedEvent, args);
        }

        private void OnCaptchaAdded(object sender, CaptchaEventArgs e)
        {
            this.model.AddCaptcha(e.Target.Ref.Id, e.Target.Properties);
            this.OnCaptchaAdded(e);
        }

        private void OnCaptchaRemoved(object sender, CaptchaEventArgs e)
        {
            this.model.RemoveCaptcha(e.Target.Ref.Id);
            this.OnCaptchaRemoved(e);
        }

        
        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.infoServiceProxy.CaptchaAdded += this.OnCaptchaAdded;
            this.infoServiceProxy.CaptchaRemoved += this.OnCaptchaRemoved;
        }
    }
}
