﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using MDownloader.Client.Controllers;
using MDownloader.Main.Client.Events;
using MDownloader.Main.Client.Models.Interfaces;
using MDownloader.Main.Client.Operations.Interfaces;
using MDownloader.Main.Client.Services;

namespace MDownloader.Main.Client.Controllers
{
    [Export]
    internal class QueueController : ControllerBase, IPartImportsSatisfiedNotification, IResourceExecutionController, IResourceLifetimeController
    {
        [Import]
        private InfoServiceProxy infoServiceProxy;

        [Import]
        private TransferServiceProxy transferServiceProxy;

        [Import]
        private ManageServiceProxy manageServiceProxy;


        public QueueModel Model { get; private set; }


        public void Initialize(Action onComplete)
        {
            this.Execute(this.infoServiceProxy.Initialize, onComplete);
        }


        public void Start(ICollection<IResource> resources)
        {
            var query = from resource in resources
                        select resource.Id;

            this.Execute(() => this.transferServiceProxy.Start(query.ToArray()));
        }

        public void Remove(ICollection<IResource> resources)
        {
            var query = from resource in resources
                        select resource.Id;

            this.Execute(() => this.manageServiceProxy.Remove(query.ToArray()));
        }


        public event EventHandler<LinkEventArgs> LinkAdded;
        private void OnLinksAdded(LinkEventArgs args)
        {
            var handler = this.LinkAdded;

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<LinkEventArgs> LinkChanged;
        private void OnLinksChanged(LinkEventArgs args)
        {
            var handler = this.LinkChanged;

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<LinkEventArgs> LinksRemoved;
        private void OnLinksRemoved(LinkEventArgs args)
        {
            var handler = this.LinksRemoved;

            if (handler != null)
                handler(this, args);
        }
     
        
        private void OnLinksAdded(object sender, LinkEventArgs e)
        {
            this.Model.AddLink(e.Target.Ref.Id, e.Target.Properties);
            this.OnLinksAdded(e);
        }

        private void OnLinksChanged(object sender, LinkEventArgs e)
        {
            this.Model.ChangeLink(e.Target.Ref.Id, e.Target.Properties);
            this.OnLinksChanged(e);
        }

        private void OnLinksRemoved(object sender, LinkEventArgs e)
        {
            this.Model.RemoveLink(e.Target.Ref.Id);
            this.OnLinksRemoved(e);
        }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.Model = new QueueModel();

            this.infoServiceProxy.LinkAdded += this.OnLinksAdded;
            this.infoServiceProxy.LinkChanged += this.OnLinksChanged;
            this.infoServiceProxy.LinkRemoved += this.OnLinksRemoved;
        }
    }
}
