﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using EFramework.Model;
using MDownloader.Main.Entities;

namespace MDownloader.Main.Client.Controllers
{
    internal class QueueModel
    {
        private readonly Container container = new Container(Guid.NewGuid());


        public Link GetLink(Guid id)
        {
            return this.container.Get<Link>(id);
        }

        public IEnumerable<Link> GetLinks()
        {
            return this.container.GetAll<Link>();
        }


        public void AddLink(Guid linkId, IDictionary<string, string> properties)
        {
            var link = new Link(linkId)
            {
                Name = properties[LinkProperties.Name],
                Description = properties[LinkProperties.Description],
                Provider = properties[LinkProperties.Provider],
                Size = Int64.Parse(properties[LinkProperties.Size], CultureInfo.InvariantCulture),
                SizeExactness = Boolean.Parse(properties[LinkProperties.SizeExactness]),
                Flags = (Link.LinkFlags)Enum.Parse(typeof(Link.LinkFlags), properties[LinkProperties.Flags]),
                Status = (Link.LinkStatus)Enum.Parse(typeof(Link.LinkStatus), properties[LinkProperties.Status]),
            };

            this.container.Add(link);
        }

        public void ChangeLink(Guid linkId, IDictionary<string, string> properties)
        {
            var link = this.container.Get<Link>(linkId);

            if (properties.ContainsKey(LinkProperties.Name))
                link.Name = properties[LinkProperties.Name];

            if (properties.ContainsKey(LinkProperties.Description))
                link.Description = properties[LinkProperties.Description];

            if (properties.ContainsKey(LinkProperties.Provider))
                link.Provider = properties[LinkProperties.Provider];

            if (properties.ContainsKey(LinkProperties.Size))
                link.Size = Int64.Parse(properties[LinkProperties.Size], CultureInfo.InvariantCulture);

            if (properties.ContainsKey(LinkProperties.SizeExactness))
                link.SizeExactness = Boolean.Parse(properties[LinkProperties.SizeExactness]);

            if (properties.ContainsKey(LinkProperties.Flags))
                link.Flags = (Link.LinkFlags)Enum.Parse(typeof(Link.LinkFlags), properties[LinkProperties.Flags]);

            if (properties.ContainsKey(LinkProperties.Status))
                link.Status = (Link.LinkStatus)Enum.Parse(typeof(Link.LinkStatus), properties[LinkProperties.Status]);
        }

        public void RemoveLink(Guid linkId)
        {
            this.container.Remove<Link>(linkId);
        }
    }
}
