﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.Composition;
using MDownloader.Client.Controllers;
using MDownloader.Main.Client.Services;
using MDownloader.Main.Client.Events;

namespace MDownloader.Main.Client.Controllers
{
    [Export]
    internal class TasksController : ControllerBase, IPartImportsSatisfiedNotification
    {
        [Import]
        private InfoServiceProxy infoServiceProxy;

        [Import]
        private TransferServiceProxy transferServiceProxy;


        public TasksModel Model { get; private set; }


        public event EventHandler<TaskEventArgs> TaskAdded;
        private void OnTasksAdded(TaskEventArgs task)
        {
            var handler = this.TaskAdded;
            var args = task;

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<TaskEventArgs> TaskChanged;
        private void OnTasksChanged(TaskEventArgs task)
        {
            var handler = this.TaskChanged;
            var args = task;

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<TaskEventArgs> TasksRemoved;
        private void OnTasksRemoved(TaskEventArgs task)
        {
            var handler = this.TasksRemoved;
            var args = task;

            if (handler != null)
                handler(this, args);
        }


        private void OnTasksAdded(object sender, TaskEventArgs e)
        {
            this.Model.AddTask(e.Target.Ref.Id, e.Target.Properties);
            this.OnTasksAdded(e);
        }

        private void OnTasksChanged(object sender, TaskEventArgs e)
        {
            this.Model.ChangeTask(e.Target.Ref.Id, e.Target.Properties);
            this.OnTasksChanged(e);
        }

        private void OnTasksRemoved(object sender, TaskEventArgs e)
        {
            this.Model.RemoveTask(e.Target.Ref.Id);
            this.OnTasksRemoved(e);
        }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.Model = new TasksModel();

            this.infoServiceProxy.TaskAdded += this.OnTasksAdded;
            this.infoServiceProxy.TaskChanged += this.OnTasksChanged;
            this.infoServiceProxy.TaskRemoved += this.OnTasksRemoved;
        }
    }
}
