﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using EFramework.Model;
using MDownloader.Main.Entities;

namespace MDownloader.Main.Client.Controllers
{
    internal class TasksModel
    {
        private readonly Container container = new Container(Guid.NewGuid());


        public Task GetTask(Guid id)
        {
            return this.container.Get<Task>(id);
        }

        public IEnumerable<Task> GetTasks()
        {
            return this.container.GetAll<Task>();
        }


        public void AddTask(Guid taskId, IDictionary<string, string> properties)
        {
            var link = new Task(taskId)
            {
                Name = properties[TaskProperties.Name],
                Provider = properties[TaskProperties.Provider],
                Progress = Int64.Parse(properties[TaskProperties.Progress], CultureInfo.InvariantCulture),
                Size = Int64.Parse(properties[TaskProperties.Size], CultureInfo.InvariantCulture),
                Speed = Int64.Parse(properties[TaskProperties.Speed], CultureInfo.InvariantCulture),
                Status = (Task.TaskStatus)Enum.Parse(typeof(Task.TaskStatus), properties[TaskProperties.Status]),
                Counter = Int32.Parse(properties[TaskProperties.Counter], CultureInfo.InvariantCulture),
            };

            this.container.Add(link);
        }

        public void ChangeTask(Guid taskId, IDictionary<string, string> properties)
        {
            var task = this.container.Get<Task>(taskId);

            if (properties.ContainsKey(TaskProperties.Name))
                task.Name = properties[TaskProperties.Name];

            if (properties.ContainsKey(TaskProperties.Provider))
                task.Provider = properties[TaskProperties.Provider];

            if (properties.ContainsKey(TaskProperties.Progress))
                task.Progress = Int64.Parse(properties[TaskProperties.Progress], CultureInfo.InvariantCulture);

            if (properties.ContainsKey(TaskProperties.Size))
                task.Size = Int64.Parse(properties[TaskProperties.Size], CultureInfo.InvariantCulture);

            if (properties.ContainsKey(TaskProperties.Speed))
                task.Speed = Int64.Parse(properties[TaskProperties.Speed], CultureInfo.InvariantCulture);

            if (properties.ContainsKey(TaskProperties.Status))
                task.Status = (Task.TaskStatus)Enum.Parse(typeof(Task.TaskStatus), properties[TaskProperties.Status]);

            if (properties.ContainsKey(TaskProperties.Counter))
                task.Counter = Int32.Parse(properties[TaskProperties.Counter], CultureInfo.InvariantCulture);
        }

        public void RemoveTask(Guid taskId)
        {
            this.container.Remove<Task>(taskId);
        }
    }
}
