﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MDownloader.Client;
using MDownloader.Client.Operations;
using MDownloader.Main.Client.Models.Interfaces;

namespace MDownloader.Main.Client.Models
{
    internal abstract class ResourceBase : PropertyNotifier, IResource, IOperationTarget
    {
        public abstract Guid Id { get; }

        public abstract string Name { get; }
        public abstract string Description { get; }
        public abstract string Provider { get; }
        public abstract string State { get; }
        public abstract string Size { get; }

        private bool isSelected;
        public bool IsSelected
        {
            get { return this.isSelected; }
            set
            {
                if (this.isSelected != value)
                {
                    this.isSelected = value;
                    this.RaisePropertyChanged(() => this.IsSelected);
                }
            }
        }


        public void Update()
        {
            this.RaisePropertyChanged(() => this.State);
        }


        public override int GetHashCode()
        {
            return this.Id.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            var other = obj as IResource;

            return other != null
                && other.Id == this.Id;
        }


        object IOperationTarget.Instance
        {
            get { return this; }
        }

        Type IOperationTarget.Type
        {
            get { return typeof(IResource); }
        }
    }
}
