﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.ServiceModel;
using System.Text;
using EFramework.Core.Events;
using EFramework.Model;
using MDownloader.Main.Client.Events;
using MDownloader.Main.Client.InfoServiceReference;
using MDownloader.Main.Entities.Transfer;

namespace MDownloader.Main.Client.Services
{
    [Export]
    [PartCreationPolicy(CreationPolicy.Shared)]
    [CallbackBehavior(UseSynchronizationContext = false, ConcurrencyMode = ConcurrencyMode.Multiple)]
    internal class InfoServiceProxy : IDownloaderInfoServiceCallback, IDisposable
    {
        private DownloaderInfoServiceClient client;
        private readonly EventsManager eventsManager;


        public InfoServiceProxy()
        {
            this.eventsManager = new EventsManager(this);
        }


        public void Initialize()
        {
            this.GetClient();
        }


        private IDownloaderInfoService GetClient()
        {
            if (this.client == null)
                this.client = new DownloaderInfoServiceClient(new InstanceContext(this), new NetTcpBinding(), new EndpointAddress("net.tcp://localhost:4712/MDownloader/Main/Info"));

            client.Subscribe();
            return this.client;
        }


        private const string LinkAddedEvent = "LinkAdded";
        public event EventHandler<LinkEventArgs> LinkAdded
        {
            add { this.eventsManager.Add(InfoServiceProxy.LinkAddedEvent, value); }
            remove { this.eventsManager.Remove(InfoServiceProxy.LinkAddedEvent, value); }
        }

        private void OnLinkAdded(LinkChange linkChange)
        {
            this.eventsManager.Invoke(InfoServiceProxy.LinkAddedEvent, new LinkEventArgs(linkChange));
        }

        public event EventHandler<LinkEventArgs> LinkChanged;
        private void OnLinkChanged(LinkChange linkChange)
        {
            var handler = this.LinkChanged;
            var args = new LinkEventArgs(linkChange);

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<LinkEventArgs> LinkRemoved;
        private void OnLinkRemoved(LinkChange linkChange)
        {
            var handler = this.LinkRemoved;
            var args = new LinkEventArgs(linkChange);

            if (handler != null)
                handler(this, args);
        }


        public event EventHandler<TaskEventArgs> TaskAdded;
        private void OnTaskAdded(TaskChange taskChange)
        {
            var handler = this.TaskAdded;
            var args = new TaskEventArgs(taskChange);

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<TaskEventArgs> TaskChanged;
        private void OnTaskChanged(TaskChange taskChange)
        {
            var handler = this.TaskChanged;
            var args = new TaskEventArgs(taskChange);

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<TaskEventArgs> TaskRemoved;
        private void OnTaskRemoved(TaskChange taskChange)
        {
            var handler = this.TaskRemoved;
            var args = new TaskEventArgs(taskChange);

            if (handler != null)
                handler(this, args);
        }


        private const string CaptchaAddedEvent = "CaptchaAdded";
        public event EventHandler<CaptchaEventArgs> CaptchaAdded
        {
            add { this.eventsManager.Add(InfoServiceProxy.CaptchaAddedEvent, value); }
            remove { this.eventsManager.Remove(InfoServiceProxy.CaptchaAddedEvent, value); }
        }

        private void OnCaptchaAdded(CaptchaChange captchaChange)
        {
            this.eventsManager.Invoke(InfoServiceProxy.CaptchaAddedEvent, new CaptchaEventArgs(captchaChange));
        }

        private const string CaptchaRemovedEvent = "CaptchaRemoved";
        public event EventHandler<CaptchaEventArgs> CaptchaRemoved
        {
            add { this.eventsManager.Add(InfoServiceProxy.CaptchaRemovedEvent, value); }
            remove { this.eventsManager.Remove(InfoServiceProxy.CaptchaRemovedEvent, value); }
        }

        private void OnCaptchaRemoved(CaptchaChange captchaChange)
        {
            this.eventsManager.Invoke(InfoServiceProxy.CaptchaRemovedEvent, new CaptchaEventArgs(captchaChange));
        }


        void IDownloaderInfoServiceCallback.OnLinkChanged(LinkChange[] linkChanges)
        {
            foreach (var linkChange in linkChanges)
            {
                switch (linkChange.Change)
                {
                    case ChangeType.Added: this.OnLinkAdded(linkChange); break;
                    case ChangeType.Changed: this.OnLinkChanged(linkChange); break;
                    case ChangeType.Removed: this.OnLinkRemoved(linkChange); break;
                }
            }
        }

        void IDownloaderInfoServiceCallback.OnFileChanged(FileChange[] fileChanges)
        {
        }

        void IDownloaderInfoServiceCallback.OnPackageChanged(PackageChange[] packageChanges)
        {
        }

        void IDownloaderInfoServiceCallback.OnTaskChanged(TaskChange[] taskChanges)
        {
            foreach (var taskChange in taskChanges)
            {
                switch (taskChange.Change)
                {
                    case ChangeType.Added: this.OnTaskAdded(taskChange); break;
                    case ChangeType.Changed: this.OnTaskChanged(taskChange); break;
                    case ChangeType.Removed: this.OnTaskRemoved(taskChange); break;
                }
            }
        }

        void IDownloaderInfoServiceCallback.OnCaptchaChanged(CaptchaChange[] captchaChanges)
        {
            foreach (var captchaChange in captchaChanges)
            {
                switch (captchaChange.Change)
                {
                    case ChangeType.Added: this.OnCaptchaAdded(captchaChange); break;
                    case ChangeType.Removed: this.OnCaptchaRemoved(captchaChange); break;
                }
            }
        }


        void IDisposable.Dispose()
        {
        }
    }
}
