﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Windows.Input;
using MDownloader.Client;
using MDownloader.Main.Client.Controllers;
using MDownloader.Main.Entities;

namespace MDownloader.Main.Client.Views
{
    [Export]
    [PartCreationPolicy(CreationPolicy.NonShared)]
    internal class CaptchaViewModel
    {
        private readonly DelegateCommand proceedCommand;

        private Captcha captcha;
        private CaptchasController controller;


        public CaptchaViewModel()
        {
            this.proceedCommand = new DelegateCommand(this.OnProceed, this.OnProceedVerify);
        }


        public void Initialize(Captcha captcha, CaptchasController controller)
        {
            this.captcha = captcha;
            this.controller = controller;
        }


        public string Text
        {
            get { return this.captcha.Text; }
            set 
            {
                this.captcha.Text = value; 
                this.proceedCommand.Update();
            }
        }

        public byte[] Image
        {
            get { return this.captcha.Data; }
        }


        public ICommand ProceedCommand
        {
            get { return this.proceedCommand;}
        }

        private void OnProceed()
        {
            this.controller.Proceed(this.captcha);
        }

        private bool OnProceedVerify()
        {
            return String.IsNullOrWhiteSpace(this.Text) == false;
        }
    }
}
