﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using MDownloader.Client.Interfaces;
using MDownloader.Main.Client.Controllers;
using MDownloader.Main.Client.Events;
using System.Windows;

namespace MDownloader.Main.Client.Views
{
    [Export, Export(typeof(ICaptchaPanel))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    internal class CaptchasViewModel : ICaptchaPanel, IPartImportsSatisfiedNotification
    {
        private readonly Dictionary<Guid, CaptchaViewModel> captchasMapping;
        private readonly ObservableCollection<CaptchaViewModel> captchas;

        [Import]
        private CaptchasController controller;

        [Import]
        private IServiceProvider serviceProvider;


        public CaptchasViewModel()
        {
            this.captchasMapping = new Dictionary<Guid, CaptchaViewModel>();
            this.captchas = new ObservableCollection<CaptchaViewModel>();
        }


        public ICollection<CaptchaViewModel> Captchas
        {
            get { return this.captchas; }
        }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.controller.CaptchaAdded += this.OnCaptchaAdded;
            this.controller.CaptchaRemoved += this.OnCaptchaRemoved;
        }


        private void OnCaptchaAdded(object sender, CaptchaEventArgs e)
        {
            var captchaId = e.Target.Ref.Id;
            var captcha = this.controller.Model.GetCaptcha(captchaId);
            var viewModel = this.serviceProvider.GetService(typeof(CaptchaViewModel)) as CaptchaViewModel;

            viewModel.Initialize(captcha, this.controller);
            this.captchasMapping.Add(captchaId, viewModel);
            Application.Current.Dispatcher.BeginInvoke(new Action<CaptchaViewModel>(this.captchas.Add), viewModel);
        }

        private void OnCaptchaRemoved(object sender, CaptchaEventArgs e)
        {
            var captchaId = e.Target.Ref.Id;
            var viewModel = this.captchasMapping[captchaId];

            this.captchasMapping.Remove(captchaId);
            Application.Current.Dispatcher.BeginInvoke(new Func<CaptchaViewModel, bool>(this.captchas.Remove), viewModel);
        }
    }
}
