﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Windows;
using MDownloader.Client.Operations;
using MDownloader.Main.Client.Controllers;
using MDownloader.Main.Client.Events;
using MDownloader.Main.Client.Models;
using MDownloader.Main.Client.Models.Interfaces;

namespace MDownloader.Main.Client.Views
{
    [Export, PartCreationPolicy(CreationPolicy.NonShared)]
    internal class QueueViewModel : IPartImportsSatisfiedNotification
    {
        [Import]
        private QueueController controller;

        [Import]
        private IOperationFactory operationFactory;


        private Dictionary<Guid, ResourceBase> resourcesMap = new Dictionary<Guid, ResourceBase>();


        private ObservableCollection<ResourceBase> resources;
        public ICollection<ResourceBase> Resources
        {
            get { return this.resources; }
        }

        public ICollection<OperationViewModel> Operations
        {
            get
            {
                var selectedResources = from resource in this.Resources
                                        where resource.IsSelected
                                        select resource;

                var chosenResources = selectedResources.ToArray();
                var chosenOperations = this.operationFactory.Collect(chosenResources);
                var viewModelsToShow = chosenOperations.ToViewModels(this.controller);

                return viewModelsToShow;
            }
        }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.resources = new ObservableCollection<ResourceBase>();

            this.controller.LinkAdded += this.OnLinksAdded;
            this.controller.LinkChanged += this.OnLinksChanged;
            this.controller.LinksRemoved += this.OnLinksRemoved;

            this.controller.Initialize(this.OnInitializeCompleted);
        }

        private void OnInitializeCompleted()
        {
        }

        private void OnLinksAdded(object sender, LinkEventArgs e)
        {
            this.resourcesMap.Add(e.Target.Ref.Id, new LinkViewModel(this.controller.Model.GetLink(e.Target.Ref.Id)));

            Application.Current.Dispatcher.BeginInvoke(
                new Action<LinkViewModel>(this.resources.Add),
                this.resourcesMap[e.Target.Ref.Id]);
        }

        private void OnLinksChanged(object sender, LinkEventArgs e)
        {
            this.resourcesMap[e.Target.Ref.Id].Update();
        }

        private void OnLinksRemoved(object sender, LinkEventArgs e)
        {
            var resource = this.resourcesMap[e.Target.Ref.Id];
            this.resourcesMap.Remove(e.Target.Ref.Id);

            Application.Current.Dispatcher.BeginInvoke(
                new Func<ResourceBase, bool>(this.resources.Remove),
                resource);
        }
    }
}
