﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Windows;
using MDownloader.Client.Operations;
using MDownloader.Main.Client.Controllers;
using MDownloader.Main.Client.Events;
using MDownloader.Main.Client.Models;
using MDownloader.Main.Client.Models.Interfaces;

namespace MDownloader.Main.Client.Views
{
    [Export, PartCreationPolicy(CreationPolicy.NonShared)]
    internal class TasksViewModel : IPartImportsSatisfiedNotification
    {
        [Import]
        private TasksController controller;

        [Import]
        private IOperationFactory operationFactory;


        private Dictionary<Guid, ITask> taskMap = new Dictionary<Guid, ITask>();


        public ICollection<ITask> Tasks { get; private set; }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.Tasks = new ObservableCollection<ITask>();

            this.controller.TaskAdded += this.OnTasksAdded;
            this.controller.TaskChanged += this.OnTasksChanged;
            this.controller.TasksRemoved += this.OnTasksRemoved;
        }

    
        private void OnTasksAdded(object sender, TaskEventArgs e)
        {
            this.taskMap.Add(e.Target.Ref.Id, new TaskViewModel(this.controller.Model.GetTask(e.Target.Ref.Id)));

            Application.Current.Dispatcher.BeginInvoke(
                new Action<TaskViewModel>(this.Tasks.Add),
                this.taskMap[e.Target.Ref.Id]);
        }

        private void OnTasksChanged(object sender, TaskEventArgs e)
        {
            this.taskMap[e.Target.Ref.Id].Update();
        }

        private void OnTasksRemoved(object sender, TaskEventArgs e)
        {
            var resource = this.taskMap[e.Target.Ref.Id];
            this.taskMap.Remove(e.Target.Ref.Id);

            Application.Current.Dispatcher.BeginInvoke(
                new Func<ITask, bool>(this.Tasks.Remove),
                resource);
        }
    }
}
