﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Windows.Input;
using MDownloader.Client;
using MDownloader.Client.Controllers;
using MDownloader.Client.Interfaces;
using MDownloader.Client.Notifications;
using MDownloader.Main.Client.Controllers;
using MDownloader.Main.Client.Views.Interactions;
using Microsoft.Practices.Prism.Interactivity.InteractionRequest;

namespace MDownloader.Main.Client.Views.Wizards.AddLinks
{
    [Export, PartCreationPolicy(CreationPolicy.NonShared), Connector(typeof(IAddWizardConnector), "/Main/Wizards/AddLinks")]
    internal class WizardViewModel : PropertyNotifier, IPartImportsSatisfiedNotification, IAddWizardConnector, IControllerAware
    {
        [Import]
        private ControllerRegistry controllerRegistry;
        private AddLinksController controller;

        private DelegateCommand<object> pageCommand;
        private DelegateCommand finishCommand;
        private DelegateCommand cancelCommand;
        private InteractionRequest<Notification> closeInteraction;


        public WizardViewModel()
        {
            this.pageCommand = new DelegateCommand<object>(this.OnPageChanged);
            this.finishCommand = new DelegateCommand(this.OnFinish);
            this.cancelCommand = new DelegateCommand(this.OnCancel);
            this.closeInteraction = new InteractionRequest<Notification>();
        }


        [Import]
        public PasteViewModel PastePage { get; private set; }

        [Import]
        public LinksViewModel LinksPage { get; private set; }


        public ExecutionNotifier Executor
        {
            get { return this.controller.Executor; }
        }

        public ICommand PageCommand
        {
            get { return this.pageCommand; }
        }

        private void OnPageChanged(object sender)
        {
            if (sender == this.LinksPage)
            {
                this.LinksPage.Initialize(this.PastePage.Text);
            }
        }

        public ICommand FinishCommand
        {
            get { return this.finishCommand; }
        }

        private void OnFinish()
        {
            this.controller.Queue(() => 
            {
                this.closeInteraction.Raise(new Notification());
            });
        }


        public ICommand CancelCommand
        {
            get { return this.cancelCommand; }
        }

        private void OnCancel()
        {
            this.closeInteraction.Raise(new Notification());
        }


        public IInteractionRequest CloseInteraction
        {
            get { return this.closeInteraction; }
        }
       
        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.controller = this.controllerRegistry.GetController<AddLinksController>(new Uri(String.Format("/Main/Wizards/AddLinks?id={0}", Guid.NewGuid()), UriKind.Relative));
            this.LinksPage.Initialize(controller.Id);
        }


        void IControllerAware.DisposeController()
        {
            this.controllerRegistry.Release(this.controller);
        }
    }
}
