﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;

namespace MDownloader.Main.Interfaces.Base.Tests
{
    [TestFixture]
    public class UrlHelperFixture
    {
        [Test]
        public void CanDealWithUriWithoutParameters()
        {
            var uri = new Uri("http://abc.com/abc.html");
            var parameters = UrlHelper.GetQueryParameters(uri);

            Assert.NotNull(parameters, "The parameters array cannot be null.");
            CollectionAssert.IsEmpty(parameters, "The parameters should be empty.");
        }

        [Test]
        public void CanFindOneParameter()
        {
            var uri = new Uri("http://abc.com/abc.html?abc=cde");
            var parameters = UrlHelper.GetQueryParameters(uri);

            Assert.NotNull(parameters, "The parameters array cannot be null.");
            Assert.AreEqual(1, parameters.Count(), "There should be exactly one parameter.");
            Assert.AreEqual("abc", parameters.First().Key, "The parameters key should be properly set.");
            Assert.AreEqual("cde", parameters.First().Value, "The parameters value should be properly set.");
        }

        [Test]
        public void CanFindMoreParameters()
        {
            var uri = new Uri("http://abc.com/abc.html?abc=cde&fgh=ijk&lmn=opq");
            var parameters = UrlHelper.GetQueryParameters(uri);

            Assert.NotNull(parameters, "The parameters array cannot be null.");
            Assert.AreEqual(3, parameters.Count(), "There should be exactly three parameter.");
        }

        [Test]
        public void CanDealWithParametersWithTheSameKeys()
        {
            var uri = new Uri("http://abc.com/abc.html?abc=cde&abc=def");
            var parameters = UrlHelper.GetQueryParameters(uri);

            Assert.NotNull(parameters, "The parameters array cannot be null.");
            Assert.AreEqual(2, parameters.Count(), "There should be exactly two parameters.");
            Assert.AreEqual("abc", parameters.ElementAt(0).Key, "The parameters key should be properly set.");
            Assert.AreEqual("abc", parameters.ElementAt(1).Key, "The parameters key should be properly set.");
        }

        [Test]
        public void CanDealWithMalformedUriWithEmptyParameter()
        {
            var uri = new Uri("http://abc.com/abc.html?abc=cde&&lmn=opq");
            var parameters = UrlHelper.GetQueryParameters(uri);

            Assert.NotNull(parameters, "The parameters array cannot be null.");
            Assert.AreEqual(2, parameters.Count(), "There should be exactly three parameter.");
        }

        [Test]
        public void CanDealWithMalformedUriWithEmptyValue()
        {
            var uri = new Uri("http://abc.com/abc.html?abc=cde&par=&lmn=opq");
            var parameters = UrlHelper.GetQueryParameters(uri);

            Assert.NotNull(parameters, "The parameters array cannot be null.");
            Assert.AreEqual(3, parameters.Count(), "There should be exactly three parameter.");
            Assert.AreEqual(String.Empty, parameters.ElementAt(1).Value, "The parameter should have default value.");
        }

        [Test]
        public void CanDealWithMailformedUriWithEmptyKey()
        {
            var uri = new Uri("http://abc.com/abc.html?abc=cde&=value&lmn=opq");
            var parameters = UrlHelper.GetQueryParameters(uri);

            Assert.NotNull(parameters, "The parameters array cannot be null.");
            Assert.AreEqual(2, parameters.Count(), "There should be exactly two parameter.");
        }

        [Test]
        public void CanDealWithMalformedUriWithAssignInValue()
        {
            var uri = new Uri("http://abc.com/abc.html?abc=cde&=value&lmn=opq=x");
            var parameters = UrlHelper.GetQueryParameters(uri);

            Assert.NotNull(parameters, "The parameters array cannot be null.");
            Assert.AreEqual(2, parameters.Count(), "There should be exactly two parameter.");
            Assert.AreEqual("opq=x", parameters.ElementAt(1).Value, "The parameter should have expected value.");
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardUriNullParameter()
        {
            UrlHelper.GetQueryParameters(default(Uri));
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardStringNullParameter()
        {
            UrlHelper.GetQueryParameters(default(string));
        }
    }
}
