﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace MDownloader.Main.Interfaces.Base
{
    public abstract class TaskBase<TContext>
    {
        private readonly AutoResetEvent handle;


        protected TaskBase()
        {
            this.handle = new AutoResetEvent(false);
        }


        public event EventHandler Started;
        private void OnStarted()
        {
            var handler = this.Started;
            var args = EventArgs.Empty;

            if (handler != null)
                handler(this, EventArgs.Empty);
        }

        public event EventHandler Finished;
        private void OnFinished()
        {
            var handler = this.Finished;
            var args = EventArgs.Empty;

            if (handler != null)
                handler(this, EventArgs.Empty);
        }


        protected abstract void Execute(TContext context);


        public WaitCallback Callback
        {
            get
            {
                return new WaitCallback(x =>
                {
                    try
                    {
                        this.OnStarted();
                        this.Execute((TContext)x);
                    }
                    finally
                    {
                        this.handle.Set();
                        this.OnFinished();
                    }
                });
            }
        }

        public WaitHandle Handle
        {
            get { return this.handle; }
        }
    }
}
