﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;

namespace MDownloader.Main.Interfaces.Base
{
    public static class UrlHelper
    {
        public static IEnumerable<QueryParameter> GetQueryParameters(Uri uri)
        {
            Ensure.IsNotNull(() => uri);

            return UrlHelper.GetQueryParameters(uri.Query.TrimStart('?'));
        }

        public static IEnumerable<QueryParameter> GetQueryParameters(string data)
        {
            Ensure.IsNotNull(() => data);

            var query = from part in data.Split(new[] { '&' }, StringSplitOptions.RemoveEmptyEntries)
                        let keyValue = part.Split(new[] { '=' }, 2, StringSplitOptions.None)
                        where keyValue.Length >= 1
                        where String.IsNullOrWhiteSpace(keyValue[0]) == false
                        select new QueryParameter
                        {
                            Key = keyValue[0],
                            Value = keyValue.ElementAtOrDefault(1) ?? String.Empty,
                        };

            return query;
        }

        public struct QueryParameter
        {
            public string Key;
            public string Value;
        }
    }
}
