﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace MDownloader.Main.Interfaces.Base
{
    public abstract class UrlParserBase
    {
        public bool IsValid(string link)
        {
            var uri = new Uri(link, UriKind.Absolute);

            if (!this.IsValidHost(uri))
                return false;

            if (!this.IsValidPath(uri))
                return false;

            if (!this.IsValidQuery(uri))
                return false;

            return true;
        }


        protected virtual bool IsValidHost(Uri uri)
        {
            var domains = this.GetSupportedDomains();

            if (domains.Count == 0)
                return false;

            foreach (var domain in domains)
            {
                if (uri.Host.Equals(domain, StringComparison.InvariantCultureIgnoreCase) == true)
                    return true;

                if (uri.Host.EndsWith("." + domain, StringComparison.InvariantCultureIgnoreCase) == true)
                    return true;
            }

            return true;
        }

        protected virtual IList<string> GetSupportedDomains()
        {
            return new string[] { };
        }

        protected abstract bool IsValidPath(Uri uri);
        protected abstract bool IsValidQuery(Uri uri);
    }
}
