﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using MDownloader.Main.Server.Writer.Implementation;
using NUnit.Framework;
using Buffer = MDownloader.Main.Server.Writer.Implementation.Buffer;

namespace MDownloader.Main.Server.Tests.Writer.Implementation
{
    [TestFixture]
    public class BufferFixture
    {
        [Test]
        public void CanCreateNewInstance()
        {
            using (var instance = new Buffer())
            {
            }
        }

        [Test]
        public void CanPushAndPullData()
        {
            using (var instance = new Buffer())
            {
                var data = new byte[] { 0x0, 0x1 };

                using (var inputStream = new MemoryStream(data, false))
                using (var outputStream = new MemoryStream())
                {

                    instance.Fill(inputStream, data.Length);
                    instance.Flush(outputStream);

                    Assert.AreEqual(data.Length, outputStream.Length, "The output stream should contains 2 elements.");
                    Assert.AreEqual(data.Length, outputStream.Position, "The output stream cursor should be at end.");

                    CollectionAssert.AreEqual(data, outputStream.ToArray());
                }
            }
        }

        [Test]
        public void WillNotPullTwiceTheSameData()
        {
            using (var instance = new Buffer())
            {
                var data = new byte[] { 0x0, 0x1 };

                using (var inputStream = new MemoryStream(data, false))
                using (var outputStream = new MemoryStream())
                {
                    instance.Fill(inputStream, data.Length);
                    instance.Flush(outputStream);
                    instance.Flush(outputStream);

                    Assert.AreEqual(data.Length, outputStream.Length, "The output stream should contains 2 elements.");
                    Assert.AreEqual(data.Length, outputStream.Position, "The output stream cursor should be at end.");

                    CollectionAssert.AreEqual(data, outputStream.ToArray());
                }
            }
        }

        [Test]
        public void CanNotify()
        {
            using (var instance = new Buffer())
            {
                var data = new byte[] { 0x0, 0x1 };

                using (var stream = new MemoryStream(data, false))
                {
                    var counter = 0L;

                    instance.Fill(stream, data.Length, value => counter += value);

                    Assert.AreEqual(data.Length, counter, "The counter should hold two bytes.");
                }
            }
        }

        [Test]
        public void CanDealWithEmptyStream()
        {
            using (var instance = new Buffer())
            {
                var data = new byte[] { };

                using (var stream = new MemoryStream(data, false))
                {
                    var counter = 0L;

                    instance.Fill(stream, 2, value => counter += value);

                    Assert.AreEqual(data.Length, counter, "The counter should hold zero bytes.");
                }
            }
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullStreamParameterInFillMethod()
        {
            using (var instance = new Buffer())
            {
                instance.Fill(null, 2);
            }
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullStreamParameterInFlushMethod()
        {
            using (var instance = new Buffer())
            {
                instance.Flush(null);
            }
        }
    }
}
