﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Model;
using MDownloader.Main.Entities;
using MDownloader.Main.Server.Model.Interfaces;

namespace MDownloader.Main.Server.Model.Implementation
{
    internal class ResourceRemover : ResourceWorker
    {
        private readonly Container container;
        private readonly HashSet<Guid> collected;


        public ResourceRemover(Container container)
        {
            this.container = container;
            this.collected = new HashSet<Guid>();
        }


        #region Collect

        public Guid[] Collect(Guid[] resources)
        {
            var links = this.container.Get<Link>(resources);
            var files = this.container.Get<File>(resources);
            var releases = this.container.Get<Release>(resources);

            foreach (var link in links)
            {
                this.collected.Add(link.Identifier.Id);

                var linkFileId = link.GetAll<File>().FirstOrDefault();
                if (linkFileId != null)
                {
                    var linkFile = this.container.Get<File>(linkFileId);

                    if (linkFile.GetAll<Link>().Count == 1)
                    {
                        this.collected.Add(linkFile.Identifier.Id);
                    }
                }
            }

            return this.collected.ToArray();
        }

        #endregion

        #region Remove

        public void Remove()
        {
            var links = this.container.Get<Link>(this.collected);
            var files = this.container.Get<File>(this.collected);
            var releases = this.container.Get<Release>(this.collected);

            foreach (var link in links)
            {
                this.DoRemove(link);
            }

            foreach (var file in files)
            {
                this.DoRemove(file);
            }

            foreach (var release in releases)
            {
                this.DoRemove(release);
            }
        }

        private void DoRemove(Link link)
        {
            if (this.Contains(link.Identifier) == false)
            {
                this.Add(link);

                var urlId = link.GetSingle<Url>();
                var url = this.container.Get<Url>(urlId);

                link.Remove(url.Identifier);
                url.Remove(link.Identifier);

                var fileId = link.GetSingle<File>();
                var file = this.container.Get<File>(fileId);

                link.Remove(file.Identifier);
                file.Remove(link.Identifier);

                this.TryRemove(file);
                this.container.Remove(url, link);
            }
        }

        private void TryRemove(File file)
        {
            if (this.Contains(file.Identifier) == false)
            {
                var linkIds = file.GetAll<Link>();
                if (linkIds.Any() == false)
                {
                    this.DoRemove(file);
                }
            }
        }

        private void DoRemove(File file)
        {
            if (this.Contains(file.Identifier) == false)
            {
                this.Add(file);

                var linkIds = file.GetAll<Link>();
                foreach (var linkId in linkIds)
                {
                    var link = this.container.Get<Link>(linkId);

                    file.Remove(link.Identifier);
                    link.Remove(file.Identifier);

                    this.DoRemove(link);
                }

                var releaseId = file.GetSingleOrDefault<Release>();
                if (releaseId != null)
                {
                    var release = this.container.Get<Release>(releaseId);

                    file.Remove(release.Identifier);
                    release.Remove(file.Identifier);

                    this.TryRemove(release);
                }

                this.container.Remove(file);
            }
        }

        private void TryRemove(Release release)
        {
            if (this.Contains(release.Identifier) == false)
            {
                var fileIds = release.GetAll<File>();
                if (fileIds.Any() == false)
                {
                    this.DoRemove(release);
                }
            }
        }

        private void DoRemove(Release release)
        {
            if (this.Contains(release.Identifier) == false)
            {
                this.Add(release);

                var fileIds = release.GetAll<File>();
                foreach (var fileId in fileIds)
                {
                    var file = this.container.Get<File>(fileId);

                    release.Remove(file.Identifier);
                    file.Remove(release.Identifier);

                    this.DoRemove(file);
                }

                this.container.Remove(release);
            }
        }

        #endregion

        #region Notify

        public void Notify(IRemoveNotifier notifier)
        {
            notifier.OnLinkRemoved(this.GetAll<Link>());
        }

        #endregion
    }
}
