﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Server.Persistence.Interfaces;
using NHibernate;
using NHibernate.Cfg;
using NHibernate.Tool.hbm2ddl;

namespace MDownloader.Main.Server.Persistence.Implementation
{
    [Export(typeof(IPersister))]
    [Export(typeof(IPersisterInternal))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    internal class PersistenceServer : IPersisterInternal
    {
        [ImportMany]
        private IParticipant[] participants;

        private ISessionFactory sessionFactory;

        public void Initialize()
        {
            var configuration = new Configuration();

            this.ConfigureProperties(configuration);
            this.ConfigureEntities(configuration);

            this.sessionFactory = configuration.BuildSessionFactory();
            this.ConfigureSchema(configuration);
        }

        public IPersisterSession OpenSession()
        {
            return new PersistenceSession(this.sessionFactory);
        }

        private void ConfigureProperties(Configuration configuration)
        {
            configuration.SetProperty("connection.provider", "NHibernate.Connection.DriverConnectionProvider, NHibernate");
            configuration.SetProperty("connection.driver_class", "NHibernate.Driver.SQLite20Driver, NHibernate");
            configuration.SetProperty("dialect", "NHibernate.Dialect.SQLiteDialect, NHibernate");
            configuration.SetProperty("query.substitutions", "true=1;false=0");
            configuration.SetProperty("connection.connection_string", "Data Source=nhibernate.db;Version=3;Compress=True");
        }

        private void ConfigureEntities(Configuration configuration)
        {
            var entitiesQuery = from participant in this.participants
                                from type in participant.GetPersistableTypes()
                                select type;

            foreach (var entityType in entitiesQuery)
            {
                configuration.AddClass(entityType);
            }

            configuration.BuildMappings();
        }

        private void ConfigureSchema(Configuration configuration)
        {
            var schemaUpdater = new SchemaUpdate(configuration);
            schemaUpdater.Execute(Console.WriteLine, true);
        }
    }
}
