﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using EFramework.Model;
using MDownloader.Core.Server;
using MDownloader.Main.Entities;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Server.Browser.Interfaces;
using MDownloader.Main.Server.Executor.Interfaces;
using MDownloader.Main.Server.Prober.Interfaces;
using MDownloader.Main.Server.Providers.Interfaces;

namespace MDownloader.Main.Server.Prober.Implementation
{
    [Export(typeof(IProber))]
    [Export(typeof(IProberInternal))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    internal class Prober : IProberInternal
    {
        [Import]
        private IExecutorInternal executor;

        [Import]
        private IProviderServiceInternal providerService;

        [Import]
        private IBrowserFactoryInternal browserFactory;


        public ICollection<IProberLink> Probe(ICollection<IProberUrl> urls)
        {
            var sets = this.Group(urls);
            var tasks = this.BuildTasks(sets).ToArray();

            this.executor.Execute<IBrowser>(this.browserFactory.CreateBrowser(), tasks);

            var query = from task in tasks
                        from link in task.GetFoundPackages()
                        select link;

            return query.ToArray();
        }


        private ICollection<UrlSet> Group(ICollection<IProberUrl> urls)
        {
            var query = from url in urls
                        let provider = this.providerService.Classify(url.Target)
                        where provider != null
                        group url by provider into byProvider
                        select new UrlSet
                        {
                            Provider = byProvider.Key,
                            Urls = byProvider.ToArray(),
                        };

            return query.ToArray();
        }

        private IEnumerable<ProbeTask> BuildTasks(ICollection<UrlSet> sets)
        {
            foreach (var set in sets)
            {
                yield return new ProbeTask(this.providerService.GetProvider(set.Provider), set);
            }
        }
    }
}
