﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Model;
using MDownloader.Core.Server;
using MDownloader.Main.Entities;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Interfaces.Base;
using MDownloader.Main.Server.Executor;

namespace MDownloader.Main.Server.Prober
{
    internal class ProbeTask : TaskBase<IBrowser>, IExecutorTask<IBrowser>
    {
        private readonly IProvider provider;
        private readonly UrlSet urlSet;
        private readonly List<IProberLink> foundLinks;


        public ProbeTask(IProvider provider, UrlSet urlSet)
        {
            this.provider = provider;
            this.urlSet = urlSet;
            this.foundLinks = new List<IProberLink>();
        }


        protected override void Execute(IBrowser context)
        {
            using (var session = provider.OpenSession(context))
            {
                var links = session.Analyze(this.GetUris());
                var query = from link in links
                            where link.IsValid
                            where link.IsAvailable
                            select link;

                this.Process(links);
            }
        }

        public IEnumerable<IProberLink> GetFoundPackages()
        {
            return this.foundLinks;
        }


        private string[] GetUris()
        {
            var query = from url in urlSet.Urls
                        select url.Target;

            return query.ToArray();
        }

        private void Process(IEnumerable<IProviderLink> links)
        {
            foreach (var link in links)
            {
                //var package = new Package<Link>();

                //var linkEntity = new Link(Guid.NewGuid())
                //{
                //    Name = link.Name,
                //    Description = link.Description,
                //    Provider = this.urlSet.Provider.Name,
                //    Size = link.Size,
                //    SizeExactness = link.SizeExactness,
                //};

                //var urlEntity = new Url(Guid.NewGuid())
                //{
                //    Target = link.Target,
                //};

                //package.Add(linkEntity, urlEntity);
                //linkEntity.Add(urlEntity.Identifier);
                //urlEntity.Add(linkEntity.Identifier);

                this.foundLinks.Add(new ProberLink
                {
                    Provider = this.urlSet.Provider.Name,
                    Target = link.Target,
                    Name = link.Name,
                    Description = link.Description,
                    Size = link.Size,
                    SizeExactness = link.SizeExactness,
                });
            }
        }
    }
}
