﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using MDownloader.Main.Entities;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Server.Providers.Interfaces;

namespace MDownloader.Main.Server.Providers.Implementation
{
    [Export(typeof(IProviderServiceInternal))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    internal class ProviderService : IProviderServiceInternal, IPartImportsSatisfiedNotification
    {
        [ImportMany]
        private Lazy<IProvider, IProviderMetadata>[] providersInternal;
        private Dictionary<IProviderMetadata, IProvider> providers;

        [ImportMany]
        private Lazy<IScheduler, IProviderMetadata>[] schedulersInternal;
        private Dictionary<IProviderMetadata, IScheduler> schedulers;

        
        public IProviderMetadata Classify(string url)
        {
            foreach (var entry in this.providers)
            {
                if (entry.Value.Supports(url))
                {
                    return entry.Key;
                }
            }

            return null;
        }

        public IProvider GetProvider(IProviderMetadata metadata)
        {
            return this.providers[metadata];
        }

        public IScheduler GetScheduler(IProviderMetadata metadata)
        {
            return this.schedulers.First(x => x.Key.Name == metadata.Name).Value;
        }



        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.providers = this.providersInternal.ToDictionary(x => x.Metadata, x => x.Value);
            this.schedulers = this.schedulersInternal.ToDictionary(x => x.Metadata, x => x.Value);
        }
    }
}
