﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MDownloader.Main.Server.Repository.Interfaces;

namespace MDownloader.Main.Server.Repository.Implementation
{
    internal class MergeInfo : IMergeInfo
    {
        private readonly HashSet<IType> added;
        private readonly HashSet<IType> changed;
        private readonly HashSet<IType> deleted;


        public MergeInfo()
        {
            this.added = new HashSet<IType>();
            this.changed = new HashSet<IType>();
            this.deleted = new HashSet<IType>();
        }


        public void AddAdded(IType instance)
        {
            this.added.Add(instance);
        }

        public void AddChanged(IType instance)
        {
            this.changed.Add(instance);
        }

        public void AddDeleted(IType instance)
        {
            this.deleted.Add(instance);
        }


        public IEnumerable<IType> GetAdded()
        {
            return this.added;
        }

        public IEnumerable<IType> GetChanged()
        {
            return this.changed;
        }

        public IEnumerable<IType> GetDeleted()
        {
            return this.deleted;
        }


        public TType Get<TType>(Guid id)
            where TType : IType
        {
            var query = from type in this.added.Concat(this.changed)
                        where type.Id == id
                        select type;

            return (TType)query.FirstOrDefault();
        }

        public TType[] Get<TType>(IEnumerable<Guid> ids)
            where TType : IType
        {
            var query = from type in this.added.Concat(this.changed)
                        join id in ids on type.Id equals id
                        where type is TType
                        select (TType)type;

            return query.ToArray();
        }
    }
}
