﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Model;
using EFramework.Model.Tracking;
using Iesi.Collections.Generic;
using MDownloader.Main.Entities;
using MDownloader.Main.Server.Repository.Interfaces;

namespace MDownloader.Main.Server.Repository.Implementation.Parts
{
    [Part(typeof(FileType))]
    internal class FilePart : IPart<FileType>
    {
        public void MergeInstances(IMergeInfo result, IScopeManager scopeManager, IEnumerable<Package> transients, IEnumerable<FileType> persistents)
        {
            var query = from transient in transients
                        from file in transient.GetAll<File>()
                        join persistent in persistents on file.Identifier.Id equals persistent.Id into temp
                        from persistent in temp.DefaultIfEmpty()
                        select new
                        {
                            State = file.Tracker.State,
                            Package = transient,
                            Persisent = persistent,
                        };

            foreach (var entry in query)
            {
                if (entry.State == State.Removed)
                {
                    if (entry.Persisent != null)
                    {
                        result.AddDeleted(entry.Persisent);
                    }
                }
                else
                {
                    if (entry.Persisent != null)
                    {
                        this.CopyProperties(result, scopeManager, entry.Package, entry.Persisent);
                    }
                    else
                    {
                        this.CopyProperties(result, scopeManager, entry.Package);
                    }
                }
            }
        }

        public FileType CopyProperties(IMergeInfo result, IScopeManager scopeManager, Package package, FileType persistent = null)
        {
            var file = package.Leader<File>();
            var scope = scopeManager.Get<FileScope>();
            var created = persistent == null;

            if (persistent == null)
            {
                persistent = new FileType { Id = file.Identifier.Id };
                result.AddAdded(persistent);
            }
            else
            {
                result.AddChanged(persistent);
            }

            if (created || scope.HasFlag(FileScope.Basic))
            {
                persistent.Name = file.Name;
                persistent.Location = file.Location;
            }

            if (created || scope.HasFlag(FileScope.Size))
            {
                persistent.Size = file.Size;
                persistent.SizeExactness = file.SizeExactness;
            }

            if (created || scope.HasFlag(FileScope.LinksReference))
            {
                var linkIds = file.GetAll<Link>();
                var linkEntities = package.Get<Link>(linkIds);
                var linkTypes = result.Get<LinkType>(linkIds.Select(x => x.Id));

                persistent.Links = new HashedSet<LinkType>();
                persistent.Links.AddAll(linkTypes);
            }

            return persistent;
        }
    }
}
