﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Model;
using EFramework.Model.Tracking;
using MDownloader.Main.Entities;
using MDownloader.Main.Server.Repository.Interfaces;

namespace MDownloader.Main.Server.Repository.Implementation.Parts
{
    [Part(typeof(LinkType))]
    internal class LinkPart : IPart<LinkType>
    {
        public void MergeInstances(IMergeInfo result, IScopeManager scopeManager, IEnumerable<Package> transients, IEnumerable<LinkType> persistents)
        {
            var query = from transient in transients
                        from link in transient.GetAll<Link>()
                        join persistent in persistents on link.Identifier.Id equals persistent.Id into temp
                        from persistent in temp.DefaultIfEmpty()
                        select new
                        {
                            State = link.Tracker.State,
                            Package = transient,
                            Persisent = persistent,
                        };

            foreach (var entry in query)
            {
                if (entry.State == State.Removed)
                {
                    if (entry.Persisent != null)
                    {
                        result.AddDeleted(entry.Persisent);
                    }
                }
                else
                {
                    if (entry.Persisent != null)
                    {
                        this.CopyProperties(result, scopeManager, entry.Package, entry.Persisent);
                    }
                    else
                    {
                        this.CopyProperties(result, scopeManager, entry.Package);
                    }
                }
            }
        }

        public LinkType CopyProperties(IMergeInfo result, IScopeManager scopeManager, Package package, LinkType persistent = null)
        {
            var link = package.Leader<Link>();
            var scope = scopeManager.Get<LinkScope>();
            var created = persistent == null;

            if (persistent == null)
            {
                persistent = new LinkType { Id = link.Identifier.Id };
                result.AddAdded(persistent);
            }
            else
            {
                result.AddChanged(persistent);
            }

            if (created || scope.HasFlag(LinkScope.Basic))
            {
                var url = package.Get<Url>(link.GetAll<Url>().First());

                persistent.Target = url.Target;
                persistent.Provider = link.Provider;
            }

            if (created || scope.HasFlag(LinkScope.Description))
            {
                persistent.Name = link.Name;
                persistent.Description = link.Description;
            }

            if (created || scope.HasFlag(LinkScope.Size))
            {
                persistent.Size = link.Size;
                persistent.SizeExactness = link.SizeExactness;
            }

            if (created || scope.HasFlag(LinkScope.Flags))
            {
                persistent.Flags = (long)link.Flags;
                persistent.Status = (int)link.Status;
            }

            if (created || scope.HasFlag(LinkScope.FileReference))
            {
                var fileId = link.GetAll<File>().FirstOrDefault();
                var fileTypes = persistent.File == null ? Enumerable.Empty<FileType>() : new[] { persistent.File };
                var filePackages = new[] { package.PackTo<File>(fileId) };

                PartManager.Get<FileType>().MergeInstances(result, scopeManager, filePackages, fileTypes);
                persistent.File = result.Get<FileType>(fileId.Id);
            }

            return persistent;
        }
    }
}
