﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using EFramework.Model;
using MDownloader.Main.Entities;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Server.Repository.Interfaces;
using NHibernate;

namespace MDownloader.Main.Server.Repository.Implementation
{
    [Export(typeof(IRepositoryInternal))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    internal class Repository : IRepositoryInternal
    {
        [Import]
        private IPersister persister;


        public Package<File>[] GetFiles()
        {
            var result = new List<Package<File>>();

            using (var session = this.persister.OpenSession())
            {
                var query = session.Current
                    .CreateCriteria<FileType>()
                    .SetFetchMode("Package", FetchMode.Lazy)
                    .SetFetchMode("Links", FetchMode.Eager)
                    .List<FileType>();

                foreach (var fileType in query)
                {
                    var file = new File(fileType.Id)
                    {
                        Name = fileType.Name,
                        Location = fileType.Location,
                        Size = fileType.Size,
                        SizeExactness = fileType.SizeExactness,
                    };

                    var package = new Package<File>(file.Identifier);
                    foreach (var linkType in fileType.Links)
                    {
                        var url = new Url(linkType.Id)
                        {
                            Target = linkType.Target,
                        };

                        var link = new Link(linkType.Id)
                        {
                            Name = linkType.Name,
                            Description = linkType.Description,
                            Provider = linkType.Provider,
                            Size = linkType.Size,
                            SizeExactness = linkType.SizeExactness,
                            Flags = (Link.LinkFlags)linkType.Flags,
                            Status = (Link.LinkStatus)linkType.Status,
                        };

                        url.Add(link.Identifier);
                        link.Add(url.Identifier);
                        
                        file.Add(link.Identifier);
                        link.Add(file.Identifier);

                        package.Add(url, link);
                    }

                    package.Add(file);
                    result.Add(package);
                }
            }

            return result.ToArray();
        }

        public IRepositorySession OpenSession()
        {
            return new Session(this.persister);
        }
    }
}
