﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Model;
using MDownloader.Main.Entities;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Server.Repository.Interfaces;
using NHibernate.Criterion;
using NHibernate;

namespace MDownloader.Main.Server.Repository.Implementation
{
    internal class Session : IRepositorySession
    {
        private readonly IPersister persister;
        private readonly IPersisterSession session;


        public Session(IPersister persister)
        {
            this.persister = persister;
            this.session = persister.OpenSession();
        }


        public void Persist(Package<Link> package)
        {
            var links = package.GetAll<Link>().ToIds();
            var persisents = session.Current
                .CreateCriteria<LinkType>()
                .Add(Restrictions.In("Id", links.ToArray()))
                .List<LinkType>();

            var scopeManager = new ScopeManager(LinkScope.All);
            var mergeInfo = Merge.MergeInstances(scopeManager, new[] { package }, persisents);

            try
            {
                foreach (var persisent in mergeInfo.GetAdded())
                {
                    session.Current.SaveOrUpdate(persisent);
                }

                foreach (var persisent in mergeInfo.GetChanged())
                {
                    session.Current.SaveOrUpdate(persisent);
                }

                foreach (var persisent in mergeInfo.GetDeleted())
                {
                    session.Current.Delete(persisent);
                }
            }
            catch
            {
            }
        }

        public void Delete(Guid[] resources)
        {
            var links = session.Current
                .CreateCriteria<LinkType>()
                .SetFetchMode("File", FetchMode.Lazy)
                .Add(Restrictions.In("Id", resources))
                .List<LinkType>();

            foreach (var link in links)
            {
                session.Current.Delete(link);
            }

            var files = session.Current
                .CreateCriteria<FileType>()
                .SetFetchMode("Package", FetchMode.Lazy)
                .SetFetchMode("Links", FetchMode.Lazy)
                .Add(Restrictions.In("Id", resources))
                .List<FileType>();

            foreach (var file in files)
            {
                session.Current.Delete(file);
            }

            var packages = session.Current
                .CreateCriteria<PackageType>()
                .SetFetchMode("Files", FetchMode.Lazy)
                .Add(Restrictions.In("Id", resources))
                .List<PackageType>();

            foreach (var package in packages)
            {
                session.Current.Delete(package);
            }
        }


        void IDisposable.Dispose()
        {
            try
            {
                session.Current.Flush();
                session.Current.Dispose();
            }
            catch
            {
            }
        }
    }
}
