﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MDownloader.Main.Interfaces;

namespace MDownloader.Main.Server.Scheduler.Implementation
{
    internal class Queue
    {
        private readonly HashSet<IScheduler> schedulers;
        private readonly Dictionary<IScheduler, IList<DownloadTask>> tasks;


        public Queue()
        {
            this.schedulers = new HashSet<IScheduler>();
            this.tasks = new Dictionary<IScheduler, IList<DownloadTask>>();
        }


        public event EventHandler<ReadyEventArgs> Ready;
        private void OnReady(IScheduler scheduler, ISchedulerToken token, DownloadTask[] tasks)
        {
            var handler = this.Ready;
            var args = new ReadyEventArgs(scheduler, token, tasks);

            if (handler != null)
            {
                handler(this, args);
            }
        }


        public void Add(IScheduler scheduler, DownloadTask task)
        {
            if (this.schedulers.Add(scheduler) == true)
            {
                scheduler.Changed += (sender, args) => this.TryExecute(scheduler);
            }

            if (this.tasks.ContainsKey(scheduler) == false)
            {
                this.tasks.Add(scheduler, new List<DownloadTask>());
            }

            this.tasks[scheduler].Add(task);
            this.TryExecute(scheduler);
        }

        public void Remove(IScheduler scheduler, DownloadTask task)
        {
            if (this.tasks.ContainsKey(scheduler) == true)
            {
                this.tasks[scheduler].Remove(task);
            }
        }

        private void TryExecute(IScheduler scheduler)
        {
            var tasks = this.tasks[scheduler];
            if (tasks.Count > 0)
            {
                var token = scheduler.GetToken();
                if (token != null)
                {
                    this.OnReady(scheduler, token, tasks.ToArray());
                }
            }
        }
    }
}
