﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Globalization;
using System.Linq;
using System.ServiceModel;
using System.Text;
using EFramework.Model;
using EFramework.Model.Serialization;
using EFramework.Model.Transfer;
using MDownloader.Main.Entities;
using MDownloader.Main.Entities.Transfer;
using MDownloader.Main.Server.Captcher;
using MDownloader.Main.Server.Captcher.Interfaces;
using MDownloader.Main.Server.Engine;
using MDownloader.Main.Server.Model;
using MDownloader.Main.Server.Model.Interfaces;
using MDownloader.Main.Server.Services.Implementation;
using MDownloader.Main.Server.Services.Implementation.Builders;
using MDownloader.Main.Server.Services.Interfaces;

namespace MDownloader.Main.Server.Services.Endpoints
{
    [Export(typeof(IDownloaderInfoService))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    [ServiceBehavior(IncludeExceptionDetailInFaults = true, ConcurrencyMode = ConcurrencyMode.Multiple)]
    internal class DownloaderInfoService : IDownloaderInfoService, IPartImportsSatisfiedNotification
    {
        [Import]
        private IModelInternal model;

        [Import]
        private ICaptcherInternal captcher;

        [Import]
        private ICallbackRepositoryInternal repository;

        [Import]
        private ICallbackDispatcherInternal dispatcher;


        public void Subscribe()
        {
            var callback = OperationContext.Current.GetCallbackChannel<IDownloaderCallback>();
            var flushSink = new CallbackFlushSink(callback);

            this.repository.AddCallback(callback);
            this.model.Flush(flushSink);
        }

        public void Unsubscribe()
        {
            var callback = OperationContext.Current.GetCallbackChannel<IDownloaderCallback>();
            this.repository.RemoveCallback(callback);
        }


        private void OnLinkAdded(object sender, LinkEventArgs args)
        {
            var builder = new LinkBuilder();
            var changes = builder.BuildAdds(args.Targets);

            this.Notify(x => x.OnLinkChanged(changes));
        }

        private void OnLinkChanged(object sender, LinkEventArgs args)
        {
            var builder = new LinkBuilder();
            var changes = builder.BuildChanges(args.Targets, args.Properties);

            this.Notify(x => x.OnLinkChanged(changes));
        }

        private void OnLinkRemoved(object sender, LinkEventArgs args)
        {
            var builder = new LinkBuilder();
            var changes = builder.BuildRemoves(args.Targets);

            this.Notify(x => x.OnLinkChanged(changes));
        }
        

        private void OnTaskAdded(object sender, TaskEventArgs args)
        {
            var builder = new TaskBuilder();
            var changes = builder.BuildAdds(args.Targets);

            this.Notify(x => x.OnTaskChanged(changes));
        }

        private void OnTaskChanged(object sender, TaskEventArgs args)
        {
            var builder = new TaskBuilder();
            var changes = builder.BuildChanges(args.Targets, args.Properties);

            this.Notify(x => x.OnTaskChanged(changes));
        }

        private void OnTaskRemoved(object sender, TaskEventArgs args)
        {
            var builder = new TaskBuilder();
            var changes = builder.BuildRemoves(args.Targets);

            this.Notify(x => x.OnTaskChanged(changes));
        }


        private void OnCaptchaAdded(object sender, CaptchaEventArgs args)
        {
            var builder = new CaptchaBuilder();
            var changes = builder.BuildAdds(args.Targets);

            this.Notify(x => x.OnCaptchaChanged(changes));
        }

        private void OnCaptchaRemoved(object sender, CaptchaEventArgs args)
        {
            var builder = new CaptchaBuilder();
            var changes = builder.BuildRemoves(args.Targets);

            this.Notify(x => x.OnCaptchaChanged(changes));
        }


        private void Notify(Action<IDownloaderCallback> callbackAction)
        {
            this.dispatcher.Do(callbackAction);
        }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.model.LinkAdded += this.OnLinkAdded;
            this.model.LinkChanged += this.OnLinkChanged;
            this.model.LinkRemoved += this.OnLinkRemoved;

            this.model.TaskAdded += this.OnTaskAdded;
            this.model.TaskChanged += this.OnTaskChanged;
            this.model.TaskRemoved += this.OnTaskRemoved;

            this.captcher.CaptchaAdded += this.OnCaptchaAdded;
            this.captcher.CaptchaRemoved += this.OnCaptchaRemoved;
        }
    }
}
