﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using MDownloader.Main.Entities;
using MDownloader.Main.Entities.Transfer;

namespace MDownloader.Main.Server.Services.Implementation.Builders
{
    internal class LinkBuilder
    {
        public LinkChange[] BuildAdds(Link[] links)
        {
            var changes = new List<LinkChange>(links.Length);

            foreach (var link in links)
            {
                changes.Add(new LinkChange
                {
                    Change = ChangeType.Added,
                    Ref = new LinkRef
                    {
                        Id = link.Identifier.Id
                    },
                    Properties = new Dictionary<string, string>
                    {
                        { LinkProperties.Name, link.Name },
                        { LinkProperties.Description, link.Description },
                        { LinkProperties.Provider, link.Provider },
                        { LinkProperties.Size, link.Size.ToString(CultureInfo.InvariantCulture) },
                        { LinkProperties.SizeExactness, link.SizeExactness.ToString() },
                        { LinkProperties.Flags, link.Flags.ToString() },
                        { LinkProperties.Status, link.Status.ToString() },
                    }
                });
            }

            return changes.ToArray();
        }

        public LinkChange[] BuildChanges(Link[] links, string[] properties)
        {
            var changes = new List<LinkChange>(links.Length);

            foreach (var link in links)
            {
                var linkChange = new LinkChange
                {
                    Change = ChangeType.Changed,
                    Ref = new LinkRef
                    {
                        Id = link.Identifier.Id
                    },
                    Properties = new Dictionary<string, string>()
                };

                foreach (var property in properties)
                {
                    switch (property)
                    {
                        case LinkProperties.Name: linkChange.Properties.Add(LinkProperties.Name, link.Name); break;
                        case LinkProperties.Description: linkChange.Properties.Add(LinkProperties.Description, link.Description); break;
                        case LinkProperties.Provider: linkChange.Properties.Add(LinkProperties.Provider, link.Provider); break;
                        case LinkProperties.Size: linkChange.Properties.Add(LinkProperties.Size, link.Size.ToString(CultureInfo.InvariantCulture)); break;
                        case LinkProperties.SizeExactness: linkChange.Properties.Add(LinkProperties.SizeExactness, link.SizeExactness.ToString()); break;
                        case LinkProperties.Flags: linkChange.Properties.Add(LinkProperties.Flags, link.Flags.ToString()); break;
                        case LinkProperties.Status: linkChange.Properties.Add(LinkProperties.Status, link.Status.ToString()); break;
                    }
                }

                changes.Add(linkChange);
            }

            return changes.ToArray();
        }
        
        public LinkChange[] BuildRemoves(Link[] links)
        {
            var changes = new List<LinkChange>(links.Length);

            foreach (var link in links)
            {
                changes.Add(new LinkChange
                {
                    Change = ChangeType.Removed,
                    Ref = new LinkRef
                    {
                        Id = link.Identifier.Id
                    }
                });
            }

            return changes.ToArray();
        }
    }
}
