﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using MDownloader.Main.Entities;
using MDownloader.Main.Entities.Transfer;

namespace MDownloader.Main.Server.Services.Implementation.Builders
{
    internal class TaskBuilder
    {
        public TaskChange[] BuildAdds(Task[] tasks)
        {
            var changes = new List<TaskChange>(tasks.Length);

            foreach (var task in tasks)
            {
                changes.Add(new TaskChange
                {
                    Change = ChangeType.Added,
                    Ref = new TaskRef
                    {
                        Id = task.Identifier.Id
                    },
                    Properties = new Dictionary<string, string>
                    {
                        { TaskProperties.Name, task.Name },
                        { TaskProperties.Provider, task.Provider },
                        { TaskProperties.Progress, task.Progress.ToString(CultureInfo.InvariantCulture) },
                        { TaskProperties.Size, task.Size.ToString(CultureInfo.InvariantCulture) },
                        { TaskProperties.Speed, task.Speed.ToString(CultureInfo.InvariantCulture) },
                        { TaskProperties.Status, task.Status.ToString() },
                        { TaskProperties.Counter, task.Counter.ToString(CultureInfo.InvariantCulture) },
                    }
                });
            }

            return changes.ToArray();
        }

        public TaskChange[] BuildChanges(Task[] tasks, string[] properties)
        {
            var changes = new List<TaskChange>(tasks.Length);

            foreach (var task in tasks)
            {
                var taskChange = new TaskChange
                {
                    Change = ChangeType.Changed,
                    Ref = new TaskRef
                    {
                        Id = task.Identifier.Id
                    },
                    Properties = new Dictionary<string, string>()
                };

                foreach (var property in properties)
                {
                    switch (property)
                    {
                        case TaskProperties.Name: taskChange.Properties.Add(TaskProperties.Name, task.Name); break;
                        case TaskProperties.Provider: taskChange.Properties.Add(TaskProperties.Provider, task.Provider); break;
                        case TaskProperties.Progress: taskChange.Properties.Add(TaskProperties.Progress, task.Progress.ToString(CultureInfo.InvariantCulture)); break;
                        case TaskProperties.Size: taskChange.Properties.Add(TaskProperties.Size, task.Size.ToString(CultureInfo.InvariantCulture)); break;
                        case TaskProperties.Speed: taskChange.Properties.Add(TaskProperties.Speed, task.Speed.ToString(CultureInfo.InvariantCulture)); break;
                        case TaskProperties.Status: taskChange.Properties.Add(TaskProperties.Status, task.Status.ToString()); break;
                        case TaskProperties.Counter: taskChange.Properties.Add(TaskProperties.Counter, task.Counter.ToString(CultureInfo.InvariantCulture)); break;
                    }
                }

                changes.Add(taskChange);
            }

            return changes.ToArray();
        }

        public TaskChange[] BuildRemoves(Task[] tasks)
        {
            var changes = new List<TaskChange>(tasks.Length);

            foreach (var link in tasks)
            {
                changes.Add(new TaskChange
                {
                    Change = ChangeType.Removed,
                    Ref = new TaskRef
                    {
                        Id = link.Identifier.Id
                    }
                });
            }

            return changes.ToArray();
        }
    }
}
