﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Text;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Server.Services;
using MDownloader.Server.Services;
using MDownloader.Main.Server.Services.Endpoints;

namespace MDownloader.Main.Server.Services.Implementation
{
    [Export(typeof(IServiceHostFactory))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    internal class ServiceHostFactory : IServiceHostFactory
    {
        [Import]
        private IServiceProvider serviceProvider;

        private ServiceHost downloaderEngineService;
        private ServiceHost downloaderInfoService;
        private ServiceHost downloaderManageService;
        private ServiceHost downloaderScheduleService;
        private ServiceHost downloaderTransferService;
        private ServiceHost downloaderCaptchaService;

        public void Initialize()
        {
            this.downloaderEngineService = this.BuildHost<IDownloaderEngineService, DownloaderEngineService>(new Uri("net.tcp://localhost:4712/MDownloader/Main/Engine"));
            this.downloaderInfoService = this.BuildHost<IDownloaderInfoService, DownloaderInfoService>(new Uri("net.tcp://localhost:4712/MDownloader/Main/Info"));
            this.downloaderManageService = this.BuildHost<IDownloaderManageService, DownloaderManageService>(new Uri("net.tcp://localhost:4712/MDownloader/Main/Manage"));
            this.downloaderScheduleService = this.BuildHost<IDownloaderScheduleService, DownloaderScheduleService>(new Uri("net.tcp://localhost:4712/MDownloader/Main/Schedule"));
            this.downloaderTransferService = this.BuildHost<IDownloaderTransferService, DownloaderTransferService>(new Uri("net.tcp://localhost:4712/MDownloader/Main/Transfer"));
            this.downloaderCaptchaService = this.BuildHost<IDownloaderCaptchaService, DownloaderCaptchaService>(new Uri("net.tcp://localhost:4712/MDownloader/Main/Captcha"));
        }

        public void Start()
        {
            this.downloaderEngineService.Open();
            this.downloaderInfoService.Open();
            this.downloaderManageService.Open();
            this.downloaderScheduleService.Open();
            this.downloaderTransferService.Open();
            this.downloaderCaptchaService.Open();
        }

        public void Stop()
        {
            this.downloaderEngineService.Close();
            this.downloaderInfoService.Close();
            this.downloaderManageService.Close();
            this.downloaderScheduleService.Close();
            this.downloaderTransferService.Close();
            this.downloaderCaptchaService.Close();
        }

        private ServiceHost BuildHost<TContract, TService>(Uri baseAddress)
            where TService : class, TContract
        {
            var host = new ServiceHost(typeof(TService));

            ServiceHostFactory.ConfigureMetadataEndpoint<TContract>(host, baseAddress);
            host.AddServiceEndpoint(typeof(TContract), new NetTcpBinding(), baseAddress);
            host.Description.Behaviors.Add(new ServiceBehavior(typeof(TContract), this.serviceProvider));

            return host;
        }

        [Conditional("DEBUG")]
        private static void ConfigureMetadataEndpoint<TContract>(ServiceHost host, Uri baseAddress)
        {
            var metadata = new ServiceMetadataBehavior();
            var httpUri = new UriBuilder(baseAddress) { Scheme = "http", Port = 4713 };

            metadata.HttpGetEnabled = true;
            metadata.HttpGetUrl = httpUri.Uri;
            metadata.MetadataExporter.PolicyVersion = PolicyVersion.Policy15;

            host.Description.Behaviors.Add(metadata);
        }
    }
}
