﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;

namespace MDownloader.Main.Server.Writer.Implementation
{
    internal class Buffer : IDisposable
    {
        private readonly BufferAdjuster adjuster;

        private long processed;
        private byte[] array;
        private int used;


        public Buffer(BufferAdjuster adjuster = null)
        {
            this.used = 0;
            this.adjuster = adjuster ?? BufferAdjuster.Default();
            this.adjuster.Adjust(ref this.array);
        }


        public void Fill(Stream stream, long capacity, Action<long> peak = null)
        {
            Ensure.IsNotNull(() => stream);

            this.adjuster.Adjust(ref this.array);

            var left = (int)Math.Min(capacity, this.array.Length);
            this.used = 0;

            while (left > 0)
            {
                var read = stream.Read(this.array, this.used, left);
                if (read <= 0)
                    break;

                if (peak != null)
                    peak(read);

                this.used += read;
                left -= read;
            }
        }

        public long Flush(Stream stream)
        {
            Ensure.IsNotNull(() => stream);

            var current = this.used;

            stream.Write(this.array, 0, this.used);
            this.used = 0;

            this.processed += current;
            this.adjuster.Tick(current);

            return current;
        }


        void IDisposable.Dispose()
        {
            this.array = null;
            GC.Collect();
        }
    }
}
