﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using MDownloader.Core.Server;

namespace MDownloader.Main.Server.Writer.Implementation
{
    internal class DefaultBufferAdjuster : BufferAdjuster
    {
        private static readonly int MinSize = 64 * 1024;

        private readonly IDateTimeProvider dateTimeProvider;
        private readonly DateTime created;
        private readonly int multiplier;
        private readonly int prober;

        private long processed;
        private int ticks;


        public DefaultBufferAdjuster(IDateTimeProvider dateTimeProvider, int multiplier = 2, int prober = 4)
        {
            Ensure.IsNotNull(() => dateTimeProvider);

            this.dateTimeProvider = dateTimeProvider;
            this.created = this.dateTimeProvider.Now();

            this.multiplier = multiplier;
            this.prober = prober;
        }


        public override void Tick(long bytes)
        {
            processed += bytes;
        }

        public override void Adjust(ref byte[] buffer)
        {
            if (buffer == null)
            {
                buffer = new byte[DefaultBufferAdjuster.MinSize];
            }
            else if (this.ticks++ % this.prober == 0)
            {
                var speed = this.GetSpeed();
                var length = buffer.Length;

                if (speed > 0 && speed >= length * this.multiplier)
                {
                    Console.WriteLine("Expanding buffer from {0} to {1}...", length, length * this.multiplier);

                    Array.Resize(ref buffer, length * this.multiplier);
                    GC.Collect();
                }
                else if (speed > 0 && speed <= length / this.multiplier && DefaultBufferAdjuster.MinSize <= length / this.multiplier)
                {
                    Console.WriteLine("Shrinking buffer from {0} to {1}...", length, length / this.multiplier);

                    Array.Resize(ref buffer, length / this.multiplier);
                    GC.Collect();
                }
            }
        }

        private int GetSpeed()
        {
            var now = this.dateTimeProvider.Now();
            var timespan = now - this.created;
            var seconds = timespan.TotalSeconds;

            if (seconds > 0)
            {
                return Convert.ToInt32(Math.Round(this.processed / seconds, 0));
            }
            else
            {
                return 0;
            }
        }
    }
}
