﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using EFramework.Model;
using MDownloader.Client.Controllers;
using MDownloader.Search.Client.Events;
using MDownloader.Search.Client.Services;
using MDownloader.Search.Entities.Transfer;

namespace MDownloader.Search.Client.Controllers
{
    [Export]
    internal class SearchController : ControllerBase, IPartImportsSatisfiedNotification
    {
        [Import]
        private InfoServiceProxy infoServiceProxy;

        [Import]
        private EngineServiceProxy engineServiceProxy;


        public SearchModel Model { get; private set; }


        public void Initialize(Action onComplete)
        {
            this.Execute(this.infoServiceProxy.Initialize, onComplete);
        }


        public void Search(string text)
        {
            var package = this.Model.NewSpecification(text);
            this.OnQueryAdded(new QueryEventArgs(new SpecificationChange { Ref = new SpecificationRef { Id = package.Leader().Identifier.Id } }));

            this.Execute(() => this.engineServiceProxy.Search(package.ToContainer()));
        }


        public event EventHandler<QueryEventArgs> QueryAdded;
        private void OnQueryAdded(QueryEventArgs args)
        {
            var handler = this.QueryAdded;

            if (handler != null)
                handler(this, args);
        }


        public event EventHandler<ResultEventArgs> ResultAdded;
        private void OnResultAdded(ResultEventArgs args)
        {
            var handler = this.ResultAdded;

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<ResultEventArgs> ResultChanged;
        private void OnResultChanged(ResultEventArgs args)
        {
            var handler = this.ResultChanged;

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<ResultEventArgs> ResultRemoved;
        private void OnResultRemoved(ResultEventArgs args)
        {
            var handler = this.ResultRemoved;

            if (handler != null)
                handler(this, args);
        }


        private void OnResultAdded(object sender, ResultEventArgs e)
        {
            this.Model.AddResult(e.Target.SpecificationRef.Id, e.Target.Ref.Id, e.Target.Properties);
            this.OnResultAdded(e);
        }

        private void OnResultChanged(object sender, ResultEventArgs e)
        {
            this.Model.ChangeResult(e.Target.Ref.Id, e.Target.Properties);
            this.OnResultChanged(e);
        }

        private void OnResultRemoved(object sender, ResultEventArgs e)
        {
            this.Model.RemoveResult(e.Target.Ref.Id);
            this.OnResultRemoved(e);
        }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.Model = new SearchModel();

            this.infoServiceProxy.ResultAdded += this.OnResultAdded;
            this.infoServiceProxy.ResultChanged += this.OnResultChanged;
            this.infoServiceProxy.ResultRemoved += this.OnResultRemoved;
        }
    }
}
