﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using EFramework.Model;
using EFramework.Model.Cloning;
using MDownloader.Search.Entities;

namespace MDownloader.Search.Client.Controllers
{
    internal class SearchModel
    {
        private readonly Container container = new Container(Guid.NewGuid());


        public Container Container
        {
            get { return this.container; }
        }

        
        public Package<Specification> NewSpecification(string query)
        {
            var package = new Package<Specification>();
            var specification = new Specification(Guid.NewGuid())
            {
                Query = query,
            };

            this.container.Add(specification);
            package.Add(specification.Clone(new CloneSettings()));

            return package;
        }

        public void AddParameters(IDictionary<string, string> parameters)
        {
            var specification = this.container.GetAll<Specification>().First();

            foreach (var parameter in parameters)
            {
                var entity = new Parameter(Guid.NewGuid())
                {
                    Key = parameter.Key,
                    Value = parameter.Value,
                };

                this.container.Add(entity);
                specification.Add(entity.Identifier);
            }
        }


        public Specification GetQuery(Guid id)
        {
            return this.container.Get<Specification>(id);
        }


        public Result GetResult(Guid id)
        {
            return this.container.Get<Result>(id);
        }

        public void AddResult(Guid specificationId, Guid resultId, IDictionary<string, string> properties)
        {
            var specification = this.container.Get<Specification>(specificationId);
            var result = new Result(resultId)
            {
                Name = properties[ResultProperties.Name],
                Provider = properties[ResultProperties.Provider],
                Files = Int32.Parse(properties[ResultProperties.Files], CultureInfo.InvariantCulture),
                Size = Int64.Parse(properties[ResultProperties.Size], CultureInfo.InvariantCulture),
                SizeExactness = Boolean.Parse(properties[ResultProperties.SizeExactness]),
                Status = (Result.ResultStatus)Enum.Parse(typeof(Result.ResultStatus), properties[ResultProperties.Status]),
            };

            this.container.Add(result);

            specification.Add(result.Identifier);
            result.Add(specification.Identifier);
        }

        public void ChangeResult(Guid resultId, IDictionary<string, string> properties)
        {
            var result = this.container.Get<Result>(resultId);

            if (properties.ContainsKey(ResultProperties.Name))
                result.Name = properties[ResultProperties.Name];

            if (properties.ContainsKey(ResultProperties.Provider))
                result.Provider = properties[ResultProperties.Provider];

            if (properties.ContainsKey(ResultProperties.Files))
                result.Files = Int32.Parse(properties[ResultProperties.Files], CultureInfo.InvariantCulture);

            if (properties.ContainsKey(ResultProperties.Size))
                result.Size = Int64.Parse(properties[ResultProperties.Size], CultureInfo.InvariantCulture);
            
            if (properties.ContainsKey(ResultProperties.SizeExactness))
                result.SizeExactness = Boolean.Parse(properties[ResultProperties.SizeExactness]);

            if (properties.ContainsKey(ResultProperties.Status))
                result.Status = (Result.ResultStatus)Enum.Parse(typeof(Result.ResultStatus), properties[ResultProperties.Status]);
        }

        public void RemoveResult(Guid linkId)
        {
            this.container.Remove<Result>(linkId);
        }
    }
}
