﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.ServiceModel;
using System.Text;
using MDownloader.Search.Client.InfoServiceReference;
using MDownloader.Search.Entities.Transfer;
using MDownloader.Search.Client.Events;

namespace MDownloader.Search.Client.Services
{
    [Export]
    [CallbackBehavior(IncludeExceptionDetailInFaults = true)]
    internal class InfoServiceProxy : ISearchInfoServiceCallback
    {
        private SearchInfoServiceClient client;


        public void Initialize()
        {
            this.GetClient();
        }


        private ISearchInfoService GetClient()
        {
            if (this.client == null)
                this.client = new SearchInfoServiceClient(new InstanceContext(this), new NetTcpBinding(), new EndpointAddress("net.tcp://localhost:4712/MDownloader/Search/Info"));

            client.Subscribe();
            return this.client;
        }


        public event EventHandler<ResultEventArgs> ResultAdded;
        private void OnResultAdded(ResultChange resultChange)
        {
            var handler = this.ResultAdded;
            var args = new ResultEventArgs(resultChange);

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<ResultEventArgs> ResultChanged;
        private void OnResultChanged(ResultChange resultChange)
        {
            var handler = this.ResultChanged;
            var args = new ResultEventArgs(resultChange);

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<ResultEventArgs> ResultRemoved;
        private void OnResultRemoved(ResultChange resultChange)
        {
            var handler = this.ResultRemoved;
            var args = new ResultEventArgs(resultChange);

            if (handler != null)
                handler(this, args);
        }


        void ISearchInfoServiceCallback.OnResultChanged(ResultChange[] resultChanges)
        {
            foreach (var resultChange in resultChanges)
            {
                switch (resultChange.Change)
                {
                    case ChangeType.Added: this.OnResultAdded(resultChange); break;
                    case ChangeType.Changed: this.OnResultChanged(resultChange); break;
                    case ChangeType.Removed: this.OnResultRemoved(resultChange); break;
                }
            }
        }
    }
}
