﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Windows;
using MDownloader.Client.Interfaces;
using MDownloader.Search.Client.Controllers;
using MDownloader.Search.Client.Events;
using MDownloader.Search.Client.Models;
using MDownloader.Search.Client.Models.Interfaces;

namespace MDownloader.Search.Client.Views
{
    [Export, Export(typeof(IModulePanel))]
    [ExportMetadata("Order", 2)]
    [PartCreationPolicy(CreationPolicy.Shared)]
    internal class PanelViewModel : IModulePanel, IPartImportsSatisfiedNotification
    {
        [Import]
        private SearchController controller;

        [Import]
        private IServiceProvider serviceProvider;


        public string Header
        {
            get { return "Search"; }
        }

        private Dictionary<Guid, ResultsViewModel> queriesMap = new Dictionary<Guid, ResultsViewModel>();
        private ObservableCollection<ResultsViewModel> queries;
        public ICollection<ResultsViewModel> Queries
        {
            get { return this.queries; }
        }

        
        [Import]
        public QueryViewModel Query { get; private set; }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.queries = new ObservableCollection<ResultsViewModel>();

            this.controller.QueryAdded += this.OnQueryAdded;

            this.controller.Initialize(this.OnInitializeCompleted);
        }

        private void OnInitializeCompleted()
        {
        }


        private void OnQueryAdded(object sender, QueryEventArgs e)
        {
            this.queriesMap.Add(e.Target.Ref.Id, (ResultsViewModel)this.serviceProvider.GetService(typeof(ResultsViewModel)));
            this.queriesMap[e.Target.Ref.Id].Initialize(new SpecificationViewModel(this.controller.Model.GetQuery(e.Target.Ref.Id)));

            Application.Current.Dispatcher.BeginInvoke(
                new Action<ResultsViewModel>(this.queries.Add),
                this.queriesMap[e.Target.Ref.Id]);
        }
    }
}
