﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Windows.Input;
using MDownloader.Client;
using MDownloader.Client.Notifications;
using MDownloader.Search.Client.Controllers;
using MDownloader.Search.Client.Views.Interactions;
using Microsoft.Practices.Prism.Interactivity.InteractionRequest;

namespace MDownloader.Search.Client.Views
{
    [Export, PartCreationPolicy(CreationPolicy.NonShared)]
    internal class QueryViewModel : IPartImportsSatisfiedNotification
    {
        [Import]
        private IServiceProvider serviceProvider;

        [Import]
        private SearchController controller;

        private readonly DelegateCommand searchCommand;
        private readonly DelegateCommand advancedCommand;
        private readonly InteractionRequest<Interaction<IAdvancedConnector>> advancedRequest;


        public QueryViewModel()
        {
            this.searchCommand = new DelegateCommand(this.OnSearch, this.OnCanSearch);
            this.advancedCommand = new DelegateCommand(this.OnAdvanced);
            this.advancedRequest = new InteractionRequest<Interaction<IAdvancedConnector>>();
        }


        public IServiceProvider ServiceProvider
        {
            get { return this.serviceProvider; }
        }


        private string searchText;
        public string SearchText
        {
            get { return this.searchText; }
            set
            {
                if (this.searchText != value)
                {
                    this.searchText = value;
                    this.searchCommand.Update();
                }
            }
        }


        public ICommand AdvancedCommand
        {
            get { return this.advancedCommand; }
        }

        private void OnAdvanced()
        {
            this.advancedRequest.Raise(new Interaction<IAdvancedConnector>(), interaction =>
            {
                var connector = interaction.Connector;
            });
        }

        public ICommand SearchCommand
        {
            get { return this.searchCommand; }
        }

        private bool OnCanSearch()
        {
            return String.IsNullOrWhiteSpace(this.searchText) == false;
        }

        private void OnSearch()
        {
            this.controller.Search(this.searchText);
        }


        public IInteractionRequest ShowAdvancedRequest
        {
            get { return this.advancedRequest; }
        }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
        }
    }
}
