﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using System.Windows;
using MDownloader.Search.Client.Controllers;
using MDownloader.Search.Client.Events;
using MDownloader.Search.Client.Models;
using MDownloader.Search.Client.Models.Interfaces;

namespace MDownloader.Search.Client.Views
{
    [Export, PartCreationPolicy(CreationPolicy.NonShared)]
    internal class ResultsViewModel : IPartImportsSatisfiedNotification
    {
        [Import]
        private SearchController controller;

        private SpecificationViewModel specification;


        public void Initialize(SpecificationViewModel specification)
        {
            this.specification = specification;
        }


        private Dictionary<Guid, IResult> resultsMap = new Dictionary<Guid, IResult>();
        private ObservableCollection<IResult> results;
        public ICollection<IResult> Results
        {
            get { return this.results; }
        }

        public string Header
        {
            get { return this.specification.Name; }
        }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.results = new ObservableCollection<IResult>();

            this.controller.ResultAdded += this.OnResultsAdded;
            this.controller.ResultChanged += this.OnResultsChanged;
            this.controller.ResultRemoved += this.OnResultsRemoved;
        }


        private void OnResultsAdded(object sender, ResultEventArgs e)
        {
            if (e.Target.SpecificationRef.Id == this.specification.Id)
            {
                this.resultsMap.Add(e.Target.Ref.Id, new ResultViewModel(this.controller.Model.GetResult(e.Target.Ref.Id)));

                Application.Current.Dispatcher.BeginInvoke(
                    new Action<ResultViewModel>(this.results.Add),
                    this.resultsMap[e.Target.Ref.Id]);
            }
        }

        private void OnResultsChanged(object sender, ResultEventArgs e)
        {
            if (e.Target.SpecificationRef.Id == this.specification.Id)
            {
                this.resultsMap[e.Target.Ref.Id].Update();
            }
        }

        private void OnResultsRemoved(object sender, ResultEventArgs e)
        {
            if (e.Target.SpecificationRef.Id == this.specification.Id)
            {
                var resource = this.resultsMap[e.Target.Ref.Id];
                this.resultsMap.Remove(e.Target.Ref.Id);

                Application.Current.Dispatcher.BeginInvoke(
                    new Func<IResult, bool>(this.results.Remove),
                    resource);
            }
        }
    }
}
