﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using EFramework.Model;
using EFramework.Model.Merging;
using EFramework.Model.Tracking;
using MDownloader.Search.Entities;
using MDownloader.Search.Server.Model.Interfaces;

namespace MDownloader.Search.Server.Model.Implementation
{
    [Export(typeof(IModelInternal))]
    internal class Model : IModelInternal
    {
        /// <summary>
        /// Holds a reference to current model container.
        /// </summary>
        private Container container;


        public Model()
        {
            this.container = new Container(Guid.NewGuid());
            this.container.Changed += this.OnChanged;
        }


        private void OnChanged(object sender, TrackerEventArgs e)
        {
            if (e.Owner is Specification)
            {
                this.OnSpecificationChanged(e.Owner as Specification, e.PropertyName);
            }

            if (e.Owner is Result)
            {
                this.OnResultChanged(e.Owner as Result, e.PropertyName);
            }
        }

        public event EventHandler<SpecificationEventArgs> SpecificationChanged;
        private void OnSpecificationChanged(Specification specification, string propertyName)
        {
            var handler = this.SpecificationChanged;
            var args = new SpecificationEventArgs(specification, propertyName);

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<ResultEventArgs> ResultAdded;
        private void OnResultAdded(params Result[] results)
        {
            var handler = this.ResultAdded;
            var args = new ResultEventArgs(results);

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<ResultEventArgs> ResultChanged;
        private void OnResultChanged(Result result, string propertyName)
        {
            var handler = this.ResultChanged;
            var args = new ResultEventArgs(result, propertyName);

            if (handler != null)
                handler(this, args);
        }

        public event EventHandler<ResultEventArgs> ResultRemoved;
        private void OnResultRemoved(params Result[] results)
        {
            var handler = this.ResultRemoved;
            var args = new ResultEventArgs(results);

            if (handler != null)
                handler(this, args);
        }



        public void Add(Package<Specification> specificatation)
        {
            var mergeSettings = new MergeSettings { CloneEntities = false };
            this.container.Merge(specificatation, mergeSettings);
        }

        public void AddResults(Package<Specification> specificatation)
        {
            var mergeSettings = new MergeSettings 
            {
                CloneEntities = false,
                OnAdded = entity => 
                {
                    if (entity is Result)
                    {
                        this.OnResultAdded(entity as Result);
                    }
                },
            };

            this.container.Merge(specificatation, mergeSettings);
        }


        public Parameter[] GetParameters(Specification specification)
        {
            var parameterIds = specification.GetAll<Parameter>();
            var parameters = this.container.Get<Parameter>(parameterIds);

            return parameters.ToArray();
        }

        public Url[] GetUrls(Result result)
        {
            var urlIds = result.GetAll<Url>();
            var urls = this.container.Get<Url>(urlIds);

            return urls.ToArray();
        }
    }
}
