﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MDownloader.Core.Server;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Interfaces.Base;
using MDownloader.Search.Entities;
using MDownloader.Search.Interfaces;
using MDownloader.Search.Server.Model.Interfaces;

namespace MDownloader.Search.Server.Scheduler
{
    internal class SearchTask : TaskBase<IBrowser>, IExecutorTask<IBrowser>
    {
        private readonly ISearchEngine searchEngine;
        private readonly Specification specification;
        private readonly Parameter[] parameters;

        private ISearchBag bag;


        public SearchTask(ISearchEngine searchEngine, Specification specification, Parameter[] parameters)
        {
            this.searchEngine = searchEngine;
            this.specification = specification;
            this.parameters = parameters;
        }


        public ISearchBag GetBag()
        {
            return this.bag;
        }

        public Specification GetSpecification()
        {
            return this.specification;
        }


        protected override void Execute(IBrowser context)
        {
            var query = this.BuildQuery();
            this.bag = this.searchEngine.Search(context, query);
        }


        private ISearchQuery BuildQuery()
        {
            var query = new SearchQuery(this.specification.Query);

            foreach (var parameter in this.parameters)
            {
                query.AddParameter(parameter.Key, parameter.Value);
            }

            return query;
        }
    }
}
