﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.ServiceModel;
using System.Text;
using MDownloader.Search.Server.Model;
using MDownloader.Search.Server.Model.Interfaces;
using MDownloader.Search.Server.Services.Implementation;
using MDownloader.Search.Server.Services.Interfaces;

namespace MDownloader.Search.Server.Services.Endpoints
{
    [Export(typeof(ISearchInfoService))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    [ServiceBehavior(IncludeExceptionDetailInFaults = true, ConcurrencyMode = ConcurrencyMode.Multiple)]
    internal class SearchInfoService : ISearchInfoService, IPartImportsSatisfiedNotification
    {
        [Import]
        private IModelInternal model;

        [Import]
        private ICallbackRepositoryInternal repository;

        [Import]
        private ICallbackDispatcherInternal dispatcher;


        public void Subscribe()
        {
            var callback = OperationContext.Current.GetCallbackChannel<ISearchCallback>();
            this.repository.AddCallback(callback);
        }

        public void Unsubscribe()
        {
            var callback = OperationContext.Current.GetCallbackChannel<ISearchCallback>();
            this.repository.RemoveCallback(callback);
        }


        private void OnResultAdded(object sender, ResultEventArgs args)
        {
            var builder = new ResultBuilder();
            var changes = builder.BuildAdds(args.Targets);

            this.Notify(x => x.OnResultChanged(changes));
        }

        private void OnResultChanged(object sender, ResultEventArgs args)
        {
            var builder = new ResultBuilder();
            var changes = builder.BuildChanges(args.Targets, args.Properties);

            this.Notify(x => x.OnResultChanged(changes));
        }

        private void OnResultRemoved(object sender, ResultEventArgs args)
        {
            var builder = new ResultBuilder();
            var changes = builder.BuildRemoves(args.Targets);

            this.Notify(x => x.OnResultChanged(changes));
        }

        private void Notify(Action<ISearchCallback> callbackAction)
        {
            this.dispatcher.Do(callbackAction);
        }


        void IPartImportsSatisfiedNotification.OnImportsSatisfied()
        {
            this.model.ResultAdded += this.OnResultAdded;
            this.model.ResultChanged += this.OnResultChanged;
            this.model.ResultRemoved += this.OnResultRemoved;
        }
    }
}
