﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using MDownloader.Search.Entities;
using MDownloader.Search.Entities.Transfer;

namespace MDownloader.Search.Server.Services.Implementation
{
    internal class ResultBuilder
    {
        public ResultChange[] BuildAdds(Result[] results)
        {
            var changes = new List<ResultChange>(results.Length);

            foreach (var result in results)
            {
                changes.Add(new ResultChange
                {
                    Change = ChangeType.Added,
                    SpecificationRef = new SpecificationRef
                    {
                        Id = result.GetSingle<Specification>().Id,
                    },
                    Ref = new ResultRef
                    {
                        Id = result.Identifier.Id
                    },
                    Properties = new Dictionary<string, string>
                    {
                        { ResultProperties.Name, result.Name },
                        { ResultProperties.Provider, result.Provider },
                        { ResultProperties.Files, result.Files.ToString() },
                        { ResultProperties.Size, result.Size.ToString(CultureInfo.InvariantCulture) },
                        { ResultProperties.SizeExactness, result.SizeExactness.ToString(CultureInfo.InvariantCulture) },
                        { ResultProperties.Status, result.Status.ToString() },
                    }
                });
            }

            return changes.ToArray();
        }

        public ResultChange[] BuildChanges(Result[] results, string[] properties)
        {
            var changes = new List<ResultChange>(results.Length);

            foreach (var result in results)
            {
                var linkChange = new ResultChange
                {
                    Change = ChangeType.Changed,
                    SpecificationRef = new SpecificationRef
                    {
                        Id = result.GetSingle<Specification>().Id,
                    },
                    Ref = new ResultRef
                    {
                        Id = result.Identifier.Id
                    },
                    Properties = new Dictionary<string, string>()
                };

                foreach (var property in properties)
                {
                    switch (property)
                    {
                        case ResultProperties.Name: linkChange.Properties.Add(ResultProperties.Name, result.Name); break;
                        case ResultProperties.Provider: linkChange.Properties.Add(ResultProperties.Provider, result.Provider); break;
                        case ResultProperties.Files: linkChange.Properties.Add(ResultProperties.Files, result.Files.ToString()); break;
                        case ResultProperties.Size: linkChange.Properties.Add(ResultProperties.Size, result.Size.ToString(CultureInfo.InvariantCulture)); break;
                        case ResultProperties.SizeExactness: linkChange.Properties.Add(ResultProperties.SizeExactness, result.SizeExactness.ToString(CultureInfo.InvariantCulture)); break;
                        case ResultProperties.Status: linkChange.Properties.Add(ResultProperties.Status, result.Status.ToString()); break;
                    }
                }

                changes.Add(linkChange);
            }

            return changes.ToArray();
        }


        public ResultChange[] BuildRemoves(Result[] results)
        {
            var changes = new List<ResultChange>(results.Length);

            foreach (var result in results)
            {
                changes.Add(new ResultChange
                {
                    Change = ChangeType.Removed,
                    SpecificationRef = new SpecificationRef
                    {
                        Id = result.GetSingle<Specification>().Id,
                    },
                    Ref = new ResultRef
                    {
                        Id = result.Identifier.Id
                    }
                });
            }

            return changes.ToArray();
        }
    }
}
