﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Text;
using MDownloader.Main.Interfaces;
using MDownloader.Search.Server.Services;
using MDownloader.Server.Services;
using MDownloader.Search.Server.Services.Endpoints;

namespace MDownloader.Search.Server.Services.Implementation
{
    [Export(typeof(IServiceHostFactory))]
    [PartCreationPolicy(CreationPolicy.Shared)]
    internal class ServiceHostFactory : IServiceHostFactory
    {
        [Import]
        private IServiceProvider serviceProvider;

        private ServiceHost searchEngineService;
        private ServiceHost searchInfoService;
        private ServiceHost searchScheduleService;

        public void Initialize()
        {
            this.searchEngineService = this.BuildHost<ISearchEngineService, SearchEngineService>(new Uri("net.tcp://localhost:4712/MDownloader/Search/Engine"));
            this.searchInfoService = this.BuildHost<ISearchInfoService, SearchInfoService>(new Uri("net.tcp://localhost:4712/MDownloader/Search/Info"));
            this.searchScheduleService = this.BuildHost<ISearchScheduleService, SearchScheduleService>(new Uri("net.tcp://localhost:4712/MDownloader/Search/Schedule"));
        }

        public void Start()
        {
            this.searchEngineService.Open();
            this.searchInfoService.Open();
            this.searchScheduleService.Open();
        }

        public void Stop()
        {
            this.searchEngineService.Close();
            this.searchInfoService.Close();
            this.searchScheduleService.Close();
        }

        private ServiceHost BuildHost<TContract, TService>(Uri baseAddress)
            where TService : class, TContract
        {
            var host = new ServiceHost(typeof(TService));

            ServiceHostFactory.ConfigureMetadataEndpoint<TContract>(host, baseAddress);
            host.AddServiceEndpoint(typeof(TContract), new NetTcpBinding(), baseAddress);
            host.Description.Behaviors.Add(new ServiceBehavior(typeof(TContract), this.serviceProvider));

            return host;
        }

        [Conditional("DEBUG")]
        private static void ConfigureMetadataEndpoint<TContract>(ServiceHost host, Uri baseAddress)
        {
            var metadata = new ServiceMetadataBehavior();
            var httpUri = new UriBuilder(baseAddress) { Scheme = "http", Port = 4713 };

            metadata.HttpGetEnabled = true;
            metadata.HttpGetUrl = httpUri.Uri;
            metadata.MetadataExporter.PolicyVersion = PolicyVersion.Policy15;

            host.Description.Behaviors.Add(metadata);
        }
    }
}
