﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.IO;
using System.Linq;
using System.Text;
using MDownloader.Main.Interfaces;

namespace MDownloader.Service
{
    [Export(typeof(IServiceProvider))]
    internal class Server : IServiceProvider
    {
        private static CompositionContainer container;

        static Server()
        {
            var catalog = new AggregateCatalog();
            var folder = Uri.UnescapeDataString(new Uri(typeof(Server).Assembly.CodeBase).AbsolutePath);

            catalog.Catalogs.Add(new AssemblyCatalog(typeof(Server).Assembly));
            catalog.Catalogs.Add(new DirectoryCatalog(Path.GetDirectoryName(folder), "MDownloader.*.Server.dll"));
            catalog.Catalogs.Add(new DirectoryCatalog(Path.GetDirectoryName(folder), "QImpl.*.dll"));

            Server.container = new CompositionContainer(catalog);
        }

        public void Start()
        {
            var downloaderEngine = Server.container.GetExportedValue<IEngine>();

            downloaderEngine.Initialize();
            downloaderEngine.Start();
        }

        #region IServiceProvider Members

        object IServiceProvider.GetService(Type serviceType)
        {
            var typeInfo = Server.container.GetType();
            var methodInfo = typeInfo.GetMethod("GetExportedValue", new Type[] { }).MakeGenericMethod(serviceType);
            var foundInstance = methodInfo.Invoke(Server.container, null);

            return foundInstance;
        }

        #endregion
    }
}
