﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking;
using EFramework.Networking.Parser;
using NUnit.Framework;
using QImpl.Core.Exceptions.UnexpectedData;

namespace QImpl.Core.Tests.Exceptions.UnexpectedData
{
    [TestFixture]
    public class ArtifactFactoryFixture
    {
        [Test]
        public void CanCreateInstance()
        {
            new ArtifactFactory();
        }

        [Test]
        public void CanCreateDocumentArtifact()
        {
            var document = HDocumentFactory.Create("<html></html>");
            var instance = new ArtifactFactory();
            var artifact = instance.Create(document);

            Assert.NotNull(artifact, "Created artifact should no be null.");
            Assert.NotNull(artifact.GetBytes(), "Artifact should be able to return bytes.");
        }

        [Test]
        public void CanCreateCallbackArtifact()
        {
            var callback = new NetworkCallback { { "abc", "cde" } };
            var instance = new ArtifactFactory();
            var artifact = instance.Create(callback);

            Assert.NotNull(artifact, "Created artifact should no be null.");
            Assert.NotNull(artifact.GetBytes(), "Artifact should be able to return bytes.");
        }

        [Test]
        public void CanCreateTextArtifact()
        {
            var text = "Abc";
            var instance = new ArtifactFactory();
            var artifact = instance.Create(text);

            Assert.NotNull(artifact, "Created artifact should no be null.");
            Assert.NotNull(artifact.GetBytes(), "Artifact should be able to return bytes.");
        }

        [Test]
        public void CanCreateBinaryArtifact()
        {
            var data = new byte[] { 0x00, 0x01, 0x02 };
            var instance = new ArtifactFactory();
            var artifact = instance.Create(data);

            Assert.NotNull(artifact, "Created artifact should no be null.");
            Assert.NotNull(artifact.GetBytes(), "Artifact should be able to return bytes.");
        }

        [Test]
        public void CanCreateStreamArtifact()
        {
            var data = new byte[] { 0x00, 0x01, 0x02 };
            using (var stream = new MemoryStream(data, false))
            {
                var instance = new ArtifactFactory();
                var artifact = instance.Create(data);

                Assert.NotNull(artifact, "Created artifact should no be null.");
                Assert.NotNull(artifact.GetBytes(), "Artifact should be able to return bytes.");
            }
        }

        [Test]
        public void CanLimitStreamArtifact()
        {
            var data = new byte[2048];
            using (var stream = new MemoryStream(data, false))
            {
                var instance = new ArtifactFactory();
                var artifact = instance.Create(data);

                Assert.NotNull(artifact, "Created artifact should no be null.");
                Assert.NotNull(artifact.GetBytes(), "Artifact should be able to return bytes.");
                Assert.LessOrEqual(1024, artifact.GetBytes().Length, "Artifact size should be not greater than 1024 bytes.");
            }
        }
    }
}
