﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using EFramework.Core.Contracts;
using MDownloader.Core.Server.Exceptions;
using QImpl.Core.Exceptions.UnexpectedData;

namespace QImpl.Core.Exceptions
{
    public static class UnexpectedDataExceptionFactory
    {
        public static UnexpectedDataException CreateException<TResponse>(params Expression<Func<ArtifactFactory, Artifact>>[] artifacts)
            where TResponse : class
        {
            Ensure.IsNotNull(() => artifacts);

            var data = new Dictionary<string, byte[]>();
            var factory = new ArtifactFactory();

            foreach (var artifact in artifacts)
            {
                Ensure.IsNotNull(() => artifact);

                var name = artifact.Parameters.First().Name;
                var compiled = artifact.Compile();
                var instance = compiled(factory);
                var bytes = instance.GetBytes();

                data.Add(name, bytes);
            }

            return new UnexpectedDataException(typeof(TResponse), data);
        }
    }
}
