﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using EFramework.Networking.Parser;
using MDownloader.Main.Interfaces.Base;
using QImpl.Core.Exceptions;

namespace QImpl.Responses.Document
{
    public abstract class DocumentParser<TResponse> : ResponseParserBase
        where TResponse : ResponsePartBase, new()
    {
        public TResponse Parse(HDocument document)
        {
            Ensure.IsNotNull(() => document);

            var result = new TResponse();

            this.Process(result, document);
            this.Validate(result, document);

            return result;
        }


        private void Process(TResponse response, HDocument document)
        {
            foreach (var processor in this.GetProcessors())
            {
                processor.Process(response, document);
            }
        }

        protected abstract IEnumerable<IDocumentProcessor<TResponse>> GetProcessors();

        private void Validate(TResponse response, HDocument _document)
        {
            if (response.Validate() == false)
            {
                throw UnexpectedDataExceptionFactory.CreateException<DocumentParser<TResponse>>(document => document.Create(_document));
            }
        }
    }
}
