﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Networking;
using QImpl.Filestube.Implementation.Enums;

namespace QImpl.Filestube.Implementation.Requests
{
    internal class SearchRequestBuilder
    {
        public NetworkRequest Get(string query, Hosting[] hosting = null, string extension = null, int? sizeFrom = null, int? sizeTo = null, Date? date = null, Sort? sort = null, int? page = null)
        {
            var getData = new NetworkGetData { { "q", query }, };

            this.AddHosting(getData, hosting);
            this.AddExtension(getData, extension);
            this.AddSizes(getData, sizeFrom, sizeTo);
            this.AddDate(getData, date);
            this.AddSort(getData, sort);
            this.AddPage(getData, page);

            return new NetworkRequest(@"http://www.filestube.com/search.html", Guid.Empty) { GetData = getData };
        }


        private void AddHosting(NetworkGetData getData, Hosting[] hosting)
        {
            if (hosting != null && hosting.Length > 0)
            {
                var query = from value in hosting
                            select (int)value;

                getData.Add("hosting", String.Join(",", query));
            }
        }

        private void AddExtension(NetworkGetData getData, string extension)
        {
            if (String.IsNullOrWhiteSpace(extension) == false)
            {
                getData.Add("select", extension);
            }
        }

        private void AddSizes(NetworkGetData getData, int? sizeFrom, int? sizeTo)
        {
            if (sizeFrom.HasValue == true)
            {
                getData.Add("sizefrom", sizeFrom.Value.ToString());
            }

            if (sizeTo.HasValue == true)
            {
                getData.Add("sizeto", sizeTo.Value.ToString());
            }
        }

        private void AddDate(NetworkGetData getData, Date? date)
        {
            if (date.HasValue == true && date.Value != Date.Whenever)
            {
                getData.Add("date", ((int)date.Value).ToString());
            }
        }

        private void AddSort(NetworkGetData getData, Sort? sort)
        {
            if (sort.HasValue == true && sort.Value != Sort.None)
            {
                switch (sort.Value)
                {
                    case Sort.DateAscending: getData.Add("sort", "da"); break;
                    case Sort.DateDescending: getData.Add("sort", "dd"); break;
                    case Sort.PopularityAscending: getData.Add("sort", "pa"); break;
                    case Sort.PopularityDescending: getData.Add("sort", "pd"); break;
                    case Sort.SizeAscending: getData.Add("sort", "sa"); break;
                    case Sort.SizeDescending: getData.Add("sort", "sd"); break;
                }
            }
        }

        private void AddPage(NetworkGetData getData, int? page)
        {
            if (page.HasValue == true && page.Value > 1)
            {
                getData.Add("page", page.Value.ToString());
            }
        }
    }
}
