﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking.Parser;
using MDownloader.Core.Server.Exceptions;
using MDownloader.Main.Interfaces.Exceptions;
using NUnit.Framework;
using QImpl.Hotfile.Implementation;

namespace QImpl.Hotfile.Tests.Implementation.Responses
{
    [TestFixture]
    public class CaptchaResponseParserFixture
    {
        [Test]
        public void CanParseTypicalResponse()
        {
            var document = this.GetDocument("Captcha.Typical.txt");
            var response = Parser.Captcha.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.IsTrue(response.HasDownloadLink, "The response should be classified as download.");
            Assert.AreEqual(@"http://hotfile.com/get/141038305/4f0dda20/f859081/96kAppleyard.part1.rar", response.DownloadLink, "The download link should be properly set.");

            Assert.IsFalse(response.HasFailure, "The response should not be classified as failure.");
        }

        [Test]
        public void CanParseFailureResponse()
        {
            var document = this.GetDocument("Captcha.Failure.txt");
            var response = Parser.Captcha.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.IsTrue(response.HasFailure, "The response should be classified as failure.");
            Assert.IsFalse(response.HasDownloadLink, "The response should not be classified as download.");
        }
        
        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardDocumentNullParameter()
        {
            Parser.Captcha.Parse(default(HDocument));
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanDetectUnexpectedData()
        {
            var document = this.GetDocument("Unexpected.Hotfile.txt");
            Parser.Captcha.Parse(document);
        }
        
        
        private HDocument GetDocument(string resourceName)
        {
            var path = String.Format("{0}.Resources.{1}", this.GetType().Namespace, resourceName);

            using (var stream = this.GetType().Assembly.GetManifestResourceStream(path))
            using (var reader = new StreamReader(stream))
            {
                return HDocumentFactory.Create(reader.ReadToEnd());
            }
        }
    }
}
