﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking.Parser;
using MDownloader.Core.Server.Exceptions;
using NUnit.Framework;
using QImpl.Hotfile.Implementation;
using QImpl.Hotfile.Implementation.Responses;

namespace QImpl.Hotfile.Tests.Implementation.Responses
{
    [TestFixture]
    public class CheckFilesResponseParserFixture
    {
        [Test]
        public void CanParseResponseWithValidFile()
        {
            var document = this.GetDocument("CheckFiles.Valid.txt");
            var response = Parser.CheckFiles.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.NotNull(response.Files, "The list of files should always be set.");
            Assert.AreEqual(1, response.Files.Count, "The list of files should contains exactly one item.");

            var file = response.Files[0];

            Assert.AreEqual("137784874-cdee641", file.Id, "The file identifier should be properly set.");
            Assert.AreEqual("Merlin.S04E12.hdtv.xvid-river.avi", file.Name, "The file name should be properly set.");
            Assert.AreEqual(Math.Round(349.2 * 1024 * 1024), file.Size, "The file size should be properly set.");
            Assert.AreEqual(CheckFilesResponse.Status.FileOk, file.Status, "The file status should be properly set.");
        }

        [Test]
        public void CanParseResponseWithInvalidFile()
        {
            var document = this.GetDocument("CheckFiles.Invalid.txt");
            var response = Parser.CheckFiles.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.NotNull(response.Files, "The list of files should always be set.");
            Assert.AreEqual(1, response.Files.Count, "The list of files should contains exactly one item.");

            var file = response.Files[0];

            Assert.AreEqual("94598566-b6e9801", file.Id, "The file identifier should be properly set.");
            Assert.AreEqual("Le_Pianiste_.part1.rar", file.Name, "The file name should be properly set.");
            Assert.IsNull(file.Size, "The file size should be null.");
            Assert.AreEqual(CheckFilesResponse.Status.NotFound, file.Status, "The file status should be properly set.");
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanDetectUnexpectedData()
        {
            var document = this.GetDocument("Unexpected.Hotfile.txt");
            Parser.CheckFiles.Parse(document);
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullDocument()
        {
            Parser.CheckFiles.Parse(null);
        }


        private HDocument GetDocument(string resourceName)
        {
            var path = String.Format("{0}.Resources.{1}", this.GetType().Namespace, resourceName);

            using (var stream = this.GetType().Assembly.GetManifestResourceStream(path))
            using (var reader = new StreamReader(stream))
            {
                return HDocumentFactory.Create(reader.ReadToEnd());
            }
        }
    }
}
