﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking.Parser;
using MDownloader.Core.Server.Exceptions;
using NUnit.Framework;
using QImpl.Hotfile.Implementation;

namespace QImpl.Hotfile.Tests.Implementation.Responses
{
    [TestFixture]
    public class DownloadResponseParserFixture
    {
        [Test]
        public void CanParseTypicalResponse()
        {
            var document = this.GetDocument("Download.Typical.txt");
            var response = Parser.Download.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.IsTrue(response.HasCaptcha, "The response should be classified as captcha.");
            Assert.AreEqual(@"http://api.recaptcha.net/challenge?k=6LfRJwkAAAAAAGmA3mAiAcAsRsWvfkBijaZWEvkD", response.CaptchaLink, "The captcha link should be properly set.");
            Assert.NotNull(response.CaptchaForm, "The captcha form should not be null.");

            Assert.IsFalse(response.HasDownloadLink, "The response should not be classified as download.");
            Assert.IsNullOrEmpty(response.DownloadLink, "The download link should not be set.");
        }

        [Test]
        public void CanParseFirstTimeResponse()
        {
            var document = this.GetDocument("Download.FirstTime.txt");
            var response = Parser.Download.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.IsTrue(response.HasDownloadLink, "The response should be classified as download.");
            Assert.AreEqual(@"http://hotfile.com/get/141038305/4f0dcaf0/31a83ae/96kAppleyard.part1.rar", response.DownloadLink, "The download link should be properly set.");

            Assert.IsFalse(response.HasCaptcha, "The response should not be classified as captcha.");
            Assert.IsNullOrEmpty(response.CaptchaLink, "The captcha link should not be set.");
            Assert.IsNull(response.CaptchaForm, "The captcha for should not be set.");
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardDocumentNullParameter()
        {
            Parser.Download.Parse(default(HDocument));
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanDetectUnexpectedData()
        {
            var document = this.GetDocument("Unexpected.Hotfile.txt");
            Parser.Download.Parse(document);
        }


        private HDocument GetDocument(string resourceName)
        {
            var path = String.Format("{0}.Resources.{1}", this.GetType().Namespace, resourceName);

            using (var stream = this.GetType().Assembly.GetManifestResourceStream(path))
            using (var reader = new StreamReader(stream))
            {
                return HDocumentFactory.Create(reader.ReadToEnd());
            }
        }
    }
}
