﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking.Parser;
using MDownloader.Core.Server.Exceptions;
using NUnit.Framework;
using QImpl.Hotfile.Implementation;

namespace QImpl.Hotfile.Tests.Implementation.Responses
{
    [TestFixture]
    public class FileInfoResponseParserFixture
    {
        [Test]
        public void CanParseTypicalResponse()
        {
            var document = this.GetDocument("FileInfo.Typical.txt");
            var response = Parser.FileInfo.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.AreEqual(@"96kAppleyard.part1.rar", response.Name, "The name should be properly set.");
            Assert.AreEqual(Math.Round(214.6 * 1024 * 1024), response.Size, "The size should be properly set.");
            Assert.AreEqual(15, response.Wait, "The timer should be properly set.");

            Assert.NotNull(response.Form, "The download form should not be null.");
            Assert.AreEqual(@"capt", response.Form.action, "Form field 'action' should be properly set.");
            Assert.AreEqual(@"1326303833", response.Form.tm, "Form field 'tm' should be properly set.");
            Assert.AreEqual(@"0c34e139ca2e97baca69dada12970999c49a25a4", response.Form.tmhash, "Form field 'tmhash' should be properly set.");
            Assert.AreEqual(@"15", response.Form.wait, "Form field 'wait' should be properly set.");
            Assert.AreEqual(@"349ebb26df7600048ed11dfbbee2adf7cfcf3740", response.Form.waithash, "Form field 'waithash' should be properly set.");
            Assert.AreEqual(@"38cb59b5d5839003f7ecc46551ccd632cd0fe2ef", response.Form.upidhash, "Form field 'upidhash' should be properly set.");
        }

        [Test]
        public void CanParseWaitingResponse()
        {
            var document = this.GetDocument("FileInfo.Waiting.txt");
            var response = Parser.FileInfo.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.AreEqual(@"96kAppleyard.part1.rar", response.Name, "The name should be properly set.");
            Assert.AreEqual(Math.Round(214.6 * 1024 * 1024), response.Size, "The size should be properly set.");
            Assert.AreEqual(1810, response.Wait, "The timer should be properly set.");

            Assert.NotNull(response.Form, "The download form should not be null.");
            Assert.AreEqual(@"capt", response.Form.action, "Form field 'action' should be properly set.");
            Assert.AreEqual(@"1326305894", response.Form.tm, "Form field 'tm' should be properly set.");
            Assert.AreEqual(@"da5021610abf90d6902bb4e78ead803fa3392332", response.Form.tmhash, "Form field 'tmhash' should be properly set.");
            Assert.AreEqual(@"15", response.Form.wait, "Form field 'wait' should be properly set.");
            Assert.AreEqual(@"ab9d35f9e94b3b6f4c5442d03347fa07dc073f81", response.Form.waithash, "Form field 'waithash' should be properly set.");
            Assert.AreEqual(@"186ade083d342505899479391ed1f515a9f9a855", response.Form.upidhash, "Form field 'upidhash' should be properly set.");
        }

        [Test]
        public void CanParseParallelDownloadingResponse()
        {
            var document = this.GetDocument("FileInfo.Parallel.txt");
            var response = Parser.FileInfo.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.AreEqual(@"96kFriedAqua.part1.rar", response.Name, "The name should be properly set.");
            Assert.AreEqual(Math.Round(214.6 * 1024 * 1024), response.Size, "The size should be properly set.");
            Assert.AreEqual(7215, response.Wait, "The timer should be properly set.");

            Assert.NotNull(response.Form, "The download form should not be null.");
            Assert.AreEqual(@"capt", response.Form.action, "Form field 'action' should be properly set.");
            Assert.AreEqual(@"1326626575", response.Form.tm, "Form field 'tm' should be properly set.");
            Assert.AreEqual(@"3d425ee303977b27d2837bff9332cd9255428c7c", response.Form.tmhash, "Form field 'tmhash' should be properly set.");
            Assert.AreEqual(@"15", response.Form.wait, "Form field 'wait' should be properly set.");
            Assert.AreEqual(@"07bae13583c60f18afaf8864ef6dcad0eab9b2d5", response.Form.waithash, "Form field 'waithash' should be properly set.");
            Assert.AreEqual(@"709c1158b5578eae06fa7947800d3ede39a67da8", response.Form.upidhash, "Form field 'upidhash' should be properly set.");
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullDocument()
        {
            Parser.FileInfo.Parse(null);
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanDetectUnexpectedData()
        {
            var document = this.GetDocument("Unexpected.Hotfile.txt");
            Parser.FileInfo.Parse(document);
        }


        private HDocument GetDocument(string resourceName)
        {
            var path = String.Format("{0}.Resources.{1}", this.GetType().Namespace, resourceName);

            using (var stream = this.GetType().Assembly.GetManifestResourceStream(path))
            using (var reader = new StreamReader(stream))
            {
                return HDocumentFactory.Create(reader.ReadToEnd());
            }
        }
    }
}
