﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using MDownloader.Core.Server;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Interfaces.Base.Feedbacks;
using QImpl.Hotfile.Implementation.Forms;
using QImpl.Hotfile.Implementation.Responses;

namespace QImpl.Hotfile.Implementation
{
    internal class Data : IProviderResourceData
    {
        private readonly IBrowserSession session;
        private readonly IRecaptcha recaptcha;

        private readonly string link;
        private readonly DownloadForm form;

        private IRecaptchaSession recaptchaSession;
        private Stream stream;
        private string attachment;
        private long size;


        public Data(IBrowserSession session, IRecaptcha recaptcha, string link, DownloadForm form)
        {
            this.session = session;
            this.recaptcha = recaptcha;
            this.link = link;
            this.form = form;
        }


        public string Attachment
        {
            get { return this.attachment; }
        }

        public long TotalSize
        {
            get { return this.size; }
        }

        public long Offset
        {
            get { return 0L; }
        }

        public long Count
        {
            get { return this.size; }
        }


        public void Initialize()
        {
            var request = Builder.Download.Get(this.link, this.form);
            var page = session.GetPage(request);
            var response = Parser.Download.Parse(page);

            if (response.HasCaptcha == true)
            {
                this.recaptchaSession = this.recaptcha.StartSession(this.session, response.CaptchaLink);
            }
            else
            {
            }
        }

        public IProviderFeedback GetFeedback()
        {
            if (this.recaptchaSession != null)
            {
                var text = String.Empty;
                var captcha = new Captcha(this.recaptchaSession, x => text = x);

                return new CaptchaFeedback(captcha, () => this.OnCaptchaSatisfied(text) );
            }

            return null;
        }

        private void OnCaptchaSatisfied(string captcha)
        {
            Ensure.IsNotNullNorEmpty(() => captcha);

            var challenge = this.recaptchaSession.GetChallenge();
            var request = Builder.Captcha.GetRequest(this.link, challenge, captcha);
            var page = session.GetPage(request);
            var response = Parser.Captcha.Parse(page);
        }


        public Stream OpenStream()
        {
            return this.stream;
        }

        public void Dispose()
        {
            if (this.stream != null)
            {
                this.stream.Close();
                this.stream.Dispose();
                this.stream = null;
            }
        }
    }
}
