﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using EFramework.Core.Contracts;
using MDownloader.Core.Server;
using MDownloader.Main.Interfaces;
using MDownloader.Main.Interfaces.Base.Feedbacks;
using QImpl.Hotfile.Implementation.Forms;
using QImpl.Hotfile.Implementation.Responses;

namespace QImpl.Hotfile.Implementation
{
    internal class Resource : IProviderResource
    {
        private readonly IBrowserSession session;
        private readonly IRecaptcha recaptcha;
        private readonly string link;

        private DownloadForm form;
        private int wait;


        public Resource(IBrowserSession session, IRecaptcha recaptcha, string link)
        {
            Ensure.IsNotNull(() => session);
            Ensure.IsNotNull(() => recaptcha);
            Ensure.IsNotNullNorEmpty(() => link);

            this.session = session;
            this.recaptcha = recaptcha;
            this.link = link;
        }


        public string Name { get; private set; }
        public string Description { get; private set; }

        public long Size { get; private set; }
        public bool SizeExactness
        {
            get { return false; }
        }


        public void Initialize()
        {
            var request = Builder.FileInfo.Get(this.link);
            var page = session.GetPage(request);
            var response = Parser.FileInfo.Parse(page);

            this.Name = response.Name;
            this.Size = response.Size;
            this.form = response.Form;
            this.wait = response.Wait;
        }

        public IProviderFeedback GetFeedback()
        {
            if (this.wait > 0)
            {
                return new WaitFeedback(TimeSpan.FromSeconds(this.wait), () =>
                {
                    this.wait = 0;
                });
            }

            return null;
        }

        public IProviderResourceData GetData()
        {
            return new Data(this.session, this.recaptcha, this.link, this.form);
        }
    }
}
