﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using EFramework.Networking.Parser;
using QImpl.Hotfile.Implementation.Forms;
using QImpl.Hotfile.Implementation.Responses.Interfaces;
using QImpl.Responses.Document;

namespace QImpl.Hotfile.Implementation.Responses.Processors
{
    /// <summary>
    /// A process to find captcha form at the page.
    /// </summary>
    internal class CaptchaProcessor : DocumentProcessorBase<ICaptchaResponsePart, HTag>
    {
        protected override Func<HDocument, HTag> GetQuery()
        {
            return document => HQuery.On(document).Execute("div#main_content form").FirstOrDefault();
        }

        protected override void Execute(ICaptchaResponsePart response, HTag element)
        {
            Ensure.IsNotNull(() => response);
            Ensure.IsNotNull(() => element);

            response.CaptchaLink = element.Descendants<HScript>().Select(x => x.Attributes[HAttributeType.Source]).FirstOrDefault();
            response.CaptchaForm = new CaptchaForm
            {
                action = HQuery.On(element).Execute("input[name='action']").Select(x => x.Attributes[HAttributeType.Value]).FirstOrDefault(),
                recaptcha_challenge_field = HQuery.On(element).Execute("textarea[name='recaptcha_challenge_field']").SelectMany(x => x.Descendants<HText>()).Select(x => x.Value).FirstOrDefault(),
                recaptcha_response_field = HQuery.On(element).Execute("input[name='recaptcha_response_field']").Select(x => x.Attributes[HAttributeType.Value]).FirstOrDefault(),
            };
        }
    }
}
