﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using EFramework.Core.Contracts;
using EFramework.Networking.Parser;
using QImpl.Hotfile.Implementation.Responses.Interfaces;
using QImpl.Responses.Document;

namespace QImpl.Hotfile.Implementation.Responses.Processors
{
    /// <summary>
    /// A processor to find file information at the page.
    /// </summary>
    internal class InfoProcessor : DocumentProcessorBase<IInfoResponsePart, HTag>
    {
        private readonly Regex sizeRegex = new Regex("^(?<size>[0-9]+(\\.[0-9]+)?)\\s*(?<multiplier>(kB|Mb|Gb))$", RegexOptions.IgnoreCase);


        protected override Func<HDocument, HTag> GetQuery()
        {
            return document => HQuery.On(document).Execute("table#download_file div.arrow_down").FirstOrDefault();
        }

        protected override void Execute(IInfoResponsePart response, HTag element)
        {
            Ensure.IsNotNull(() => response);
            Ensure.IsNotNull(() => element);

            var parts = element.Descendants<HText>().Select(x => x.Value).ToArray();
            var name = parts.ElementAt(1);
            var size = parts.ElementAt(3);

            if (String.IsNullOrWhiteSpace(name) == false)
            {
                response.Name = name.Trim();
            }

            if (String.IsNullOrWhiteSpace(size) == false)
            {
                var match = this.sizeRegex.Match(size.TrimEnd());
                if (match.Success == true)
                {
                    var sizeText = match.Groups["size"].Value;
                    var sizeValue = Decimal.Parse(sizeText, NumberStyles.AllowDecimalPoint, new NumberFormatInfo { NumberDecimalSeparator = "." });
                    var multiplierText = match.Groups["multiplier"].Value;
                    var multiplierValue = 1;

                    switch (multiplierText.ToLower())
                    {
                        case "kb": multiplierValue = 1024; break;
                        case "mb": multiplierValue = 1024 * 1024; break;
                        case "gb": multiplierValue = 1024 * 1024 * 1024; break;
                    }

                    response.Size = (long)Math.Round(multiplierValue * sizeValue);
                }
            }
        }
    }
}
