﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using EFramework.Core.Contracts;
using EFramework.Networking.Parser;
using QImpl.Hotfile.Implementation.Responses.Interfaces;
using QImpl.Responses.Document;

namespace QImpl.Hotfile.Implementation.Responses.Processors
{
    /// <summary>
    /// A processor to find waiting period at the page.
    /// </summary>
    internal class WaitProcessor : DocumentProcessorBase<IWaitResponsePart, IEnumerable<HScript>>
    {
        private readonly Regex waitRegex = new Regex("timerend=d\\.getTime\\(\\)\\+(?<wait>[0-9]+);", RegexOptions.IgnoreCase);


        protected override Func<HDocument, IEnumerable<HScript>> GetQuery()
        {
            return document => document.Descendants<HScript>();
        }

        protected override void Execute(IWaitResponsePart response, IEnumerable<HScript> scripts)
        {
            Ensure.IsNotNull(() => response);
            Ensure.IsNotNull(() => scripts);

            var query = from script in scripts
                        where String.IsNullOrWhiteSpace(script.Content) == false
                        from line in script.Content.Split(Environment.NewLine.ToCharArray(), StringSplitOptions.RemoveEmptyEntries)
                        let match = this.waitRegex.Match(line)
                        where match.Success == true
                        select Int32.Parse(match.Groups["wait"].Value);

            var waiters = query.ToArray();
            if (waiters.Any() == true)
            {
                response.Wait = waiters.Sum() / 1000;
            }
        }
    }
}
