﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using MDownloader.Core.Server;
using MDownloader.Main.Interfaces;

namespace QImpl.Hotfile.Implementation
{
    internal class Session : IProviderSession
    {
        private readonly IBrowser browser;
        private readonly IRecaptcha recaptcha;


        public Session(IBrowser browser, IRecaptcha recaptcha)
        {
            Ensure.IsNotNull(() => browser);
            Ensure.IsNotNull(() => recaptcha);

            this.browser = browser;
            this.recaptcha = recaptcha;
        }


        public ICollection<IProviderLink> Analyze(ICollection<string> links)
        {
            var result = new List<IProviderLink>(links.Count);
            var session = this.browser.OpenSession();

            var parser = new UrlParser();
            var builder = new UrlBuilder();

            var request = Builder.CheckFiles.Get(links.ToArray());
            var page = session.GetPage(request);
            var response = Parser.CheckFiles.Parse(page);

            foreach (var file in response.Files)
            {
                result.Add(new Link
                {
                    Id = file.Id,
                    Target = builder.BuildLink(file.Id, file.Name),
                    Status = file.Status.ToString(),
                    Name = file.Name,
                    Description = String.Empty,
                    Size = file.Size.HasValue ? file.Size.Value : 0,
                    SizeExactness = false,
                    IsValid = file.Status != Responses.CheckFilesResponse.Status.NotFound,
                    IsAvailable = file.Status == Responses.CheckFilesResponse.Status.FileOk,
                });
            }

            return result;
        }

        public IProviderResource Download(string link)
        {
            return new Resource(browser.OpenSession(), this.recaptcha, link);
        }

        public void Dispose()
        {
        }
    }
}
