﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Networking;
using NUnit.Framework;
using QImpl.Megaupload.Implementation;
using QImpl.Megaupload.Implementation.Responses;

namespace QImpl.Megaupload.Tests.Implementation.Responses
{
    [TestFixture]
    public class CheckFilesResponseParserFixture
    {
        [Test]
        public void CanParseResponseWithTwoResult()
        {
            var text = @"0=www.megaupload.com&1=www.megaporn.com&id0=0&s=209715200&d=0&n=Merlin.2008.S04E13.REPACK.HDTV.XviD-TLA2.part1.rar&id1=3";
            var postData = new NetworkPostData
            {
                { "id0", "RT8P2EI8" },
                { "id1", "FSI7FZWG" },
            };

            var response = Parser.CheckFiles.Parse(postData, text);

            Assert.NotNull(response, "The response can never be null.");
            Assert.NotNull(response.Files, "The list of files should always be set.");
            Assert.AreEqual(2, response.Files.Count, "The list of files should contains exactly two item.");

            var file1 = response.Files[0];
            var file2 = response.Files[1];

            Assert.AreEqual("RT8P2EI8", file1.Id, "The file identifier should be properly set.");
            Assert.AreEqual("Merlin.2008.S04E13.REPACK.HDTV.XviD-TLA2.part1.rar", file1.Name, "The file name should be properly set.");
            Assert.AreEqual(209715200, file1.Size, "The file size should be properly set.");
            Assert.AreEqual(CheckFilesResponse.Status.FileOk, file1.Status, "The file status should be properly set.");

            Assert.AreEqual("FSI7FZWG", file2.Id, "The file identifier should be properly set.");
            Assert.IsNullOrEmpty(file2.Name, "The file name should be not set.");
            Assert.IsNull(file2.Size, "The file size not be set.");
            Assert.AreEqual(CheckFilesResponse.Status.NotFound, file2.Status, "The file status should be properly set.");
        }

        [Test]
        public void CanHandleEmptyResult()
        {
            var text = String.Empty;
            var postData = new NetworkPostData
            {
                { "id0", "RT8P2EI8" },
                { "id1", "FSI7FZWG" },
            };

            var response = Parser.CheckFiles.Parse(postData, text);

            Assert.NotNull(response, "The response can never be null.");
            Assert.NotNull(response.Files, "The list of files should always be set.");
            Assert.AreEqual(0, response.Files.Count, "The list of files should contains no items.");
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardTextParameter()
        {
            Parser.CheckFiles.Parse(new NetworkPostData(), null);
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardPostDataParameter()
        {
            Parser.CheckFiles.Parse(null, String.Empty);
        }
    }
}
