﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking.Parser;
using MDownloader.Core.Server.Exceptions;
using MDownloader.Main.Interfaces.Exceptions;
using NUnit.Framework;
using QImpl.Megaupload.Implementation;
using QImpl.Megaupload.Implementation.Exceptions;
using QImpl.Megaupload.Implementation.Responses;

namespace QImpl.Megaupload.Tests.Implementation.Responses
{
    [TestFixture]
    public class FileInfoResponseParserFixture
    {
        [Test]
        public void CanParseTypicalResponse()
        {
            var document = this.GetDocument("FileInfo.Typical.txt");
            var response = Parser.FileInfo.Parse(document);

            Assert.NotNull(response, "The response can never be null.");
            Assert.AreEqual(@"BD82RTB2111489.rar", response.Name, "The name should be properly set.");
            Assert.AreEqual(Math.Round(213.93 * 1024 * 1024), response.Size, "The size should be properly set.");
            Assert.AreEqual(@"http://www888.megaupload.com/files/2da7b31efa5deb50dbebb0eb1ec52be4/BD82RTB2111489.rar", response.Link, "The download link should be properly set.");
        }

        [Test, ExpectedException(typeof(MegauploadException))]
        public void CanParseResponseWithNotAvailableLink()
        {
            var document = this.GetDocument("FileInfo.LinkNotAvailable.txt");
            Parser.FileInfo.Parse(document);
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullDocument()
        {
            Parser.FileInfo.Parse(null);
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanDetectUnexpectedData()
        {
            var document = this.GetDocument("Unexpected.Megaupload.txt");
            Parser.FileInfo.Parse(document);
        }


        private HDocument GetDocument(string resourceName)
        {
            var path = String.Format("{0}.Resources.{1}", this.GetType().Namespace, resourceName);

            using (var stream = this.GetType().Assembly.GetManifestResourceStream(path))
            using (var reader = new StreamReader(stream))
            {
                return HDocumentFactory.Create(reader.ReadToEnd());
            }
        }
    }
}
