﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking;
using MDownloader.Core.Server.Exceptions;
using NUnit.Framework;
using QImpl.Megaupload.Implementation;
using QImpl.Megaupload.Implementation.Exceptions;

namespace QImpl.Megaupload.Tests.Implementation.Responses
{
    [TestFixture]
    public class StreamResponseParserFixture
    {
        [Test]
        public void CanHandleValidResponse()
        {
            var stream = new MemoryStream(Encoding.UTF8.GetBytes("Abc"));
            var callback = new NetworkCallback
            {
                { "ETag", "G0OLDP9X" },
                { "Content-Disposition", "attachment" },
                { "Content-Type", "application/octet-stream" },
                { "Content-Length", "3" },
            };

            var response = Parser.Stream.Parse(stream, callback);

            Assert.NotNull(response, "The response can never be null.");
            Assert.AreEqual(String.Empty, response.Attachment, "The attachment should be properly set.");
            Assert.AreEqual(3, response.Size, "The stream size should be correctly set.");
            Assert.NotNull(response.Data, "The stream should be not null.");
        }

        [Test, ExpectedException(typeof(MegauploadException))]
        public void CanDetectDownloadLimitExceededMessage()
        {
            var stream = new MemoryStream(Encoding.UTF8.GetBytes("<HTML><HEAD><META HTTP-EQUIV=\"Refresh\" CONTENT=\"1; url=http://www.megaupload.com/?c=premium&l=1\"></HEAD><BODY>Download limit exceeded</BODY></HTML>"));
            var callback = new NetworkCallback
            {
                { "Content-Type", "text/html" },
                { "Retry-After", "120" },
            };

            Parser.Stream.Parse(stream, callback);
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanDetectUnexpectedData()
        {
            var stream = this.GetDocument("Unexpected.Megaupload.txt");
            var callback = new NetworkCallback
            {
                { "Content-Type", "text/html" },
                { "Retry-After", "120" },
            };

            Parser.Stream.Parse(stream, callback);
        }


        private Stream GetDocument(string resourceName)
        {
            var path = String.Format("{0}.Resources.{1}", this.GetType().Namespace, resourceName);
            var stream = this.GetType().Assembly.GetManifestResourceStream(path);

            return stream;
        }
    }
}
