﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using EFramework.Networking;
using MDownloader.Main.Interfaces.Base;

namespace QImpl.Megaupload.Implementation.Responses
{
    internal class CheckFilesResponseParser
    {
        public CheckFilesResponse Parse(NetworkPostData postData, string text)
        {
            Ensure.IsNotNull(() => postData);
            Ensure.IsNotNull(() => text);

            return new CheckFilesResponse
            {
                Files = this.ParseEntries(postData, text),
            };
        }


        private List<CheckFilesResponse.Entry> ParseEntries(NetworkPostData postData, string text)
        {
            Ensure.IsNotNull(() => postData);
            Ensure.IsNotNull(() => text);

            var parameters = UrlHelper.GetQueryParameters(text).ToArray();
            var result = new List<CheckFilesResponse.Entry>(parameters.Length);
            var current = default(CheckFilesResponse.Entry);

            foreach (var parameter in parameters)
            {
                if (parameter.Key.StartsWith("id", StringComparison.InvariantCultureIgnoreCase))
                {
                    var id = postData[parameter.Key];
                    if (String.IsNullOrWhiteSpace(id) == true)
                    {
                        current = null;
                    }
                    else
                    {
                        current = new CheckFilesResponse.Entry { Id = id, Status = parameter.Value.Equals("0") ? CheckFilesResponse.Status.FileOk : CheckFilesResponse.Status.NotFound };
                        result.Add(current);
                    }
                }

                if (parameter.Key.StartsWith("s", StringComparison.InvariantCultureIgnoreCase) && current != null)
                {
                    long size;
                    if (Int64.TryParse(parameter.Value, out size))
                    {
                        current.Size = size;
                    }
                }

                if (parameter.Key.StartsWith("n", StringComparison.InvariantCultureIgnoreCase) && current != null)
                {
                    current.Name = parameter.Value;
                }
            }

            return result;
        }
    }
}
