﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MDownloader.Core.Server;
using MDownloader.Main.Interfaces;

namespace QImpl.Megaupload.Implementation
{
    internal class Session : IProviderSession
    {
        private readonly IBrowser browser;


        public Session(IBrowser browser)
        {
            this.browser = browser;
        }


        public ICollection<IProviderLink> Analyze(ICollection<string> links)
        {
            var result = new List<IProviderLink>(links.Count);
            var session = this.browser.OpenSession();

            var parser = new UrlParser();
            var builder = new UrlBuilder();

            var ids = parser.GetIds(links);
            var request = Builder.CheckFiles.Get(ids);
            var text = session.GetText(request);
            var response = Parser.CheckFiles.Parse(request.PostData, text);

            foreach (var file in response.Files)
            {
                result.Add(new Link
                {
                    Id = file.Id,
                    Target = builder.BuildLink(file.Id),
                    Status = file.Status.ToString(),
                    Name = file.Name,
                    Description = String.Empty,
                    Size = file.Size.HasValue ? file.Size.Value : 0,
                    SizeExactness = true,
                    IsValid = file.Status != Responses.CheckFilesResponse.Status.NotFound,
                    IsAvailable = file.Status == Responses.CheckFilesResponse.Status.FileOk,
                });
            }

            return result;

        }

        public IProviderResource Download(string link)
        {
            return new Resource(browser.OpenSession(), link);
        }

        public void Dispose()
        {
        }
    }
}
