﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using MDownloader.Main.Interfaces.Base;

namespace QImpl.Megaupload.Implementation
{
    // http://www.megaupload.com/?d=0RY4VREB

    internal class UrlParser : UrlParserBase
    {
        private static readonly Regex ResourceIdRegex = new Regex("^[0-9A-Z]{8}$");


        protected override bool IsValidHost(Uri uri)
        {
            if (uri.Host.Equals("megaupload.com", StringComparison.InvariantCultureIgnoreCase))
                return true;

            if (uri.Host.EndsWith(".megaupload.com", StringComparison.InvariantCultureIgnoreCase))
                return true;

            return false;
        }

        protected override bool IsValidPath(Uri uri)
        {
            if (uri.Segments.Length == 1)
            {
                if (!uri.Segments[0].Equals("/", StringComparison.InvariantCultureIgnoreCase))
                    return false;

                return true;
            }

            return false;
        }

        protected override bool IsValidQuery(Uri uri)
        {
            var parameters = UrlHelper.GetQueryParameters(uri).ToArray();

            if (parameters.Length == 1 && parameters[0].Key.Equals("d", StringComparison.InvariantCultureIgnoreCase))
            {
                if (UrlParser.ResourceIdRegex.IsMatch(parameters[0].Value) == false)
                    return false;

                return true;
            }

            return false;
        }


        public string GetId(string link)
        {
            var query = from parameter in UrlHelper.GetQueryParameters(new Uri(link, UriKind.Absolute))
                        where UrlParser.ResourceIdRegex.IsMatch(parameter.Value)
                        select parameter.Value;

            return query.First();
        }

        public string[] GetIds(IEnumerable<string> links)
        {
            var query = from link in links
                        select this.GetId(link);

            return query.ToArray();
        }
    }
}
