﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NUnit.Framework;
using QImpl.Rapidshare.Implementation;
using QImpl.Rapidshare.Implementation.Responses;

namespace QImpl.Rapidshare.Tests.Implementation.Responses
{
    [TestFixture]
    public class CheckFilesResponseParserFixture
    {
        [Test]
        public void CanParseResponseWithOneResult()
        {
            var text = @"1329005725,ses_so_mat_to.part7.rar,77494336,239,1,dt,0C21F2A09E838621DAC9E99FDF6B7FA0";
            var response = Parser.CheckFiles.Parse(text);

            Assert.NotNull(response, "The response can never be null.");
            Assert.NotNull(response.Files, "The list of files should always be set.");
            Assert.AreEqual(1, response.Files.Count, "The list of files should contains exactly one item.");

            var file = response.Files[0];

            Assert.AreEqual("1329005725", file.Id, "File id should be properly set.");
            Assert.AreEqual("ses_so_mat_to.part7.rar", file.Name, "File name should be properly set.");
            Assert.AreEqual(77494336, file.Size, "File size should be propertly set.");
            Assert.AreEqual("239", file.Server, "Server id should be properly set.");
            Assert.AreEqual(CheckFilesResponse.Status.FileOk, file.Status, "File status should be properly set.");
            Assert.AreEqual("dt", file.Host, "Server name should be property set.");
            Assert.AreEqual("0C21F2A09E838621DAC9E99FDF6B7FA0", file.MD5, "File hash should be properly set.");
        }

        [Test]
        public void CanParseResponseWithMoreResults()
        {
            var text = "1329005725,ses_so_mat_to.part7.rar,77494336,239,1,tl4,0C21F2A09E838621DAC9E99FDF6B7FA0\r"
                     + "487830256,ses_so_mat_to.part6.rar,209000000,699,1,tl3,5EA4CF464AA579387BC300C3597C79D0\r"
                     + "3442561521,ses_so_mat_to.part5.rar,209000000,247,1,l32,4591C2C485D8A60340C4826734067780\r"
                     + "80192378,ses_so_mat_to.part4.rar,209000000,648,1,tl2,86C442D0E2003B2943E5A1F4A7FE0BCB\r"
                     + "1265177889,ses_so_mat_to.part3.rar,209000000,739,1,dt,82281C0D7AD1A56F49A89A4099DFE455\r"
                     + "1211017487,ses_so_mat_to.part2.rar,209000000,299,1,l32,9C826C10B30B1181CAF6749E6159DCAD\r"
                     + "1197692220,ses_so_mat_to.part1.rar,209000000,407,1,tl,8C306CD9993870A3CB432CAD4EA887CF\r";

            var response = Parser.CheckFiles.Parse(text);

            Assert.NotNull(response, "The response can never be null.");
            Assert.NotNull(response.Files, "The list of files should always be set.");
            Assert.AreEqual(7, response.Files.Count, "The list of files should contains exactly seven item.");
        }

        [Test]
        public void CanHandleEmptyResult()
        {
            var text = String.Empty;
            var response = Parser.CheckFiles.Parse(text);

            Assert.NotNull(response, "The response can never be null.");
            Assert.NotNull(response.Files, "The list of files should always be set.");
            Assert.AreEqual(0, response.Files.Count, "The list of files should contains no items.");
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardTextParameter()
        {
            Parser.CheckFiles.Parse(null);
        }
    }
}
