﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MDownloader.Core.Server.Exceptions;
using NUnit.Framework;
using QImpl.Rapidshare.Implementation;
using QImpl.Rapidshare.Implementation.Exceptions;
using QImpl.Rapidshare.Implementation.Responses;

namespace QImpl.Rapidshare.Tests.Implementation.Responses
{
    [TestFixture]
    public class DownloadResponseParserFixture
    {
        [Test]
        public void CanHandleValidResponse()
        {
            var text = @"DL:rs584dt.rapidshare.com,0290C49BC4431AC70EA3B490A8DE85130086E18F1623691953866083070785822190751D634A45C9BEFEC9B60B116EBF305D5616E6D728E1D6A494F74547ABFDCA51B5A56960546A1F57735F8EE4EEDB0290C49BC4431AC70EA3B490A8DE85130086E18F1623691953866083070785823F2A732A85F08C6E98E793B05368EB42,0,2F8323FB2849179D1E40672FCE4AABF9";
            var response = Parser.Download.Parse(text);

            Assert.NotNull(response, "The response can never be null.");
            Assert.AreEqual("rs584dt.rapidshare.com", response.Server, "The server should be properly set.");
            Assert.AreEqual("0290C49BC4431AC70EA3B490A8DE85130086E18F1623691953866083070785822190751D634A45C9BEFEC9B60B116EBF305D5616E6D728E1D6A494F74547ABFDCA51B5A56960546A1F57735F8EE4EEDB0290C49BC4431AC70EA3B490A8DE85130086E18F1623691953866083070785823F2A732A85F08C6E98E793B05368EB42", response.DownloadAuthentication, "The download authentication should be properly set.");
            Assert.AreEqual(0, response.Counter, "The counter should be properly set.");
            Assert.AreEqual("2F8323FB2849179D1E40672FCE4AABF9", response.MD5, "The file HashCode should be properly set.");
        }

        [Test, ExpectedException(typeof(RapidshareException))]
        public void CanHandleResponseWhenInvalidFileIdWasProvided()
        {
            var text = @"ERROR: File ID invalid. (1b3bfd9e)";
            Parser.Download.Parse(text);
        }

        [Test, ExpectedException(typeof(RapidshareException))]
        public void CanHandleResponseWhenDownloadTicketIsNotReady()
        {
            var text = @"ERROR: Download ticket not ready. (fc2abb69)";
            Parser.Download.Parse(text);
        }

        [Test, ExpectedException(typeof(RapidshareException))]
        public void CanHandleResponseWhenDownloadRequiresRapidProAccount()
        {
            var text = @"ERROR: You need RapidPro to download more files from your IP address. (8d5611a9)";
            Parser.Download.Parse(text);
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanHandleResponseWithUnknownData()
        {
            var text = "Abc";
            Parser.Download.Parse(text);
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardTextParameter()
        {
            Parser.Download.Parse(null);
        }
    }
}
