﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking;
using MDownloader.Core.Server.Exceptions;
using NUnit.Framework;
using QImpl.Rapidshare.Implementation;

namespace QImpl.Rapidshare.Tests.Implementation.Responses
{
    [TestFixture]
    public class StreamResponseParserFixture
    {
        [Test]
        public void CanHandleValidResponse()
        {
            var stream = new MemoryStream(Encoding.UTF8.GetBytes("Abc"));
            var callback = new NetworkCallback
            {
                { "Content-Disposition", "Attachment; filename=ses_so_mat_to.part7.rar" },
                { "Content-Type", "application/octet-stream" },
                { "Content-Length", "3" },
            };

            var response = Parser.Stream.Parse(stream, callback);

            Assert.NotNull(response, "The response can never be null.");
            Assert.AreEqual("ses_so_mat_to.part7.rar", response.Attachment, "The attachment should be properly set.");
            Assert.AreEqual(3, response.Size, "The stream size should be correctly set.");
            Assert.NotNull(response.Data, "The stream should be not null.");
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanHandleHtmlResponse()
        {
            var stream = new MemoryStream(Encoding.UTF8.GetBytes("Abc"));
            var callback = new NetworkCallback
            {
                { "Content-Type", "text/xml; charset=utf-8" },
                { "Content-Length", "3" },
            };

            Parser.Stream.Parse(stream, callback);
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardStreamParameter()
        {
            Parser.Stream.Parse(null, new NetworkCallback());
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardCallbackParameter()
        {
            Parser.Stream.Parse(new MemoryStream(), null);
        }
    }
}
