﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using MDownloader.Core.Server;
using MDownloader.Main.Interfaces;

namespace QImpl.Rapidshare.Implementation
{
    internal class Data : IProviderResourceData
    {
        private readonly IBrowserSession session;

        private string link;
        private string server;
        private string downloadAuthentication;

        private Stream stream;
        private string attachment;
        private long size;


        public Data(IBrowserSession session, string link, string server, string downloadAuthentication)
        {
            this.session = session;
            this.link = link;
            this.server = server;
            this.downloadAuthentication = downloadAuthentication;
        }


        public string Attachment
        {
            get { return this.attachment; }
        }

        public long TotalSize
        {
            get { return this.size; }
        }

        public long Offset
        {
            get { return 0L; }
        }

        public long Count
        {
            get { return this.size; }
        }


        public void Initialize()
        {
            var parser = new UrlParser();
            var fileid = parser.GetId(this.link);
            var filename = parser.GetName(this.link);

            var request = Builder.Stream.Get(fileid, filename, this.server, this.downloadAuthentication);
            var stream = session.GetStream(request);
            var response = Parser.Stream.Parse(stream, request.Callback);

            this.attachment = response.Attachment;
            this.size = response.Size;
            this.stream = response.Data;
        }

        public IProviderFeedback GetFeedback()
        {
            return null;
        }


        public Stream OpenStream()
        {
            return this.stream;
        }

        public void Dispose()
        {
            if (this.stream != null)
            {
                this.stream.Close();
                this.stream.Dispose();
                this.stream = null;
            }
        }
    }
}
