﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Networking;

namespace QImpl.Rapidshare.Implementation.Requests
{
    internal class CheckFilesRequestBuilder
    {
        // sub=checkfiles
        //
        // Description:	
        //         Gets status details about a list of given files. (files parameter limited to 3000 bytes. filenames parameter limited to 30000 bytes.)
        // Parameters:
        //         files=comma separated list of file ids
        //         filenames=comma separated list of the respective filename. Example: files=50444381,50444382 filenames=test1.rar,test2.rar
        // Reply fields:	
        //         1:File ID
        //         2:Filename
        //         3:Size (in bytes. If size is 0, this file does not exist.)
        //         4:Server ID
        //         5:Status integer, which can have the following numeric values:
        //             0=File not found
        //             1=File OK
        //             3=Server down
        //             4=File marked as illegal
        //         6:Short host (Use the short host to get the best download mirror: http://rs$serverid$shorthost.rapidshare.com/files/$fileid/$filename)
        //         7:MD5 (hexadecimal)
        // Reply format:	
        //         integer,string,integer,integer,integer,string,string
        
        
        public NetworkRequest Get(IDictionary<string, string> pairs)
        {
            var files = new StringBuilder();
            var filenames = new StringBuilder();

            foreach (var pair in pairs)
            {
                files.Append(pair.Key);
                files.Append(",");

                filenames.Append(pair.Value);
                filenames.Append(",");
            }

            if (files.Length > 0 && filenames.Length > 0)
            {
                files.Remove(files.Length - 1, 1);
                filenames.Remove(filenames.Length - 1, 1);
            }

            return new NetworkRequest(@"https://api.rapidshare.com/cgi-bin/rsapi.cgi", Guid.Empty)
            {
                GetData = new NetworkGetData
                {
                    { "sub", "checkfiles" },
                    { "files", files.ToString() },
                    { "filenames", filenames.ToString() },
                }
            };
        }
    }
}
