﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Networking;

namespace QImpl.Rapidshare.Implementation.Requests
{
    internal class StreamRequestBuilder
    {
        // sub=download
        //
        // Description:
        //         Call this function to download a file from RapidShare as a free user or a RapidPro user. Free downloads are always using http://, while RapidPro
        //         downloads must use https:// (SSL) for security reasons. Since this function is also called by browsers, we append a JavaScript to every error message,
        //         which redirects the user to our homepage. Line 1 is the plain error string as usual. Line 2 is JavaScript code for browser redirection.
        // Parameters:
        //         fileid=File ID
        //         filename=Filename
        //         dlauth=The download authentication string (only used by free users)
        //         try=If 1, it will always return only "DL:$hostname,$dlauth,$countdown"
        //         login=Your login (if using premium downloads)
        //         password=Your password (if using premium downloads)
        //         [start|range]=<x|x-y> You can specifiy where to start downloading. Valid examples are: start=123000 start=0-10000 position=5000-10000 position=50000
        // Reply:	
        //         The reply is dynamic. As always, errors start with "ERROR: <string>". The different scenarios are now explained.
        //         We always assume that no error will be given. If there is an error, you should display it to your user (or parse it and display something else).
        //
        //         Downloading as a free user:
        //             Step 1: Call this function with fileid and filename. You will get "DL:$hostname,$dlauth,$countdown,$md5hex".
        //             Step 2: Wait $countdown seconds.
        //             Step 3: Call the server $hostname with download authentication string $dlauth after $countdown seconds. You instantly get the file.
        //                 WARNING: In our terms of service, we prohibit the automation of downloads for free users. It IS allowed to create tools,
        //                 which download ONE file as a free user from RapidShare. It is NOT allowed to implement queuing mechanisms. Resuming aborted
        //                 downloads is technically disabled.
        //
        //         Downloading as a RapidPro user:
        //             Step 1: Call this function with fileid, filename, login and password.
        //                 If you called the right server and "try" is not given, you will instantly receive the file. If you called the wrong server,
        //                 or try=1 is given, you will get "DL:$hostname,$dlauth,$countdown,$md5hex". See side note #1.
        //             Step 2: Call the same function on the server $hostname. You instantly get the file. Since you are a RapidPro user, $dlauth and
        //                 $countdown are always 0. You MUST use https in this step.
        //
        // Side notes:
        //         #1: The instant download request can only be fulfilled on the server hosting the file. To completely avoid this error, you have to know the server ID,
        //         where the file is hosted. There are scenarios where the server ID is available. For example if you call sub=checkfiles before. If so, you will not get
        //         this error message if you directly call the correct server. If you do not know the server ID, you just have to parse for this error and send your request
        //         again to the correct server.


        public NetworkRequest Get(string fileid, string filename, string server, string downloadAuthentication)
        {
            var result = new NetworkRequest(String.Format(@"http://{0}/cgi-bin/rsapi.cgi", server), Guid.Empty)
            {
                GetData = new NetworkGetData
                {
                    { "sub", "download" },
                    { "fileid", fileid },
                    { "filename", filename },
                    { "dlauth", downloadAuthentication },
                },
                Callback = new NetworkCallback
                {
                }
            };

            return result;
        }
    }
}
