﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using EFramework.Core.Contracts;
using MDownloader.Main.Interfaces.Exceptions;
using QImpl.Core.Exceptions;
using QImpl.Rapidshare.Implementation.Exceptions;

namespace QImpl.Rapidshare.Implementation.Responses
{
    internal class DownloadResponseParser
    {
        private static readonly Regex ErrorRegex = new Regex(@"ERROR:\s*(?<message>[^\000]+)\s*\((?<id>[0-9a-z]{8})\)", RegexOptions.IgnoreCase);

        // DL:rs584dt.rapidshare.com,0290C49BC4431AC70EA3B490A8DE85130086E18F1623691953866083070785822190751D634A45C9BEFEC9B60B116EBF305D5616E6D728E1D6A494F74547ABFDCA51B5A56960546A1F57735F8EE4EEDB0290C49BC4431AC70EA3B490A8DE85130086E18F1623691953866083070785823F2A732A85F08C6E98E793B05368EB42,0,2F8323FB2849179D1E40672FCE4AABF9
        // ERROR: File ID invalid. (1b3bfd9e)
        // ERROR: Download ticket not ready. (fc2abb69)
        // ERROR: You need RapidPro to download more files from your IP address. (8d5611a9)

        public DownloadResponse Parse(string text)
        {
            Ensure.IsNotNull(() => text);

            if (text.StartsWith("DL:", StringComparison.InvariantCultureIgnoreCase))
            {
                var parts = text.Split(new[] { ':' }, 2, StringSplitOptions.None);
                var segments = parts[1].Split(new[] { ',' }, StringSplitOptions.None);

                return new DownloadResponse
                {
                    Server = segments[0],
                    DownloadAuthentication = segments[1],
                    Counter = Int32.Parse(segments[2]),
                    MD5 = segments[3],
                };
            }

            if (text.StartsWith("ERROR:", StringComparison.InvariantCultureIgnoreCase))
            {
                var match = DownloadResponseParser.ErrorRegex.Match(text);
                if (match.Success == true)
                {
                    var id = match.Groups["id"].Value;
                    var message = match.Groups["message"].Value.Trim();

                    throw new RapidshareException(id, message);
                }
            }

            throw UnexpectedDataExceptionFactory.CreateException<DownloadResponseParser>(result => result.Create(text));
        }
    }
}
