﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using EFramework.Networking;
using MDownloader.Main.Interfaces.Exceptions;
using QImpl.Core.Exceptions;

namespace QImpl.Rapidshare.Implementation.Responses
{
    internal class StreamResponseParser
    {
        public StreamResponse Parse(Stream stream, NetworkCallback callback)
        {
            Ensure.IsNotNull(() => stream);
            Ensure.IsNotNull(() => callback);

            if (this.IsBinary(callback) == true)
            {
                var result = new StreamResponse();

                this.ProcessAttachment(result, callback);
                this.ProcessSize(result, callback);
                result.Data = stream;

                if (result.Validate() == false)
                {
                    throw UnexpectedDataExceptionFactory.CreateException<StreamResponseParser>(networkCallback => networkCallback.Create(callback));
                }

                return result;
            }
            else
            {
                try
                {
                    var buffer = new byte[1024];
                    var read = stream.Read(buffer, 0, buffer.Length);

                    Array.Resize(ref buffer, Math.Max(0, read));

                    throw UnexpectedDataExceptionFactory.CreateException<StreamResponseParser>(data => data.Create(buffer), x => x.Create(callback));
                }
                finally
                {
                    stream.Close();
                    stream.Dispose();
                }
            }
        }


        private void ProcessAttachment(StreamResponse response, NetworkCallback callback)
        {
            Ensure.IsNotNull(() => response);
            Ensure.IsNotNull(() => callback);

            if (NetworkParser.HasContentDisposition(callback) == true)
            {
                response.Attachment = NetworkParser.GetContentDispositionAttachment(callback);
            }
        }

        private void ProcessSize(StreamResponse response, NetworkCallback callback)
        {
            Ensure.IsNotNull(() => response);
            Ensure.IsNotNull(() => callback);

            if (NetworkParser.HasContentLength(callback) == true)
            {
                response.Size = NetworkParser.GetContentLength(callback);
            }
        }

        private bool IsBinary(NetworkCallback callback)
        {
            Ensure.IsNotNull(() => callback);

            if (NetworkParser.HasContentType(callback) == true)
            {
                var type = NetworkParser.GetContentType(callback);
                switch (type)
                {
                    case @"application/octet-stream": return true;
                }
            }

            return false;
        }
    }
}
