﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Linq;
using System.Text;
using MDownloader.Core.Server;
using MDownloader.Main.Interfaces;

namespace QImpl.Rapidshare.Implementation
{
    internal class Session : IProviderSession
    {
        private readonly IBrowser browser;


        public Session(IBrowser browser)
        {
            this.browser = browser;
        }


        public ICollection<IProviderLink> Analyze(ICollection<string> links)
        {
            var result = new List<IProviderLink>(links.Count);
            var session = this.browser.OpenSession();
            
            var parser = new UrlParser();
            var builder = new UrlBuilder();

            var pairs = parser.Split(links);
            var request = Builder.CheckFiles.Get(pairs);
            var text = session.GetText(request);
            var response = Parser.CheckFiles.Parse(text);

            foreach (var file in response.Files)
            {
                result.Add(new Link
                {
                    Id = file.Id,
                    Target = builder.BuildLink(file.Id, file.Name),
                    Status = file.Status.ToString(),
                    Name = file.Name,
                    Description = String.Empty,
                    Size = file.Size,
                    SizeExactness = true,
                    IsValid = file.Status != Responses.CheckFilesResponse.Status.NotFound,
                    IsAvailable = file.Status == Responses.CheckFilesResponse.Status.FileOk,
                });
            }

            return result;
        }

        public IProviderResource Download(string link)
        {
            return new Resource(this.browser.OpenSession(), link);
        }


        public void Dispose()
        {
        }
    }
}
