﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MDownloader.Main.Interfaces.Base;

namespace QImpl.Rapidshare.Implementation
{
    // http://rapidshare.com/files/4134535857/8Harry.Potter.And.The.Deathly.Hallows.Part.2.2011.1080p.x264-cass.mkv
    // https://rs427l32.rapidshare.com/#!download|427l35|4134535857|8Harry.Potter.And.The.Deathly.Hallows.Part.2.2011.1080p.x264-cass.mkv|1779148|R~DE688A798103B06FC2FA085A908DBB8A|0|0
    // https://rapidshare.com/#!download|820l35|940536937|Harry_Potter_And_The_Deathly_Hallows_Part_2_2011_DVDRip.rar|837152|R~0|0|0|This file is marked as illegal. (a7bc2095)

    internal class UrlParser : UrlParserBase
    {
        protected override bool IsValidHost(Uri uri)
        {
            if (uri.Host.Equals("rapidshare.com", StringComparison.InvariantCultureIgnoreCase))
                return true;

            if (uri.Host.EndsWith(".rapidshare.com", StringComparison.InvariantCultureIgnoreCase))
                return true;

            return false;
        }

        protected override bool IsValidPath(Uri uri)
        {
            if (uri.Segments.Length == 4)
            {
                if (!uri.Segments[1].Equals("files/", StringComparison.InvariantCultureIgnoreCase))
                    return false;

                return true;
            }
            else if (uri.Segments.Length == 1)
            {
                var parts = uri.OriginalString.Split(new[] { '#' }, 2, StringSplitOptions.RemoveEmptyEntries);
                if (parts.Length != 2)
                    return false;

                var info = parts[1].Split(new[] { '|' });
                if (info.Length < 8)
                    return false;

                if (!info[0].Equals("!download", StringComparison.InvariantCultureIgnoreCase))
                    return false;

                return true;
            }

            return false;
        }

        protected override bool IsValidQuery(Uri uri)
        {
            return true;
        }


        public IDictionary<string, string> Split(IEnumerable<string> links)
        {
            var query = from link in links
                        let uri = new Uri(link, UriKind.Absolute)
                        select new
                        {
                            Id = this.GetId(uri),
                            Name = this.GetName(uri),
                        };

            var groups = from entry in query
                         group entry by entry.Id into byId
                         select new
                         {
                             Id = byId.Key,
                             Name = byId.First().Name,
                         };

            return groups.ToDictionary(x => x.Id, x => x.Name);
        }

        public string GetId(string link)
        {
            return this.GetId(new Uri(link, UriKind.Absolute));
        }

        public string GetName(string link)
        {
            return this.GetName(new Uri(link, UriKind.Absolute));
        }

        private string GetId(Uri uri)
        {
            if (uri.Segments.Length == 4)
            {
                return uri.Segments[2].Replace("/", String.Empty);
            }
            else
            {
                return uri.OriginalString
                    .Split(new[] { '#' }, 2, StringSplitOptions.RemoveEmptyEntries)[1]
                    .Split(new[] { '|' })[2];
            }
        }

        private string GetName(Uri uri)
        {
            if (uri.Segments.Length == 4)
            {
                return uri.Segments[3].Replace("/", String.Empty);
            }
            else
            {
                return uri.OriginalString
                    .Split(new[] { '#' }, 2, StringSplitOptions.RemoveEmptyEntries)[1]
                    .Split(new[] { '|' })[3];
            }
        }
    }
}
