﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking.Parser;
using NUnit.Framework;
using QImpl.Recaptcha.Implementation;

namespace QImpl.Recaptcha.Tests.Implementation.Responses
{
    [TestFixture]
    public class ChallengeResponseParserFixture
    {
        [Test]
        public void CanParseTypicalResponse()
        {
            var text = this.GetText("Challenge.Typical.txt");
            var response = Parser.Challenge.Parse(text);

            Assert.NotNull(response, "The response can never be null.");
            Assert.AreEqual(@"6LfRJwkAAAAAAGmA3mAiAcAsRsWvfkBijaZWEvkD", response.Site, "The site should be properly set.");
            Assert.AreEqual(@"03AHJ_Vus3JeAgUxfJzqrioOMRIIadEOURx1IWCbf23OnmvVtqQixqU-gw9A-tj7LPQltveHmwtE58ZzUFZ1GDoOTM6Hygtdn9Ge0THfKQUa3jbyUmF-J4p9t3x0hYXh4IBUKpXFuaknpeibU_FauFBDSSVqgKF-fAgw", response.Challenge, "The challenge should be properly set.");
            Assert.IsFalse(response.IsIncorect, "The IsIncorrect flag should be properly set.");
            Assert.IsNullOrEmpty(response.ProgrammingError, "The programming error should not be set.");
            Assert.IsNullOrEmpty(response.ErrorMessage, "The error message should not be set.");
            Assert.AreEqual(@"http://www.google.com/recaptcha/api/", response.Server, "The server should be properly set.");
            Assert.AreEqual(1800, response.Timeout, "The timeout should be properly set.");
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullTextParameter()
        {
            Parser.Challenge.Parse(default(string));
        }


        private string GetText(string resourceName)
        {
            var path = String.Format("{0}.Resources.{1}", this.GetType().Namespace, resourceName);

            using (var stream = this.GetType().Assembly.GetManifestResourceStream(path))
            using (var reader = new StreamReader(stream))
            {
                return reader.ReadToEnd();
            }
        }
    }
}
