﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Networking;
using MDownloader.Core.Server.Exceptions;
using NUnit.Framework;
using QImpl.Recaptcha.Implementation;

namespace QImpl.Recaptcha.Tests.Implementation.Responses
{
    [TestFixture]
    public class ImageResponseParserFixture
    {
        [Test]
        public void CanParseTypicalResponse()
        {
            var data = new byte[] { 0x00, 0x01 };
            var callback = new NetworkCallback
            {
                { "Content-Type", "image/jpeg" },
                { "Content-Length", "2" },
            };

            using (var stream = new MemoryStream(data))
            {
                var response = Parser.Image.Parse(stream, callback);

                Assert.NotNull(response, "The response can never be null.");
                Assert.NotNull(response.Data, "The image data should not be null.");
                CollectionAssert.AreEqual(data, response.Data, "The image data should match.");
            }
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanDetectTooLargeData()
        {
            var data = new byte[] { 0x00, 0x01 };
            var callback = new NetworkCallback
            {
                { "Content-Type", "image/jpeg" },
                { "Content-Length", "1048576" },
            };

            using (var stream = new MemoryStream(data))
            {
                Parser.Image.Parse(stream, callback);
            }
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanDetectInconsistenceInSize()
        {
            var data = new byte[] { 0x00, 0x01 };
            var callback = new NetworkCallback
            {
                { "Content-Type", "image/jpeg" },
                { "Content-Length", "1024" },
            };

            using (var stream = new MemoryStream(data))
            {
                Parser.Image.Parse(stream, callback);
            }
        }

        [Test, ExpectedException(typeof(UnexpectedDataException))]
        public void CanDetectUnknownImageType()
        {
            var data = new byte[] { 0x00, 0x01 };
            var callback = new NetworkCallback
            {
                { "Content-Type", "image/xxx" },
                { "Content-Length", "2" },
            };

            using (var stream = new MemoryStream(data))
            {
                var response = Parser.Image.Parse(stream, callback);
            }
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullStreamParameter()
        {
            var callback = new NetworkCallback
            {
                { "Content-Type", "image/xxx" },
                { "Content-Length", "2" },
            };

            Parser.Image.Parse(default(Stream), callback);
        }

        [Test, ExpectedException(typeof(ArgumentException))]
        public void CanGuardNullCallbackParameter()
        {
            var data = new byte[] { 0x00, 0x01 };
            using (var stream = new MemoryStream(data))
            {
                Parser.Image.Parse(stream, default(NetworkCallback));
            }
        }
    }
}
