﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using EFramework.Core.Contracts;
using EFramework.Networking.Parser;
using QImpl.Core.Exceptions;

namespace QImpl.Recaptcha.Implementation.Responses
{
    internal class ChallengeResponseParser
    {
        private readonly Regex siteRegex = new Regex(@"site\s*:\s*'(?<site>[^']+)'", RegexOptions.IgnoreCase);
        private readonly Regex challengeRegex = new Regex(@"challenge\s*:\s*'(?<challenge>[^']+)'", RegexOptions.IgnoreCase);
        private readonly Regex isIncorectRegex = new Regex(@"is_incorrect\s*:\s*(?<is_incorect>(false|true)+)", RegexOptions.IgnoreCase);
        private readonly Regex programmingErrorRegex = new Regex(@"programming_error\s*:\s*'(?<programming_error>[^']*)'", RegexOptions.IgnoreCase);
        private readonly Regex errorMessageRegex = new Regex(@"error_message\s*:\s*'(?<error_message>[^']*)'", RegexOptions.IgnoreCase);
        private readonly Regex serverRegex = new Regex(@"server\s*:\s*'(?<server>[^']+)'", RegexOptions.IgnoreCase);
        private readonly Regex timoutRegex = new Regex(@"timeout\s*:\s*(?<timeout>[0-9]+)", RegexOptions.IgnoreCase);


        public ChallengeResponse Parse(string text)
        {
            Ensure.IsNotNull(() => text);

            var result = new ChallengeResponse();

            this.Process(result, text);

            if (result.Validate() == false)
            {
                throw UnexpectedDataExceptionFactory.CreateException<ChallengeResponseParser>(data => data.Create(text));
            }

            return result;
        }

        
        private void Process(ChallengeResponse result, string text)
        {
            Ensure.IsNotNull(() => text);

            var siteMatch = this.siteRegex.Match(text);
            if (siteMatch.Success == true)
            {
                result.Site = siteMatch.Groups["site"].Value;
            }

            var challengeMatch = this.challengeRegex.Match(text);
            if (challengeMatch.Success == true)
            {
                result.Challenge = challengeMatch.Groups["challenge"].Value;
            }

            var isIncorrectMatch = this.isIncorectRegex.Match(text);
            if (isIncorrectMatch.Success == true)
            {
                result.IsIncorect = isIncorrectMatch.Groups["is_incorrect"].Value.Equals("true");
            }

            var programmingErrorMatch = this.programmingErrorRegex.Match(text);
            if (programmingErrorMatch.Success == true)
            {
                result.ProgrammingError = programmingErrorMatch.Groups["programming_error"].Value;
            }

            var errorMessageMatch = this.errorMessageRegex.Match(text);
            if (errorMessageMatch.Success == true)
            {
                result.ErrorMessage = errorMessageMatch.Groups["error_message"].Value;
            }

            var serverMatch = this.serverRegex.Match(text);
            if (serverMatch.Success == true)
            {
                result.Server = serverMatch.Groups["server"].Value;
            }

            var timoutMatch = this.timoutRegex.Match(text);
            if (timoutMatch.Success == true)
            {
                result.Timeout = Int32.Parse(timoutMatch.Groups["timeout"].Value);
            }
        }
    }
}
