﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using EFramework.Networking;
using QImpl.Core.Exceptions;

namespace QImpl.Recaptcha.Implementation.Responses
{
    internal class ImageResponseParser
    {
        public ImageResponse Parse(Stream stream, NetworkCallback callback)
        {
            Ensure.IsNotNull(() => stream);
            Ensure.IsNotNull(() => callback);

            var result = new ImageResponse();

            this.Process(result, stream, callback);

            if (result.Validate() == false)
            {
                throw UnexpectedDataExceptionFactory.CreateException<ChallengeResponseParser>(networkCallback => networkCallback.Create(callback));
            }

            return result;
        }


        private void Process(ImageResponse result, Stream stream, NetworkCallback callback)
        {
            Ensure.IsNotNull(() => result);
            Ensure.IsNotNull(() => stream);
            Ensure.IsNotNull(() => callback);

            if (NetworkParser.HasContentLength(callback) == false || NetworkParser.GetContentLength(callback) > 100000)
            {
                return;
            }

            if (NetworkParser.HasContentType(callback) == false || NetworkParser.GetContentType(callback) != "image/jpeg")
            {
                return;
            }

            var buffer = new byte[NetworkParser.GetContentLength(callback)];
            var position = 0;
            var left = buffer.Length;

            while (left > 0)
            {
                var read = stream.Read(buffer, position, left);
                if (read <= 0)
                    break;

                position += read;
                left -= read;
            }

            if (left == 0)
            {
                result.Data = buffer;
            }
        }
    }
}
