﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using EFramework.Core.Contracts;
using MDownloader.Core.Server;
using MDownloader.Main.Interfaces;

namespace QImpl.Recaptcha.Implementation
{
    internal class Session : IRecaptchaSession
    {
        private readonly IBrowserSession browserSession;
        private readonly string token;

        private string challenge;
        

        public Session(IBrowserSession browserSession, string token)
        {
            Ensure.IsNotNull(() => browserSession);
            Ensure.IsNotNullNorEmpty(() => token);

            this.browserSession = browserSession;
            this.token = token;

            this.Initialize();
        }


        public string GetChallenge()
        {
            return this.challenge;
        }

        public byte[] GetImage()
        {
            var request = Builder.Image.Get(this.challenge);
            var stream = this.browserSession.GetStream(request);
            var response = Parser.Image.Parse(stream, request.Callback);

            return response.Data;
        }

        public byte[] GetAudio()
        {
            return null;
        }

        public void Refresh()
        {
        }


        private void Initialize()
        {
            var request = Builder.Challenge.Get(this.token);
            var text = this.browserSession.GetText(request);
            var response = Parser.Challenge.Parse(text);

            this.challenge = response.Challenge;
        }
    }
}
